@@ -4,6 +4,8 @@
 #include <functional>
 #include <cassert>
 #include <sigc++/trackable.h>
+#include <sigc++/signal.h>
+#include <mutex>
 
 #include <string>
 #include <set>
@@ -88,6 +90,10 @@ public:
 	virtual std::ostream& getErrorStream() const = 0;
 	virtual std::ostream& getWarningStream() const = 0;
 
+    // Provides a single mutex object which should be locked by client code
+    // before writing to the any of the above streams.
+    virtual std::mutex& getStreamLock() const = 0;
+
 	/**
 	 * Sets up the paths and stores them into the registry.
 	 */
@@ -214,7 +220,7 @@ public:
 	 *
 	 * This method does not cause the RegisterableModule to be initialised.
 	 */
-	virtual void registerModule(RegisterableModulePtr module) = 0;
+	virtual void registerModule(const RegisterableModulePtr& module) = 0;
 
 	/**
 	 * Initialise all of the modules previously registered with
@@ -259,6 +265,15 @@ public:
 	 */
 	virtual const ApplicationContext& getApplicationContext() const = 0;
 
+    /**
+     * Invoked when all modules have been initialised.
+     */
+    virtual sigc::signal<void> signal_allModulesInitialised() const = 0;
+
+    /**
+    * Invoked when all modules have been shut down (i.e. after shutdownModule()).
+    */
+    virtual sigc::signal<void> signal_allModulesUninitialised() const = 0;
 };
 
 namespace module {
@@ -270,13 +285,6 @@ namespace module {
 	 * \ingroup module
 	 */
 
-	/**
-	 * Global ModuleRegistry accessor function exported by the main binary.
-	 * Note: Don't use this from within modules (entity, shaders, etc.),
-	 * this only works from within the DarkRadiant core binary (Win32).
-	 */
-	IModuleRegistry& getRegistry();
-
 	/** greebo: This is a container holding a reference to the registry.
 	 *          The getRegistry() symbol above is not exported to the
 	 *          modules in Win32 compiles. That's why this structure
