@@ -97,11 +97,6 @@ public:
 	 * Removes an entire path from the menus.
  	 */
 	virtual void remove(const std::string& path) = 0;
-
-	/** Recursively iterates over the menu items and updates
-	 *  the accelerator strings by requesting them from the EventManager.
-	 */
-	virtual void updateAccelerators() = 0;
 };
 
 class IToolbarManager
@@ -164,8 +159,10 @@ public:
 	 *        if no icon is desired.
 	 * @pos: the position to insert. Use POS_FRONT or POS_BACK to put the element
 	 *       at the front or back of the status bar container.
+	 * @description: a description shown when the mouse pointer hovers of this item.
  	 */
-	virtual void addTextElement(const std::string& name, const std::string& icon, int pos) = 0;
+	virtual void addTextElement(const std::string& name, const std::string& icon, int pos,
+								const std::string& description) = 0;
 
 	/**
 	 * Updates the content of the named text element. The name must refer to
