@@ -13,14 +13,9 @@ wxString TreeModel::Column::getWxType() const
 	if (types.empty())
 	{
 		types[String] = "string";
-#ifdef __WXGTK__
-		// For wxGTK we store numbers as strings
+		// we store numbers as strings
 		types[Integer] = "string";
 		types[Double] = "string";
-#else
-		types[Integer] = "long";
-		types[Double] = "double";
-#endif
 		types[Boolean] = "bool";
 		types[Icon] = "icon";
 		types[IconText] = "wxDataViewIconText";
@@ -50,6 +45,9 @@ public:
 	typedef std::vector<wxDataViewItemAttr> Attributes;
 	Attributes attributes;
 
+	typedef std::vector<bool> EnabledFlags; // Each value can be flagged as enabled/disabled
+	EnabledFlags enabledFlags;
+
 	// Public constructor, does not accept NULL pointers
 	Node(Node* parent_) :
 		parent(parent_),
@@ -88,7 +86,10 @@ public:
 	}
 };
 
+// -------------------------------------------------------------------------------
+
 wxDEFINE_EVENT(EV_TREEMODEL_POPULATION_FINISHED, TreeModel::PopulationFinishedEvent);
+wxDEFINE_EVENT(EV_TREEMODEL_POPULATION_PROGRESS, TreeModel::PopulationProgressEvent);
 
 TreeModel::PopulationFinishedEvent::PopulationFinishedEvent(int id) : 
 	wxEvent(id, EV_TREEMODEL_POPULATION_FINISHED),
@@ -122,6 +123,38 @@ void TreeModel::PopulationFinishedEvent::SetTreeModel(TreeModel::Ptr store)
 	_treeModel = store;
 }
 
+TreeModel::PopulationProgressEvent::PopulationProgressEvent(int id) :
+    wxEvent(id, EV_TREEMODEL_POPULATION_PROGRESS)
+{}
+
+TreeModel::PopulationProgressEvent::PopulationProgressEvent(const wxString& message, int id) :
+    wxEvent(id, EV_TREEMODEL_POPULATION_PROGRESS),
+    _message(message)
+{}
+
+
+TreeModel::PopulationProgressEvent::PopulationProgressEvent(const TreeModel::PopulationProgressEvent& event) :
+    wxEvent(event),
+    _message(event._message)
+{}
+
+wxEvent* TreeModel::PopulationProgressEvent::Clone() const
+{
+    return new PopulationProgressEvent(*this);
+}
+
+const wxString& TreeModel::PopulationProgressEvent::GetMessage() const
+{
+    return _message;
+}
+
+void TreeModel::PopulationProgressEvent::SetMessage(const wxString& message)
+{
+    _message = message;
+}
+
+// -------------------------------------------------------------------------------
+
 TreeModel::TreeModel(const ColumnRecord& columns, bool isListModel) :
 	_columns(columns),
 	_rootNode(Node::createRoot()),
@@ -891,4 +924,34 @@ wxDataViewItem TreeModel::FindPrevString(const wxString& needle,
 	return functor.getMatch();
 }
 
+bool TreeModel::IsEnabled(const wxDataViewItem& item, unsigned int col) const
+{
+	Node* owningNode = item.IsOk() ? static_cast<Node*>(item.GetID()) : _rootNode.get();
+
+	if (col < owningNode->enabledFlags.size())
+	{
+		return owningNode->enabledFlags[col];
+	}
+	
+	// Column values without flags render as enabled by default
+	return true;
+}
+
+void TreeModel::SetEnabled(const wxDataViewItem& item, unsigned int col, bool enabled)
+{
+	if (!item.IsOk())
+	{
+		return;
+	}
+
+	Node* owningNode = static_cast<Node*>(item.GetID());
+
+	if (owningNode->enabledFlags.size() < col + 1)
+	{
+		owningNode->enabledFlags.resize(col + 1, true); // fill with true by default
+	}
+
+	owningNode->enabledFlags[col] = enabled;
+}
+
 } // namespace
