@@ -11,7 +11,10 @@
 #include "modelskin.h"
 #include "entitylib.h"
 #include "scene/Node.h"
+#include "scene/BasicRootNode.h"
 #include "wxutil/dialog/MessageBox.h"
+#include "string/convert.h"
+#include <sstream>
 
 #include "iuimanager.h"
 
@@ -32,11 +35,8 @@ namespace
 ModelPreview::ModelPreview(wxWindow* parent) :
     RenderPreview(parent, false),
 	_lastModel(""),
-	_defaultCamDistanceFactor(6.0f)
-{ 
-	_defaultTransform = Matrix4::getRotationAboutZDegrees(-45);
-	_defaultTransform = _defaultTransform.getMultipliedBy(Matrix4::getRotation(Vector3(1,1,0), 45));
-}
+	_defaultCamDistanceFactor(2.8f)
+{}
 
 // Set the model, this also resets the camera
 void ModelPreview::setModel(const std::string& model)
@@ -81,11 +81,14 @@ void ModelPreview::setModel(const std::string& model)
 			// Reset preview time
 			stopPlayback();
 
-			// Reset the rotation to the default one
-			_rotation = _defaultTransform;
+			// Reset the model rotation
+            resetModelRotation();
 
-			// Calculate camera distance so model is appropriately zoomed
-			_camDist = -(_modelNode->localAABB().getRadius() * _defaultCamDistanceFactor);
+            // Reset the default view, facing down to the model from diagonally above the bounding box
+            double distance = _modelNode->localAABB().getRadius() * _defaultCamDistanceFactor;
+
+            setViewOrigin(Vector3(1, 1, 1) * distance);
+            setViewAngles(Vector3(34, 135, 0));
 		}
 
 		_lastModel = model;
@@ -116,11 +119,6 @@ void ModelPreview::setSkin(const std::string& skin) {
 	queueDraw();
 }
 
-void ModelPreview::setDefaultOrientation(const Matrix4& transform)
-{
-	_defaultTransform = transform;
-}
-
 void ModelPreview::setDefaultCamDistanceFactor(float factor)
 {
 	_defaultCamDistanceFactor = factor;
@@ -132,15 +130,28 @@ void ModelPreview::setupSceneGraph()
 
     try
     {
+        _rootNode.reset(new scene::BasicRootNode);
+
         _entity = GlobalEntityCreator().createEntity(
             GlobalEntityClassManager().findClass(FUNC_STATIC_CLASS));
 
+        _rootNode->addChildNode(_entity);
+
         _entity->enable(scene::Node::eHidden);
 
         // This entity is acting as our root node in the scene
-        getScene()->setRoot(_entity);
+        getScene()->setRoot(_rootNode);
+
+        // Set up the light
+        _light = GlobalEntityCreator().createEntity(
+            GlobalEntityClassManager().findClass("light"));
+
+        Node_getEntity(_light)->setKeyValue("light_radius", "600 600 600");
+        Node_getEntity(_light)->setKeyValue("origin", "0 0 300");
+
+        _rootNode->addChildNode(_light);
     }
-    catch (std::runtime_error& ex)
+    catch (std::runtime_error&)
     {
         wxutil::Messagebox::ShowError(
             (boost::format(_("Unable to setup the preview,\n"
@@ -160,7 +171,46 @@ AABB ModelPreview::getSceneBounds()
 
 bool ModelPreview::onPreRender()
 {
-	return _modelNode != NULL;
+    if (_light)
+    {
+        Vector3 lightOrigin = _viewOrigin + Vector3(0, 0, 20);
+
+        // Position the light just above the camera
+        Node_getEntity(_light)->setKeyValue("origin", string::to_string(lightOrigin));
+
+        // Let the light encompass the object
+        float radius = (getSceneBounds().getOrigin() - lightOrigin).getLength() * 2.0f;
+        radius = std::max(radius, 200.f);
+
+        std::ostringstream value;
+        value << radius << ' ' << radius << ' ' << radius;
+        
+        Node_getEntity(_light)->setKeyValue("light_radius", value.str());
+
+        Node_getEntity(_light)->setKeyValue("_color", "0.6 0.6 0.6");
+    }
+
+	return _modelNode != nullptr;
+}
+
+void ModelPreview::onModelRotationChanged()
+{
+    if (_entity)
+    {
+        // Update the model rotation on the entity
+        std::ostringstream value;
+        value << _modelRotation.xx() << ' '
+              << _modelRotation.xy() << ' '
+              << _modelRotation.xz() << ' '
+              << _modelRotation.yx() << ' '
+              << _modelRotation.yy() << ' '
+              << _modelRotation.yz() << ' '
+              << _modelRotation.zx() << ' '
+              << _modelRotation.zy() << ' '
+              << _modelRotation.zz();
+
+        Node_getEntity(_entity)->setKeyValue("rotation", value.str());
+    }
 }
 
 RenderStateFlags ModelPreview::getRenderFlagsFill()
