@@ -9,6 +9,7 @@
 #include "i18n.h"
 
 #include "scene/Node.h"
+#include "scene/BasicRootNode.h"
 #include "entitylib.h"
 
 #include "string/string.h"
@@ -127,26 +128,30 @@ void ParticlePreview::setParticle(const std::string& name)
         // Reset preview time
         stopPlayback();
 
-        // Reset the rotation to the default one
-        _rotation = Matrix4::getRotation(Vector3(0,-1,0), Vector3(0,-0.3f,1));
-        _rotation.multiplyBy(Matrix4::getRotation(Vector3(0,1,0), Vector3(1,-1,0)));
-
         // Call update(0) once to enable the bounds calculation
-        _particleNode->getParticle()->update(_rotation);
+        _particleNode->getParticle()->update(_modelView);
+
+        // Reset the model rotation
+        resetModelRotation();
 
         // Use particle AABB to adjust camera distance
         const AABB& particleBounds = _particleNode->getParticle()->getBounds();
 
         if (particleBounds.isValid())
         {
-            _camDist = -2.0f * static_cast<float>(particleBounds.getRadius());
+            // Reset the default view, facing down to the model from diagonally above the bounding box
+            double distance = particleBounds.getRadius() * 2.0f;
+
+            setViewOrigin(Vector3(1, 1, 1) * distance);
         }
         else
         {
             // Bounds not valid, fall back to default
-            _camDist = -40.0f;
+            setViewOrigin(Vector3(1, 1, 1) * 40.0f);
         }
 
+        setViewAngles(Vector3(34, 135, 0));
+
         _lastParticle = nameClean;
 
         // Start playback when switching particles
@@ -163,15 +168,19 @@ void ParticlePreview::setupSceneGraph()
 
 	try
 	{
+        _rootNode.reset(new scene::BasicRootNode);
+        
 		_entity = GlobalEntityCreator().createEntity(
 			GlobalEntityClassManager().findClass(FUNC_EMITTER_CLASS));
 
+        _rootNode->addChildNode(_entity);
+
 		_entity->enable(scene::Node::eHidden);
 
 		// This entity is acting as our root node in the scene
-		getScene()->setRoot(_entity);
+        getScene()->setRoot(_rootNode);
 	}
-	catch (std::runtime_error& ex)
+	catch (std::runtime_error&)
 	{
 		wxutil::Messagebox::ShowError(
 			(boost::format(_("Unable to setup the preview,\n"
@@ -243,6 +252,26 @@ void ParticlePreview::onPostRender()
     }
 }
 
+void ParticlePreview::onModelRotationChanged()
+{
+    if (_entity)
+    {
+        // Update the model rotation on the entity
+        std::ostringstream value;
+        value << _modelRotation.xx() << ' '
+            << _modelRotation.xy() << ' '
+            << _modelRotation.xz() << ' '
+            << _modelRotation.yx() << ' '
+            << _modelRotation.yy() << ' '
+            << _modelRotation.yz() << ' '
+            << _modelRotation.zx() << ' '
+            << _modelRotation.zy() << ' '
+            << _modelRotation.zz();
+
+        Node_getEntity(_entity)->setKeyValue("rotation", value.str());
+    }
+}
+
 void ParticlePreview::drawAxes()
 {
     glDisable(GL_TEXTURE_2D);
