@@ -3,6 +3,7 @@
 #include "ifilter.h"
 #include "i18n.h"
 #include "igl.h"
+#include "icamera.h"
 #include "iscenegraphfactory.h"
 #include "irendersystemfactory.h"
 #include "iuimanager.h"
@@ -34,8 +35,15 @@ namespace
 RenderPreview::RenderPreview(wxWindow* parent, bool enableAnimation) :
     _mainPanel(loadNamedPanel(parent, "RenderPreviewPanel")),
 	_glWidget(new wxutil::GLWidget(_mainPanel, std::bind(&RenderPreview::drawPreview, this), "RenderPreview")),
+    _initialised(false),
     _renderSystem(GlobalRenderSystemFactory().createRenderSystem()),
     _sceneWalker(_renderer, _volumeTest),
+    _viewOrigin(0, 0, 0),
+    _viewAngles(0, 0, 0),
+    _modelView(Matrix4::getIdentity()),
+    _modelRotation(Matrix4::getIdentity()),
+    _lastX(0),
+    _lastY(0),
     _renderingInProgress(false),
     _timer(this),
     _previewWidth(0),
@@ -49,40 +57,62 @@ RenderPreview::RenderPreview(wxWindow* parent, bool enableAnimation) :
 
 	_glWidget->Connect(wxEVT_SIZE, wxSizeEventHandler(RenderPreview::onSizeAllocate), NULL, this);
 	_glWidget->Connect(wxEVT_MOUSEWHEEL, wxMouseEventHandler(RenderPreview::onGLScroll), NULL, this);
-	_glWidget->Connect(wxEVT_MOTION, wxMouseEventHandler(RenderPreview::onGLMotion), NULL, this);
+    _glWidget->Connect(wxEVT_MOTION, wxMouseEventHandler(RenderPreview::onGLMotion), NULL, this);
 	_glWidget->Connect(wxEVT_LEFT_DOWN, wxMouseEventHandler(RenderPreview::onGLMouseClick), NULL, this);
     _glWidget->Connect(wxEVT_LEFT_DCLICK, wxMouseEventHandler(RenderPreview::onGLMouseClick), NULL, this);
+    _glWidget->Connect(wxEVT_RIGHT_DOWN, wxMouseEventHandler(RenderPreview::onGLMouseClick), NULL, this);
+    _glWidget->Connect(wxEVT_RIGHT_DCLICK, wxMouseEventHandler(RenderPreview::onGLMouseClick), NULL, this);
+    _glWidget->Connect(wxEVT_KEY_DOWN, wxKeyEventHandler(RenderPreview::onGLKeyPress), NULL, this);
 
-	wxToolBar* toolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewAnimToolbar");
+    setupToolbars(enableAnimation);
 
-	_toolbarSizer = toolbar->GetContainingSizer();
+    // Clicks are eaten when the FreezePointer is active, request to receive them
+    _freezePointer.connectMouseEvents(
+        std::bind(&RenderPreview::onGLMouseClick, this, std::placeholders::_1),
+        std::bind(&RenderPreview::onGLMouseRelease, this, std::placeholders::_1));
+}
 
-	// Set up the toolbar
+void RenderPreview::setupToolbars(bool enableAnimation)
+{
+    wxToolBar* toolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewAnimToolbar");
+
+    _toolbarSizer = toolbar->GetContainingSizer();
+
+    // Set up the toolbar
     if (enableAnimation)
     {
         connectToolbarSignals();
     }
     else
     {
-		toolbar->Hide();
+        toolbar->Hide();
     }
 
-	// Connect filters menu to toolbar
-	wxToolBar* filterToolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewFilterToolbar");
+    // Connect filters menu to toolbar
+    wxToolBar* filterToolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewFilterToolbar");
 
-	wxMenu* filterSubmenu = _filtersMenu->getMenuWidget();
+    wxMenu* filterSubmenu = _filtersMenu->getMenuWidget();
 
-	wxToolBarToolBase* filterTool = filterToolbar->AddTool(wxID_ANY, _("Filters"), 
-		wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + "iconFilter16.png"), 
-		_("Filters"), wxITEM_DROPDOWN);
-	filterToolbar->SetDropdownMenu(filterTool->GetId(), filterSubmenu);
+    wxToolBarToolBase* filterTool = filterToolbar->AddTool(wxID_ANY, _("Filters"),
+                                                           wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + "iconFilter16.png"),
+                                                           _("Filters"), wxITEM_DROPDOWN);
+    filterToolbar->SetDropdownMenu(filterTool->GetId(), filterSubmenu);
 
-	filterToolbar->Realize();
+    filterToolbar->Realize();
 
     // Get notified of filter changes
     GlobalFilterSystem().filtersChangedSignal().connect(
         sigc::mem_fun(this, &RenderPreview::filtersChanged)
-    );
+        );
+
+    wxToolBar* renderToolbar = findNamedObject<wxToolBar>(_mainPanel, "RenderPreviewRenderModeToolbar");
+
+    renderToolbar->Connect(getToolBarToolByLabel(renderToolbar, "texturedModeButton")->GetId(),
+                           wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onRenderModeChanged), NULL, this);
+    renderToolbar->Connect(getToolBarToolByLabel(renderToolbar, "lightingModeButton")->GetId(),
+                           wxEVT_TOOL, wxCommandEventHandler(RenderPreview::onRenderModeChanged), NULL, this);
+
+    updateActiveRenderModeButton();
 }
 
 void RenderPreview::connectToolbarSignals()
@@ -107,6 +137,20 @@ RenderPreview::~RenderPreview()
     _timer.Stop();
 }
 
+void RenderPreview::updateActiveRenderModeButton()
+{
+    wxToolBar* toolbar = static_cast<wxToolBar*>(_mainPanel->FindWindow("RenderPreviewRenderModeToolbar"));
+
+    if (getLightingModeEnabled())
+    {
+        toolbar->ToggleTool(getToolBarToolByLabel(toolbar, "lightingModeButton")->GetId(), true);
+    }
+    else
+    {
+        toolbar->ToggleTool(getToolBarToolByLabel(toolbar, "texturedModeButton")->GetId(), true);
+    }
+}
+
 void RenderPreview::filtersChanged()
 {
     if (!getScene()->root()) return;
@@ -122,7 +166,10 @@ void RenderPreview::addToolbar(wxToolBar* toolbar)
 
 void RenderPreview::queueDraw()
 {
-	_glWidget->Refresh();
+    if (!_renderingInProgress)
+    {
+        _glWidget->Refresh();
+    }
 }
 
 void RenderPreview::setSize(int width, int height)
@@ -132,42 +179,7 @@ void RenderPreview::setSize(int width, int height)
 
 void RenderPreview::initialisePreview()
 {
-#if 0
-#ifdef WIN32
-    // greebo: Unfortunate hack to fix the grey GL renderviews in the EntityChooser
-    // and other windows that are hidden instead of destroyed when closed.
-    Gtk::Container* container = get_parent();
-    bool wasShown = get_visible();
-
-    if (container != NULL)
-    {
-        if (wasShown)
-        {
-            hide();
-        }
-
-        container->remove(*this);
-        container->add(*this);
-
-        if (wasShown)
-        {
-            show();
-        }
-    }
-
-#endif
-#endif
-	// Grab the GL widget with sentry object
-	wxPaintDC dc(_glWidget);
-	_glWidget->SetCurrent(GlobalOpenGL().getwxGLContext());
-	
-    // Set up the camera
-    glMatrixMode(GL_PROJECTION);
-    glLoadIdentity();
-    gluPerspective(PREVIEW_FOV, 1, 0.1, 10000);
-
-    glMatrixMode(GL_MODELVIEW);
-    glLoadIdentity();
+    _initialised = true;
 
     // Set up the lights
     glEnable(GL_LIGHTING);
@@ -186,11 +198,44 @@ void RenderPreview::initialisePreview()
     glLightfv(GL_LIGHT1, GL_DIFFUSE, l1Dif);
     glLightfv(GL_LIGHT1, GL_POSITION, l1Pos);
 
-    if (_renderSystem->shaderProgramsAvailable())
+    if (GlobalOpenGL().shaderProgramsAvailable())
     {
-        _renderSystem->setShaderProgram(
-            RenderSystem::SHADER_PROGRAM_INTERACTION
-        );
+        setLightingModeEnabled(false);
+    }
+
+    updateModelViewMatrix();
+}
+
+void RenderPreview::setViewOrigin(const Vector3& origin)
+{
+    _viewOrigin = origin;
+
+    updateModelViewMatrix();
+}
+
+void RenderPreview::setViewAngles(const Vector3& angles)
+{
+    _viewAngles = angles;
+
+    updateModelViewMatrix();
+}
+
+bool RenderPreview::getLightingModeEnabled()
+{
+    return _renderSystem->getCurrentShaderProgram() == RenderSystem::SHADER_PROGRAM_INTERACTION;
+}
+
+void RenderPreview::setLightingModeEnabled(bool enabled)
+{
+    if (enabled && !getLightingModeEnabled())
+    {
+        _renderSystem->setShaderProgram(RenderSystem::SHADER_PROGRAM_INTERACTION);
+        queueDraw();
+    }
+    else if (!enabled && getLightingModeEnabled())
+    {
+        _renderSystem->setShaderProgram(RenderSystem::SHADER_PROGRAM_NONE);
+        queueDraw();
     }
 }
 
@@ -237,25 +282,56 @@ Matrix4 RenderPreview::getProjectionMatrix(float near_z, float far_z, float fiel
     );
 }
 
-Matrix4 RenderPreview::getModelViewMatrix()
+const Matrix4& RenderPreview::getModelViewMatrix()
 {
-    // Premultiply with the translations
-    Matrix4 modelview;
+    return _modelView;
+}
 
-    {
-        glMatrixMode(GL_MODELVIEW);
-        glLoadIdentity();
+Matrix4 RenderPreview::calculateModelViewMatrix()
+{
+    static const Matrix4 RADIANT2OPENGL = Matrix4::byColumns(
+        0, -1, 0, 0,
+        0,  0, 1, 0,
+       -1,  0, 0, 0,
+        0,  0, 0, 1
+    );
 
-        glTranslatef(0, 0, _camDist); // camera translation
-        glMultMatrixd(_rotation); // post multiply with rotations
+    // greebo: This is modeled after the camera's modelview code
+    // Some notes: first the matrix stack is built in the reverse order,
+    // then a final call to inverse() is made to bring them in the correct order
 
-        // Load the matrix from openGL
-        glGetDoublev(GL_MODELVIEW_MATRIX, modelview);
-    }
+    // In the final form the matrix stack does this:
+    // 0. We start with the view at the 0,0,0 origin, facing down the negative z axis
+    // 1. Move the view point away from the model (translate by -viewOrigin)
+    // 2. Rotate the view using Euler angles (rotating about two specific axes)
+    // 3. Apply the radiant2openGL matrix which basically rotates the model 90 degrees
+    //    around two axes. This is needed to resemble the view of a human (who - with 
+    //    pitch, yaw and roll angles equal to zero - looks parallel to the xy plane,
+    //    and not down the negative z axis like openGL would.
+
+    // Translate the model by viewOrigin (with the later inverse() call this will 
+    // be the first applied translation by -viewOrigin)
+    Matrix4 modelview = Matrix4::getTranslation(_viewOrigin);
+
+    // Rotate the view like a human would turn their head. Due to the radiant2openGL transform
+    // the axes are used differently. Pitch is rotating around y instead of x, for example.
+    Vector3 radiant_eulerXYZ(0, _viewAngles[ui::CAMERA_PITCH], -_viewAngles[ui::CAMERA_YAW]);
+    modelview.rotateByEulerXYZDegrees(radiant_eulerXYZ);
+
+    // As last step apply the radiant2openGL transform which rotates first around z, then around y
+    modelview.multiplyBy(RADIANT2OPENGL);
+
+    // To get the translation transform applied first, inverse the whole stack
+    modelview.invert();
 
     return modelview;
 }
 
+void RenderPreview::updateModelViewMatrix()
+{
+    _modelView = calculateModelViewMatrix();
+}
+
 void RenderPreview::startPlayback()
 {
 	if (_timer.IsRunning())
@@ -285,7 +361,7 @@ void RenderPreview::stopPlayback()
 	toolbar->EnableTool(getToolBarToolByLabel(toolbar, "pauseTimeButton")->GetId(), false);
 	toolbar->EnableTool(getToolBarToolByLabel(toolbar, "stopTimeButton")->GetId(), false);
 
-    _glWidget->Refresh();
+    queueDraw();
 }
 
 bool RenderPreview::onPreRender()
@@ -332,13 +408,27 @@ void RenderPreview::drawPreview()
 {
     if (_renderingInProgress) return; // avoid double-entering this method
 
+    if (!_initialised)
+    {
+        initialisePreview();
+    }
+
     util::ScopedBoolLock lock(_renderingInProgress);
 
     glViewport(0, 0, _previewWidth, _previewHeight);
 
     // Set up the render and clear the drawing area in any case
     glDepthMask(GL_TRUE);
-    glClearColor(0.3f, 0.3f, 0.3f, 1.0f);
+    
+    if (getLightingModeEnabled())
+    {
+        glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
+    }
+    else
+    {
+        glClearColor(0.3f, 0.3f, 0.3f, 1.0f);
+    }
+
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 
     // Set up the camera
@@ -361,6 +451,9 @@ void RenderPreview::drawPreview()
 
     RenderStateFlags flags = getRenderFlagsFill();
 
+    // Hack-inject the model rotation matrix just before the render pass
+    //Matrix4 modelView = _volumeTest.GetModelview().getMultipliedBy(_modelRotation);
+
     // Launch the back end rendering
     _renderSystem->render(flags, _volumeTest.GetModelview(), projection);
 
@@ -387,82 +480,136 @@ void RenderPreview::renderWireFrame()
 
 void RenderPreview::onGLMouseClick(wxMouseEvent& ev)
 {
-	// Unset the focus on this GL preview, we don't want to 
-	// catch mousewheel events all over the place
-	wxWindow* parent = _glWidget->GetParent();
+    _lastX = ev.GetX();
+    _lastY = ev.GetY();
 
-	while (parent != NULL && parent->GetParent() != NULL)
-	{
-		parent = parent->GetParent();
-	}
+    if (ev.RightDown())
+    {
+        if (_freezePointer.isCapturing(_glWidget))
+        {
+            _freezePointer.endCapture();
+            return;
+        }
 
-	if (parent != NULL)
-	{
-		parent->SetFocus();
-	}
+        // Key event processing requires focus
+        _glWidget->SetFocus();
+
+        _freezePointer.startCapture(_glWidget,
+            [&](int x, int y, int mouseState) { onGLMotionDelta(x, y, mouseState); },
+            [&]() {}); // capture is released by FreezePointer
+    }
+}
+
+void RenderPreview::onGLMouseRelease(wxMouseEvent& ev)
+{
 }
 
 void RenderPreview::onGLMotion(wxMouseEvent& ev)
 {
-	if (ev.LeftIsDown()) // dragging with mouse button
+    if (ev.LeftIsDown()) // dragging with mouse button
     {
-        static double _lastX = ev.GetX();
-        static double _lastY = ev.GetY();
-
         // Calculate the mouse delta as a vector in the XY plane, and store the
         // current position for the next event.
-        Vector3 deltaPos(static_cast<float>(ev.GetX() - _lastX),
-                         static_cast<float>(_lastY - ev.GetY()),
-                         0);
+        Vector3 deltaPos(ev.GetX() - _lastX, _lastY - ev.GetY(), 0);
+
         _lastX = ev.GetX();
         _lastY = ev.GetY();
 
-        // Calculate the axis of rotation. This is the mouse vector crossed with the Z axis,
-        // to give a rotation axis in the XY plane at right-angles to the mouse delta.
-        static Vector3 _zAxis(0, 0, 1);
-        Vector3 axisRot = deltaPos.crossProduct(_zAxis);
+        // Get the rotation axes in model space
+        Vector3 localXRotAxis = _modelView.getInverse().transformDirection(Vector3(1, 0, 0));
+        Vector3 localZRotAxis = Vector3(0, 0, 1);
 
-		// Grab the contex for this widget
-		if (_glWidget->SetCurrent(GlobalOpenGL().getwxGLContext()))
+        if (deltaPos.y() != 0)
         {
-            // Premultiply the current modelview matrix with the rotation,
-            // in order to achieve rotations in eye space rather than object
-            // space. At this stage we are only calculating and storing the
-            // matrix for the GLDraw callback to use.
-            glLoadIdentity();
-            glRotated(-2, axisRot.x(), axisRot.y(), axisRot.z());
-            glMultMatrixd(_rotation);
-
-            // Save the new GL matrix for GL draw
-            glGetDoublev(GL_MODELVIEW_MATRIX, _rotation);
-
-            _glWidget->Refresh(); // trigger the GLDraw method to draw the actual model
+            _modelRotation.premultiplyBy(Matrix4::getRotation(localXRotAxis, degrees_to_radians(deltaPos.y())));
         }
+
+        if (deltaPos.x() != 0)
+        {
+            _modelRotation.premultiplyBy(Matrix4::getRotation(localZRotAxis, degrees_to_radians(-deltaPos.x())));
+        }
+
+        // Notify the subclasses to do something with this matrix
+        onModelRotationChanged();
+
+        queueDraw();
     }
 }
 
+void RenderPreview::onGLMotionDelta(int x, int y, unsigned int mouseState)
+{
+    const float dtime = 0.1f;
+    const float angleSpeed = 3; // camerasettings::anglespeed
+
+    _viewAngles[ui::CAMERA_PITCH] += y * dtime * angleSpeed;
+    _viewAngles[ui::CAMERA_YAW] += x * dtime * angleSpeed;
+
+    if (_viewAngles[ui::CAMERA_PITCH] > 90)
+        _viewAngles[ui::CAMERA_PITCH] = 90;
+    else if (_viewAngles[ui::CAMERA_PITCH] < -90)
+        _viewAngles[ui::CAMERA_PITCH] = -90;
+
+    if (_viewAngles[ui::CAMERA_YAW] >= 360)
+        _viewAngles[ui::CAMERA_YAW] -= 360;
+    else if (_viewAngles[ui::CAMERA_YAW] <= 0)
+        _viewAngles[ui::CAMERA_YAW] += 360;
+
+    updateModelViewMatrix();
+
+    queueDraw();
+}
+
 AABB RenderPreview::getSceneBounds()
 {
     return AABB(Vector3(0,0,0), Vector3(64,64,64));
 }
 
+void RenderPreview::resetModelRotation()
+{
+    _modelRotation = Matrix4::getIdentity();
+
+    onModelRotationChanged();
+}
+
 void RenderPreview::onGLScroll(wxMouseEvent& ev)
 {
     // Scroll increment is a fraction of the AABB radius
-    float inc = static_cast<float>(getSceneBounds().getRadius()) * 0.1f;
+    float inc = static_cast<float>(getSceneBounds().getRadius()) * 0.3f;
+
+    Vector3 forward(_modelView[2], _modelView[6], _modelView[10]);
 
 	if (ev.GetWheelRotation() > 0)
     {
-        _camDist += inc;
+        _viewOrigin -= forward * inc;
     }
     else if (ev.GetWheelRotation() < 0)
     {
-        _camDist -= inc;
+        _viewOrigin += forward * inc;
     }
 
-    if (!_renderingInProgress)
+    updateModelViewMatrix();
+
+    queueDraw();
+}
+
+void RenderPreview::onRenderModeChanged(wxCommandEvent& ev)
+{
+    if (ev.GetInt() == 0) // un-toggled
     {
-        _glWidget->Refresh();
+        return; // Don't react on UnToggle events
+    }
+
+    wxToolBar* toolbar = static_cast<wxToolBar*>(_mainPanel->FindWindow("RenderPreviewRenderModeToolbar"));
+
+    // This function will be called twice, once for the inactivating button and
+    // once for the activating button
+    if (getToolBarToolByLabel(toolbar, "texturedModeButton")->GetId() == ev.GetId())
+    {
+        setLightingModeEnabled(false);
+    }
+    else if (getToolBarToolByLabel(toolbar, "lightingModeButton")->GetId() == ev.GetId())
+    {
+        setLightingModeEnabled(true);
     }
 }
 
@@ -504,7 +651,7 @@ void RenderPreview::onStepForwardClick(wxCommandEvent& ev)
     }
 
     _renderSystem->setTime(_renderSystem->getTime() + MSEC_PER_FRAME);
-    _glWidget->Refresh();
+    queueDraw();
 }
 
 void RenderPreview::onStepBackClick(wxCommandEvent& ev)
@@ -523,7 +670,7 @@ void RenderPreview::onStepBackClick(wxCommandEvent& ev)
         _renderSystem->setTime(_renderSystem->getTime() - MSEC_PER_FRAME);
     }
 
-    _glWidget->Refresh();
+    queueDraw();
 }
 
 void RenderPreview::onSizeAllocate(wxSizeEvent& ev)
@@ -567,12 +714,52 @@ void RenderPreview::drawTime()
     GlobalOpenGL().drawString((boost::format("%.3f sec.") % (_renderSystem->getTime() * 0.001f)).str());
 }
 
+void RenderPreview::onGLKeyPress(wxKeyEvent& ev)
+{
+    if (!_freezePointer.isCapturing(_glWidget))
+    {
+        return;
+    }
+
+    float inc = static_cast<float>(getSceneBounds().getRadius()) * 0.12f;
+
+    if (ev.ShiftDown())
+    {
+        inc *= 4.0f;
+    }
+
+    Vector3 forward(_modelView[2], _modelView[6], _modelView[10]);
+    Vector3 right(_modelView[0], _modelView[4], _modelView[8]);
+
+    switch (ev.GetKeyCode())
+    {
+    case WXK_UP:
+        _viewOrigin -= forward * inc;
+        break;
+    case WXK_DOWN:
+        _viewOrigin += forward * inc;
+        break;
+    case WXK_RIGHT:
+        _viewOrigin += right * inc;
+        break;
+    case WXK_LEFT:
+        _viewOrigin -= right * inc;
+        break;
+    default:
+        ev.Skip();
+        return;
+    }
+
+    updateModelViewMatrix();
+    queueDraw();
+}
+
 void RenderPreview::_onFrame(wxTimerEvent& ev)
 {
     if (!_renderingInProgress)
     {
         _renderSystem->setTime(_renderSystem->getTime() + MSEC_PER_FRAME);
-        _glWidget->Refresh();
+        queueDraw();
     }
 }
 
