@@ -11,6 +11,7 @@
 #include "iscenegraph.h"
 #include "irender.h"
 
+#include "../FreezePointer.h"
 #include "render/ShaderStateRenderer.h"
 #include "render/SceneRenderWalker.h"
 #include "render/NopVolumeTest.h"
@@ -39,9 +40,12 @@ class RenderPreview :
 private:
     void connectToolbarSignals();
     void drawPreview();
-    void onGLMotion(wxMouseEvent& ev);
     void onGLScroll(wxMouseEvent& ev);
 	void onGLMouseClick(wxMouseEvent& ev);
+    void onGLMouseRelease(wxMouseEvent& ev);
+    void onGLMotion(wxMouseEvent& ev);
+    void onGLMotionDelta(int x, int y, unsigned int mouseState);
+    void onGLKeyPress(wxKeyEvent& ev);
 
 	void onStartPlaybackClick(wxCommandEvent& ev);
 	void onStopPlaybackClick(wxCommandEvent& ev);
@@ -51,12 +55,18 @@ private:
 
     void onSizeAllocate(wxSizeEvent& ev);
     void filtersChanged();
+    void onRenderModeChanged(wxCommandEvent& ev);
 
     void drawTime();
 
     // Called each frame by wxTimer
     void _onFrame(wxTimerEvent& ev);
 
+    void updateModelViewMatrix();
+    void updateActiveRenderModeButton();
+
+    void setupToolbars(bool enableAnimation);
+
 protected:
 	wxPanel* _mainPanel;
 
@@ -67,6 +77,10 @@ private:
     // GL widget
     GLWidget* _glWidget;
 
+    bool _initialised;
+
+    FreezePointer _freezePointer;
+
 protected:
     // The backend rendersystem instance
     RenderSystemPtr _renderSystem;
@@ -78,11 +92,18 @@ protected:
     // The scene adaptor passing nodes into our front-end renderer
     render::SceneRenderWalker _sceneWalker;
 
-    // Current distance between camera and preview
-    GLfloat _camDist;
+    // Current viewer position and view angles
+    Vector3 _viewOrigin;
+    Vector3 _viewAngles;
 
-    // Current rotation matrix
-    Matrix4 _rotation;
+    // Current modelview matrix
+    Matrix4 _modelView;
+
+    // The local model orientation
+    Matrix4 _modelRotation;
+
+    int _lastX;
+    int _lastY;
 
     // Mutex flag to avoid draw call bunching
     bool _renderingInProgress;
@@ -108,7 +129,14 @@ protected:
     virtual void setupSceneGraph();
 
     virtual Matrix4 getProjectionMatrix(float near_z, float far_z, float fieldOfView, int width, int height);
-    virtual Matrix4 getModelViewMatrix();
+    virtual const Matrix4& getModelViewMatrix();
+
+    virtual Matrix4 calculateModelViewMatrix();
+
+    void resetModelRotation();
+
+    // When the user requests a model rotation change (by dragging)
+    virtual void onModelRotationChanged() {}
 
     virtual void startPlayback();
     virtual void stopPlayback();
@@ -131,6 +159,9 @@ protected:
 
     void associateRenderSystem();
 
+    // Can be overridden by subclasses to update their scene/models
+    virtual void onRenderModeChanged() {}
+
     /**
      * \brief
      * Construct a RenderPreview
@@ -163,6 +194,18 @@ public:
         return _renderSystem;
     }
 
+    // Defines the position of the camera
+    void setViewOrigin(const Vector3& origin);
+
+    // Defines the view angles (euler angles in degrees)
+    void setViewAngles(const Vector3& angles);
+
+    // Check whether lighting mode is enabled
+    bool getLightingModeEnabled();
+
+    // Enable/disable lighting mode
+    void setLightingModeEnabled(bool enabled);
+
 	/// Schedule a GL widget redraw operation
     void queueDraw();
 };
