@@ -31,11 +31,12 @@ public:
 	AIVocalSetPropertyEditor(wxWindow* parent, Entity* entity,
 		const std::string& key, const std::string& options);
 
-	wxPanel* getWidget();
+	wxPanel* getWidget() override;
+	void updateFromEntity() override;
 
 	IPropertyEditorPtr createNew(wxWindow* parent, Entity* entity,
 								 const std::string& key,
-								 const std::string& options);
+								 const std::string& options) override;
 
 	std::string runDialog(Entity* entity, const std::string& key);
 
