@@ -13,8 +13,9 @@ class Doom3EntityCreator :
 public:
 
     // EntityCreator implementation
-	IEntityNodePtr createEntity(const IEntityClassPtr& eclass);
-	void connectEntities(const scene::INodePtr& source, const scene::INodePtr& target);
+	IEntityNodePtr createEntity(const IEntityClassPtr& eclass) override;
+	void connectEntities(const scene::INodePtr& source, const scene::INodePtr& target) override;
+    ITargetManagerPtr createTargetManager() override;
 
 	// RegisterableModule implementation
 	virtual const std::string& getName() const;
