@@ -174,6 +174,11 @@ void Light::updateOrigin() {
     GlobalSelectionSystem().pivotChanged();
 }
 
+const Vector3& Light::getUntransformedOrigin() const
+{
+    return m_originKey.get();
+}
+
 void Light::originChanged()
 {
     // The "origin" key has been changed, reset the current working copy to that value
@@ -531,8 +536,8 @@ void Light::renderProjectionPoints(RenderableCollector& collector,
                                    const Matrix4& localToWorld) const 
 {
     // Add the renderable light target
-    collector.highlightPrimitives(false);
-    collector.highlightFaces(false);
+    collector.setHighlightFlag(RenderableCollector::Highlight::Primitives, false);
+    collector.setHighlightFlag(RenderableCollector::Highlight::Faces, false);
 
     collector.SetState(_rRight.getShader(), RenderableCollector::eFullMaterials);
     collector.SetState(_rRight.getShader(), RenderableCollector::eWireframeOnly);
@@ -564,8 +569,8 @@ void Light::renderLightCentre(RenderableCollector& collector,
                               const VolumeTest& volume,
                               const Matrix4& localToWorld) const 
 {
-    collector.highlightPrimitives(false);
-    collector.highlightFaces(false);
+    collector.setHighlightFlag(RenderableCollector::Highlight::Primitives, false);
+    collector.setHighlightFlag(RenderableCollector::Highlight::Faces, false);
     collector.SetState(_rCentre.getShader(), RenderableCollector::eFullMaterials);
     collector.SetState(_rCentre.getShader(), RenderableCollector::eWireframeOnly);
 
@@ -665,12 +670,6 @@ void Light::rotate(const Quaternion& rotation)
     m_rotation.rotate(rotation);
 }
 
-const Matrix4& Light::getLocalPivot() const {
-    m_localPivot = m_rotation.getMatrix4();
-    m_localPivot.t().getVector3() = _lightBox.origin;
-    return m_localPivot;
-}
-
 // greebo: This returns the AABB of the WHOLE light (this includes the volume and all its selectable vertices)
 // Used to test the light for selection on mouse click.
 const AABB& Light::localAABB() const
