@@ -197,7 +197,7 @@ Matrix4 RenderableParticleBunch::getAimedMatrix(const Vector3& particleVelocity)
     Vector3 z = object2Vel.z().getVector3();
 
     // The particle needs to be rotated by this angle around the velocity axis
-    float aimedAngle = z.angle(-viewProj);
+    double aimedAngle = z.angle(-viewProj);
 
     // Use the cross to check whether to rotate in negative or positive direction
     if (z.crossProduct(-viewProj).dot(vel) > 0)
@@ -609,8 +609,7 @@ void RenderableParticleBunch::pushAimedParticles(ParticleRenderInfo& particle, s
 
         float height = static_cast<float>(velocity.getLength());
 
-        aimedParticle.aspect = 2 * aimedParticle.size / height;
-        aimedParticle.size = height * 0.5f;
+        aimedParticle.aspect = height / (2 * aimedParticle.size);
 
         // Calculate the vertical texture coordinates
         aimedParticle.tWidth = 1.0f / static_cast<float>(numQuads);
