@@ -6,28 +6,25 @@
 
 #include "debugging/ScopedDebugTimer.h"
 
-#include <iostream>
+#include "itextstream.h"
 
 namespace sound
 {
 
 // Constructor
 SoundManager::SoundManager() :
-	_emptyShader(new SoundShader("", "")),
-    _shadersLoaded(false)
+    _defLoader(std::bind(&SoundManager::loadShadersFromFilesystem, this)),
+	_emptyShader(new SoundShader("", ""))
 {}
 
 // Enumerate shaders
 void SoundManager::forEachShader(std::function<void(const ISoundShader&)> f)
-const
 {
     ensureShadersLoaded();
 
-	for (ShaderMap::const_iterator i = _shaders.begin();
-		 i != _shaders.end();
-		 ++i)
+	for (const ShaderMap::value_type& pair : _shaders)
 	{
-		f(*i->second);
+		f(*pair.second);
 	}
 }
 
@@ -38,10 +35,10 @@ bool SoundManager::playSound(const std::string& fileName)
 
 	// Try to open the file as it is
 	ArchiveFilePtr file = GlobalFileSystem().openFile(name);
-	std::cout << "Trying: " << name << std::endl;
+    rConsole() << "Trying: " << name << std::endl;
 	if (file != NULL) {
 		// File found, play it
-		std::cout << "Found file: " << name << std::endl;
+        rConsole() << "Found file: " << name << std::endl;
 		if (_soundPlayer) _soundPlayer->play(*file);
 		return true;
 	}
@@ -54,20 +51,20 @@ bool SoundManager::playSound(const std::string& fileName)
 
 	// Try to open the .ogg variant
 	name = root + ".ogg";
-	std::cout << "Trying: " << name << std::endl;
+    rConsole() << "Trying: " << name << std::endl;
 	file = GlobalFileSystem().openFile(name);
 	if (file != NULL) {
-		std::cout << "Found file: " << name << std::endl;
+        rConsole() << "Found file: " << name << std::endl;
 		if (_soundPlayer) _soundPlayer->play(*file);
 		return true;
 	}
 
 	// Try to open the file with .wav extension
 	name = root + ".wav";
-	std::cout << "Trying: " << name << std::endl;
+    rConsole() << "Trying: " << name << std::endl;
 	file = GlobalFileSystem().openFile(name);
 	if (file != NULL) {
-		std::cout << "Found file: " << name << std::endl;
+        rConsole() << "Found file: " << name << std::endl;
 		if (_soundPlayer) _soundPlayer->play(*file);
 		return true;
 	}
@@ -76,25 +73,29 @@ bool SoundManager::playSound(const std::string& fileName)
 	return false;
 }
 
-void SoundManager::stopSound() {
+void SoundManager::stopSound()
+{
 	if (_soundPlayer) _soundPlayer->stop();
 }
 
 ISoundShaderPtr SoundManager::getSoundShader(const std::string& shaderName)
 {
     ensureShadersLoaded();
+
 	ShaderMap::const_iterator found = _shaders.find(shaderName);
 
     // If the name was found, return it, otherwise return an empty shader object
-	return (found != _shaders.end()) ? found->second : _emptyShader;
+	return found != _shaders.end() ? found->second : _emptyShader;
 }
 
-const std::string& SoundManager::getName() const {
+const std::string& SoundManager::getName() const
+{
 	static std::string _name(MODULE_SOUNDMANAGER);
 	return _name;
 }
 
-const StringSet& SoundManager::getDependencies() const {
+const StringSet& SoundManager::getDependencies() const
+{
 	static StringSet _dependencies;
 
 	if (_dependencies.empty()) {
@@ -104,10 +105,12 @@ const StringSet& SoundManager::getDependencies() const {
 	return _dependencies;
 }
 
-void SoundManager::loadShadersFromFilesystem() const
+void SoundManager::loadShadersFromFilesystem()
 {
+    ShaderMapPtr foundShaders = std::make_shared<ShaderMap>();
+
 	// Pass a SoundFileLoader to the filesystem
-	SoundFileLoader loader(_shaders);
+    SoundFileLoader loader(*foundShaders);
 
     GlobalFileSystem().forEachFile(
         SOUND_FOLDER,			// directory
@@ -116,17 +119,14 @@ void SoundManager::loadShadersFromFilesystem() const
         99						// max depth
     );
 
-    rMessage() << _shaders.size() << " sound shaders found." << std::endl;
+    _shaders.swap(*foundShaders);
 
-    _shadersLoaded = true;
+    rMessage() << _shaders.size() << " sound shaders found." << std::endl;
 }
 
-void SoundManager::ensureShadersLoaded() const
+void SoundManager::ensureShadersLoaded()
 {
-    if (!_shadersLoaded)
-    {
-        loadShadersFromFilesystem();
-    }
+    _defLoader.ensureFinished();
 }
 
 void SoundManager::initialiseModule(const ApplicationContext& ctx)
@@ -136,6 +136,7 @@ void SoundManager::initialiseModule(const ApplicationContext& ctx)
     ApplicationContext::ArgumentList::const_iterator found(
         std::find(args.begin(), args.end(), "--disable-sound")
     );
+
     if (found == args.end())
     {
         rMessage() << "SoundManager: initialising sound playback"
@@ -144,9 +145,11 @@ void SoundManager::initialiseModule(const ApplicationContext& ctx)
     }
     else
     {
-        rMessage() << "SoundManager: sound ouput disabled"
+        rMessage() << "SoundManager: sound output disabled"
                              << std::endl;
     }
+
+    _defLoader.start();
 }
 
 } // namespace sound
