@@ -46,33 +46,29 @@ const std::size_t Brush::CONE_MAX_SIDES = 32;
 const std::size_t Brush::SPHERE_MIN_SIDES = 3;
 const std::size_t Brush::SPHERE_MAX_SIDES = 7;
 
-Brush::Brush(BrushNode& owner, const Callback& evaluateTransform, const Callback& boundsChanged) :
+Brush::Brush(BrushNode& owner, const Callback& evaluateTransform) :
     _owner(owner),
-	_instanceCounter(0),
-    _undoStateSaver(NULL),
-    m_map(0),
+    _undoStateSaver(nullptr),
+    _mapFileChangeTracker(nullptr),
     _faceCentroidPoints(GL_POINTS),
     _uniqueVertexPoints(GL_POINTS),
     _uniqueEdgePoints(GL_POINTS),
     m_evaluateTransform(evaluateTransform),
-    m_boundsChanged(boundsChanged),
     m_planeChanged(false),
     m_transformChanged(false),
 	_detailFlag(Structural)
 {
-    planeChanged();
+    onFacePlaneChanged();
 }
 
-Brush::Brush(BrushNode& owner, const Brush& other, const Callback& evaluateTransform, const Callback& boundsChanged) :
+Brush::Brush(BrushNode& owner, const Brush& other, const Callback& evaluateTransform) :
     _owner(owner),
-	_instanceCounter(0),
-    _undoStateSaver(NULL),
-    m_map(0),
+    _undoStateSaver(nullptr),
+    _mapFileChangeTracker(nullptr),
     _faceCentroidPoints(GL_POINTS),
     _uniqueVertexPoints(GL_POINTS),
     _uniqueEdgePoints(GL_POINTS),
     m_evaluateTransform(evaluateTransform),
-    m_boundsChanged(boundsChanged),
     m_planeChanged(false),
     m_transformChanged(false),
 	_detailFlag(Structural)
@@ -106,7 +102,7 @@ IFace& Brush::addFace(const Plane3& plane)
 {
     // Allocate a new Face
     undoSave();
-    push_back(FacePtr(new Face(*this, plane, this)));
+    push_back(FacePtr(new Face(*this, plane)));
 
     return *m_faces.back();
 }
@@ -115,7 +111,7 @@ IFace& Brush::addFace(const Plane3& plane, const Matrix4& texDef, const std::str
 {
     // Allocate a new Face
     undoSave();
-    push_back(FacePtr(new Face(*this, plane, texDef, shader, this)));
+    push_back(FacePtr(new Face(*this, plane, texDef, shader)));
 
     return *m_faces.back();
 }
@@ -141,65 +137,34 @@ void Brush::detach(BrushObserver& observer)
     m_observers.erase(&observer);
 }
 
-void Brush::forEachFace(const BrushVisitor& visitor) const {
-    for(Faces::const_iterator i = m_faces.begin(); i != m_faces.end(); ++i) {
-        visitor.visit(*(*i));
-    }
-}
-
 void Brush::forEachFace(const std::function<void(Face&)>& functor) const
 {
-    // Visit all faces, de-referencing the FacePtr using the lambda, and call the given functor
-    std::for_each(m_faces.begin(), m_faces.end(), [&] (const FacePtr& face) { functor(*face); } );
+    for (const FacePtr& face : m_faces) functor(*face);
 }
 
-void Brush::forEachFace_onInsertIntoScene(IMapFileChangeTracker* map) const {
-    for(Faces::const_iterator i = m_faces.begin(); i != m_faces.end(); ++i) {
-        (*i)->onInsertIntoScene(map);
-    }
-}
-
-void Brush::forEachFace_onRemoveFromScene(IMapFileChangeTracker* map) const {
-    for(Faces::const_iterator i = m_faces.begin(); i != m_faces.end(); ++i)
-    {
-        (*i)->onRemoveFromScene(map);
-    }
-}
-
-void Brush::onInsertIntoScene(IMapFileChangeTracker* map)
+void Brush::connectUndoSystem(IMapFileChangeTracker& changeTracker)
 {
-    if (++_instanceCounter == 1)
-    {
-        m_map = map;
-		_undoStateSaver = GlobalUndoSystem().getStateSaver(*this);
-        forEachFace_onInsertIntoScene(m_map);
-    }
+    assert(_undoStateSaver == nullptr);
+    
+    // Keep a reference around, we need it when faces are changing
+    _mapFileChangeTracker = &changeTracker;
+
+	_undoStateSaver = GlobalUndoSystem().getStateSaver(*this, changeTracker);
+
+    // Notify each face that we have a tracker
+    forEachFace([&](Face& face) { face.connectUndoSystem(changeTracker); });
 }
 
-void Brush::onRemoveFromScene(IMapFileChangeTracker* map)
+void Brush::disconnectUndoSystem(IMapFileChangeTracker& changeTracker)
 {
-    if (--_instanceCounter == 0)
-    {
-        forEachFace_onRemoveFromScene(m_map);
-        m_map = NULL;
-        _undoStateSaver = NULL;
-        GlobalUndoSystem().releaseStateSaver(*this);
-    }
-}
+    assert(_undoStateSaver != nullptr);
+    
+    // Notify each face
+    forEachFace([&](Face& face) { face.disconnectUndoSystem(changeTracker); });
 
-// observer
-void Brush::planeChanged() {
-    m_planeChanged = true;
-    aabbChanged();
-    _owner.lightsChanged();
-}
-
-void Brush::shaderChanged()
-{
-    planeChanged();
-
-    // Queue an UI update of the texture tools
-    ui::SurfaceInspector::update();
+    _mapFileChangeTracker = nullptr;
+    _undoStateSaver = nullptr;
+    GlobalUndoSystem().releaseStateSaver(*this);
 }
 
 void Brush::setShader(const std::string& newShader) {
@@ -263,7 +228,7 @@ void Brush::evaluateBRep() const {
 
 void Brush::transformChanged() {
     m_transformChanged = true;
-    planeChanged();
+    onFacePlaneChanged();
 }
 
 void Brush::evaluateTransform() {
@@ -274,8 +239,9 @@ void Brush::evaluateTransform() {
     }
 }
 
-void Brush::aabbChanged() {
-    m_boundsChanged();
+void Brush::aabbChanged()
+{
+    _owner.boundsChanged();
 }
 
 const AABB& Brush::localAABB() const {
@@ -308,10 +274,9 @@ void Brush::translate(const Vector3& translation)
 
 void Brush::transform(const Matrix4& matrix)
 {
-    bool mirror = matrix.getHandedness() == Matrix4::LEFTHANDED;
-
-    for(Faces::iterator i = m_faces.begin(); i != m_faces.end(); ++i) {
-        (*i)->transform(matrix, mirror);
+    for (const FacePtr& face : m_faces)
+    {
+        face->transform(matrix);
     }
 }
 
@@ -349,12 +314,9 @@ void Brush::appendFaces(const Faces& other) {
     }
 }
 
-void Brush::undoSave() {
-    if (m_map != 0) {
-        m_map->changed();
-    }
-
-    if (_undoStateSaver != NULL)
+void Brush::undoSave()
+{
+    if (_undoStateSaver != nullptr)
 	{
         _undoStateSaver->save(*this);
     }
@@ -374,7 +336,7 @@ void Brush::importState(const IUndoMementoPtr& state)
 	_detailFlag = memento._detailFlag;
     appendFaces(memento._faces);
 
-    planeChanged();
+    onFacePlaneChanged();
 
     for(Observers::iterator i = m_observers.begin(); i != m_observers.end(); ++i) {
         (*i)->DEBUG_verify();
@@ -387,8 +349,8 @@ FacePtr Brush::addFace(const Face& face) {
         return FacePtr();
     }
     undoSave();
-    push_back(FacePtr(new Face(*this, face, this)));
-    planeChanged();
+    push_back(FacePtr(new Face(*this, face)));
+    onFacePlaneChanged();
     return m_faces.back();
 }
 
@@ -398,8 +360,8 @@ FacePtr Brush::addPlane(const Vector3& p0, const Vector3& p1, const Vector3& p2,
         return FacePtr();
     }
     undoSave();
-    push_back(FacePtr(new Face(*this, p0, p1, p2, shader, projection, this)));
-    planeChanged();
+    push_back(FacePtr(new Face(*this, p0, p1, p2, shader, projection)));
+    onFacePlaneChanged();
     return m_faces.back();
 }
 
@@ -448,8 +410,9 @@ void Brush::reserve(std::size_t count) {
 void Brush::push_back(Faces::value_type face) {
     m_faces.push_back(face);
 
-    if (_instanceCounter != 0) {
-        m_faces.back()->onInsertIntoScene(m_map);
+    if (_undoStateSaver)
+    {
+        m_faces.back()->connectUndoSystem(*_mapFileChangeTracker);
     }
 
     for (Observers::iterator i = m_observers.begin(); i != m_observers.end(); ++i) {
@@ -458,9 +421,11 @@ void Brush::push_back(Faces::value_type face) {
     }
 }
 
-void Brush::pop_back() {
-    if (_instanceCounter != 0) {
-        m_faces.back()->onRemoveFromScene(m_map);
+void Brush::pop_back()
+{
+    if (_undoStateSaver)
+    {
+        m_faces.back()->disconnectUndoSystem(*_mapFileChangeTracker);
     }
 
     m_faces.pop_back();
@@ -470,9 +435,11 @@ void Brush::pop_back() {
     }
 }
 
-void Brush::erase(std::size_t index) {
-    if (_instanceCounter != 0) {
-        m_faces[index]->onRemoveFromScene(m_map);
+void Brush::erase(std::size_t index)
+{
+    if (_undoStateSaver)
+    {
+        m_faces[index]->disconnectUndoSystem(*_mapFileChangeTracker);
     }
 
     m_faces.erase(m_faces.begin() + index);
@@ -482,16 +449,40 @@ void Brush::erase(std::size_t index) {
     }
 }
 
-void Brush::connectivityChanged() {
-    for (Observers::iterator i = m_observers.begin(); i != m_observers.end(); ++i) {
-        (*i)->connectivityChanged();
+void Brush::onFacePlaneChanged()
+{
+    m_planeChanged = true;
+    aabbChanged();
+    _owner.lightsChanged();
+}
+
+void Brush::onFaceShaderChanged()
+{
+    onFacePlaneChanged();
+
+    // Queue an UI update of the texture tools
+    ui::SurfaceInspector::update();
+}
+
+void Brush::onFaceConnectivityChanged()
+{
+    for (auto i : m_observers)
+    {
+        i->connectivityChanged();
     }
 }
 
-void Brush::clear() {
+void Brush::onFaceEvaluateTransform()
+{
+    evaluateTransform();
+}
+
+void Brush::clear() 
+{
     undoSave();
-    if (_instanceCounter != 0) {
-        forEachFace_onRemoveFromScene(m_map);
+    if (_undoStateSaver)
+    {
+        forEachFace([&](Face& face) { face.disconnectUndoSystem(*_mapFileChangeTracker); });
     }
 
     m_faces.clear();
@@ -530,7 +521,7 @@ void Brush::removeEmptyFaces() {
     while (i < m_faces.size()) {
         if (!m_faces[i]->contributes()) {
             erase(i);
-            planeChanged();
+            onFacePlaneChanged();
         }
         else {
             ++i;
@@ -616,16 +607,18 @@ void Brush::copy(const Brush& other)
         addFace(*(*i));
     }
 
-    planeChanged();
+    onFacePlaneChanged();
 }
 
-void Brush::constructCuboid(const AABB& bounds, const std::string& shader, const TextureProjection& projection)
+void Brush::constructCuboid(const AABB& bounds, const std::string& shader)
 {
     const unsigned char box[3][2] = { { 0, 1 }, { 2, 0 }, { 1, 2 } };
 
     Vector3 mins(bounds.origin - bounds.extents);
     Vector3 maxs(bounds.origin + bounds.extents);
 
+    TextureProjection projection;
+
     clear();
     reserve(6);
 
@@ -651,11 +644,18 @@ void Brush::constructCuboid(const AABB& bounds, const std::string& shader, const
             addPlane(mins, planepts1, planepts2, shader, projection);
         }
     }
+
+    // Passing in the default-constructed projection will result in a very tiny texture scale, fix that
+    for (const FacePtr& face : m_faces)
+    {
+        face->applyDefaultTextureScale();
+    }
 }
 
-void Brush::constructPrism(const AABB& bounds, std::size_t sides, int axis, 
-                           const std::string& shader, const TextureProjection& projection)
+void Brush::constructPrism(const AABB& bounds, std::size_t sides, int axis, const std::string& shader)
 {
+    TextureProjection projection;
+
     if (sides < PRISM_MIN_SIDES)
     {
         rError() << "brushPrism: sides " << sides << ": too few sides, minimum is " << PRISM_MIN_SIDES << std::endl;
@@ -721,11 +721,18 @@ void Brush::constructPrism(const AABB& bounds, std::size_t sides, int axis,
 
         addPlane(planepts[0], planepts[1], planepts[2], shader, projection);
     }
+
+    // Passing in the default-constructed projection will result in a very tiny texture scale, fix that
+    for (const FacePtr& face : m_faces)
+    {
+        face->applyDefaultTextureScale();
+    }
 }
 
-void Brush::constructCone(const AABB& bounds, std::size_t sides, 
-                          const std::string& shader, const TextureProjection& projection)
+void Brush::constructCone(const AABB& bounds, std::size_t sides, const std::string& shader)
 {
+    TextureProjection projection;
+
     if (sides < CONE_MIN_SIDES)
     {
         rError() << "brushCone: sides " << sides << ": too few sides, minimum is " << CONE_MIN_SIDES << std::endl;
@@ -773,11 +780,18 @@ void Brush::constructCone(const AABB& bounds, std::size_t sides,
 
         addPlane(planepts[0], planepts[1], planepts[2], shader, projection);
     }
+
+    // Passing in the default-constructed projection will result in a very tiny texture scale, fix that
+    for (const FacePtr& face : m_faces)
+    {
+        face->applyDefaultTextureScale();
+    }
 }
 
-void Brush::constructSphere(const AABB& bounds, std::size_t sides, 
-                            const std::string& shader, const TextureProjection& projection)
+void Brush::constructSphere(const AABB& bounds, std::size_t sides, const std::string& shader)
 {
+    TextureProjection projection;
+
     if (sides < SPHERE_MIN_SIDES)
     {
         rError() << "brushSphere: sides " << sides << ": too few sides, minimum is " << SPHERE_MIN_SIDES << std::endl;
@@ -828,6 +842,12 @@ void Brush::constructSphere(const AABB& bounds, std::size_t sides,
             addPlane(planepts[0], planepts[1], planepts[2], shader, projection);
         }
     }
+
+    // Passing in the default-constructed projection will result in a very tiny texture scale, fix that
+    for (const FacePtr& face : m_faces)
+    {
+        face->applyDefaultTextureScale();
+    }
 }
 
 // greebo: this code is modeled after http://geomalgorithms.com/a13-_intersect-4.html
@@ -962,8 +982,8 @@ void Brush::removeDegenerateFaces() {
         Winding& degen = m_faces[i]->getWinding();
 
         if (degen.size() == 2) {
-            /*std::cout << "Removed degenerate face: " << Vector3(m_faces[i]->getPlane().plane3().normal())
-                                << " - " << float(m_faces[i]->getPlane().plane3().dist()) << "\n";*/
+            /*rConsole() << "Removed degenerate face: " << Vector3(m_faces[i]->getPlane().plane3().normal())
+                                << " - " << float(m_faces[i]->getPlane().plane3().dist()) << std::endl;*/
 
             // this is an "edge" face, where the plane touches the edge of the brush
             {
@@ -1074,8 +1094,9 @@ bool Brush::buildWindings() {
                 // update brush bounds
                 const Winding& winding = f.getWinding();
 
-                for (Winding::const_iterator i = winding.begin(); i != winding.end(); ++i) {
-                    m_aabb_local.includePoint(i->vertex);
+                for (const WindingVertex& wv : winding)
+				{
+                    m_aabb_local.includePoint(wv.vertex);
                 }
 
                 // update texture coordinates
