@@ -139,6 +139,7 @@ public:
 // ---------- CamWnd Implementation --------------------------------------------------
 
 CamWnd::CamWnd(wxWindow* parent) :
+    MouseToolHandler(IMouseToolGroup::Type::CameraView),
 	_mainWxWidget(loadNamedPanel(parent, "CamWndPanel")),
     _id(++_maxId),
     _view(true),
@@ -148,17 +149,12 @@ CamWnd::CamWnd(wxWindow* parent) :
     _freeMoveEnabled(false),
 	_wxGLWidget(new wxutil::GLWidget(_mainWxWidget, std::bind(&CamWnd::onRender, this), "CamWnd")),
     _timer(this),
-    _timerLock(false),
-    _deferredDraw(std::bind(&CamWnd::performDeferredDraw, this))
+    _timerLock(false)
 {
 	Connect(wxEVT_TIMER, wxTimerEventHandler(CamWnd::onFrame), NULL, this);
 
     constructGUIComponents();
 
-    GlobalMap().signal_mapValidityChanged().connect(
-        sigc::mem_fun(_deferredDraw, &DeferredDraw::onMapValidChanged)
-    );
-
     // Deactivate all commands, just to make sure
     disableDiscreteMoveEvents();
     disableFreeMoveEvents();
@@ -204,7 +200,7 @@ void CamWnd::constructToolbar()
 	const wxToolBarToolBase* texturedBtn = getToolBarToolByLabel(camToolbar, "texturedBtn");
 	const wxToolBarToolBase* lightingBtn = getToolBarToolByLabel(camToolbar, "lightingBtn");
 
-    if (!GlobalRenderSystem().shaderProgramsAvailable())
+    if (!GlobalOpenGL().shaderProgramsAvailable())
     {
         //lightingBtn->set_sensitive(false);
 		camToolbar->EnableTool(lightingBtn->GetId(), false);
@@ -275,7 +271,7 @@ void CamWnd::onGLExtensionsInitialised()
 	wxToolBar* camToolbar = findNamedObject<wxToolBar>(_mainWxWidget, "CamToolbar");
 	const wxToolBarToolBase* lightingBtn = getToolBarToolByLabel(camToolbar, "lightingBtn");
 
-	camToolbar->EnableTool(lightingBtn->GetId(), GlobalRenderSystem().shaderProgramsAvailable());
+    camToolbar->EnableTool(lightingBtn->GetId(), GlobalOpenGL().shaderProgramsAvailable());
 }
 
 void CamWnd::setFarClipButtonSensitivity()
@@ -334,7 +330,7 @@ SelectionTestPtr CamWnd::createSelectionTestForPoint(const Vector2& point)
     float selectEpsilon = registry::getValue<float>(RKEY_SELECT_EPSILON);
 
     // Get the mouse position
-    DeviceVector deviceEpsilon(selectEpsilon / getCamera().width, selectEpsilon / getCamera().height);
+    Vector2 deviceEpsilon(selectEpsilon / getCamera().width, selectEpsilon / getCamera().height);
 
     // Copy the current view and constrain it to a small rectangle
     render::View scissored(_view);
@@ -523,10 +519,6 @@ void CamWnd::enableFreeMove()
 
     enableFreeMoveEvents();
 
-    _freezePointer.startCapture(_wxGLWidget,
-        [&](int x, int y, int mouseState) { handleGLMouseMoveFreeMoveDelta(x, y, mouseState); },
-        [&]() { disableFreeMove(); }); // Disable free look mode when focus is lost
-	
     update();
 }
 
@@ -538,8 +530,6 @@ void CamWnd::disableFreeMove()
 
     disableFreeMoveEvents();
 
-    _freezePointer.endCapture();
-
     addHandlersMove();
 
     update();
@@ -758,13 +748,16 @@ void CamWnd::Cam_Draw()
 
     drawTime();
 
-    if (_activeMouseTool)
+    if (!_activeMouseTools.empty())
     {
         glMatrixMode(GL_PROJECTION);
         glLoadIdentity();
         glOrtho(0, (float)_camera.width, 0, (float)_camera.height, -100, 100);
 
-        _activeMouseTool->renderOverlay();
+        for (const ActiveMouseTools::value_type& i : _activeMouseTools)
+        {
+            i.second->renderOverlay();
+        }
     }
 
     // bind back to the default texture so that we don't have problems
@@ -777,18 +770,13 @@ void CamWnd::onRender()
 	draw();
 }
 
-void CamWnd::performDeferredDraw()
-{
-	_wxGLWidget->Refresh(false);
-}
-
 void CamWnd::draw()
 {
     if (_drawing) return;
 
     util::ScopedBoolLock lock(_drawing);
 
-    if (GlobalMap().isValid() && GlobalMainFrame().screenUpdatesEnabled())
+    if (GlobalMainFrame().screenUpdatesEnabled())
 	{
         GlobalOpenGL().assertNoErrors();
 
@@ -931,7 +919,7 @@ void CamWnd::queueDraw()
         return;
     }
 
-    _deferredDraw.draw();
+	_wxGLWidget->Refresh(false);
 }
 
 Vector3 CamWnd::getCameraOrigin() const
@@ -944,6 +932,21 @@ void CamWnd::setCameraOrigin(const Vector3& origin)
     _camera.setOrigin(origin);
 }
 
+Vector3 CamWnd::getRightVector() const
+{
+	return _camera.vright;
+}
+
+Vector3 CamWnd::getUpVector() const
+{
+	return _camera.vup;
+}
+
+Vector3 CamWnd::getForwardVector() const
+{
+	return _camera.vpn;
+}
+
 Vector3 CamWnd::getCameraAngles() const
 {
     return _camera.getAngles();
@@ -1033,21 +1036,79 @@ ui::CameraMouseToolEvent CamWnd::createMouseEvent(const Vector2& point, const Ve
     return ui::CameraMouseToolEvent(*this, normalisedDeviceCoords, delta);
 }
 
-void CamWnd::handleGLCapturedMouseMove(int x, int y, unsigned int mouseState)
+MouseTool::Result CamWnd::processMouseDownEvent(const MouseToolPtr& tool, const Vector2& point)
 {
-    if (!_activeMouseTool) return;
+    CameraMouseToolEvent ev = createMouseEvent(point);
+    return tool->onMouseDown(ev);
+}
 
-    bool mouseToolReceivesDeltas = (_activeMouseTool->getPointerMode() & ui::MouseTool::PointerMode::MotionDeltas) != 0;
+MouseTool::Result CamWnd::processMouseUpEvent(const MouseToolPtr& tool, const Vector2& point)
+{
+    CameraMouseToolEvent ev = createMouseEvent(point);
+    return tool->onMouseUp(ev);
+}
 
-    // New MouseTool event, passing the delta only
-    ui::CameraMouseToolEvent ev = mouseToolReceivesDeltas ?
+MouseTool::Result CamWnd::processMouseMoveEvent(const MouseToolPtr& tool, int x, int y)
+{
+    bool mouseToolReceivesDeltas = (tool->getPointerMode() & MouseTool::PointerMode::MotionDeltas) != 0;
+
+    // New MouseTool event, optionally passing the delta only
+    CameraMouseToolEvent ev = mouseToolReceivesDeltas ?
         createMouseEvent(Vector2(0, 0), Vector2(x, y)) :
         createMouseEvent(Vector2(x, y));
 
-    if (_activeMouseTool->onMouseMove(ev) == ui::MouseTool::Result::Finished)
+    return tool->onMouseMove(ev);
+}
+
+void CamWnd::startCapture(const ui::MouseToolPtr& tool)
+{
+    if (_freezePointer.isCapturing(_wxGLWidget))
     {
-        clearActiveMouseTool();
+        return;
     }
+
+    unsigned int pointerMode = tool->getPointerMode();
+
+    _freezePointer.startCapture(_wxGLWidget,
+        [&](int x, int y, int mouseState) // Motion Functor
+        { 
+            MouseToolHandler::onGLCapturedMouseMove(x, y, mouseState); 
+
+            if (freeMoveEnabled())
+            {
+                handleGLMouseMoveFreeMoveDelta(x, y, mouseState);
+            }
+        }, 
+        [&, tool]() { MouseToolHandler::handleCaptureLost(tool); }, // called when the capture is lost.
+        (pointerMode & MouseTool::PointerMode::Freeze) != 0,
+        (pointerMode & MouseTool::PointerMode::Hidden) != 0,
+        (pointerMode & MouseTool::PointerMode::MotionDeltas) != 0
+    );
+}
+
+void CamWnd::endCapture()
+{
+    if (!_freezePointer.isCapturing(_wxGLWidget))
+    {
+        return;
+    }
+
+    _freezePointer.endCapture();
+}
+
+IInteractiveView& CamWnd::getInteractiveView()
+{
+    return *this;
+}
+
+void CamWnd::forceRedraw()
+{
+    if (_drawing)
+    {
+        return;
+    }
+
+    _wxGLWidget->Update();
 }
 
 void CamWnd::onGLMouseButtonPress(wxMouseEvent& ev)
@@ -1057,97 +1118,18 @@ void CamWnd::onGLMouseButtonPress(wxMouseEvent& ev)
 	// which will propagate any key events accordingly.
 	GlobalMainFrame().getWxTopLevelWindow()->SetFocus();
 
-    // Run the regular routine handling the mousetool interaction
-    ui::MouseToolStack toolStack = GlobalCamera().getMouseToolsForEvent(ev);
-
-    // Construct the mousedown event and see if the tool is able to handle it
-    ui::CameraMouseToolEvent mouseEvent = createMouseEvent(Vector2(ev.GetX(), ev.GetY()));
-
-    _activeMouseTool = toolStack.handleMouseDownEvent(mouseEvent);
-
-    if (!_activeMouseTool)
-    {
-        return;
-    }
-
-    unsigned int pointerMode = _activeMouseTool->getPointerMode();
-
-    // Check if the mousetool requires pointer freeze, only do this if we're not already capturing
-    if ((pointerMode & ui::MouseTool::PointerMode::Capture) != 0 && !_freezePointer.isCapturing(_wxGLWidget))
-    {
-        _freezePointer.startCapture(_wxGLWidget,
-            [&](int x, int y, int mouseState) { handleGLCapturedMouseMove(x, y, mouseState); },   // Motion Functor
-            [&]() { clearActiveMouseTool(); }, // End move function, also called when the capture is lost.
-            (pointerMode & ui::MouseTool::PointerMode::Freeze) != 0,
-            (pointerMode & ui::MouseTool::PointerMode::Hidden) != 0,
-            (pointerMode & ui::MouseTool::PointerMode::MotionDeltas) != 0
-        );
-    }
-
-    // Register a hook to capture the ESC key during the active phase
-    _escapeListener.reset(new wxutil::KeyEventFilter(WXK_ESCAPE, [&]()
-    {
-        _activeMouseTool->onCancel();
-
-        // This also removes the active escape listener
-        clearActiveMouseTool();
-    }));
+    // Pass the call to the actual handler
+    MouseToolHandler::onGLMouseButtonPress(ev);
 }
 
 void CamWnd::onGLMouseButtonRelease(wxMouseEvent& ev)
 {
-    if (!_activeMouseTool) return;
-
-    // Construct the mousedown event and see which tool is able to handle it
-    ui::CameraMouseToolEvent mouseEvent = createMouseEvent(Vector2(ev.GetX(), ev.GetY()));
-
-    // Ask the active mousetool to handle this event
-    ui::MouseTool::Result result = _activeMouseTool->onMouseUp(mouseEvent);
-
-    if (result == ui::MouseTool::Result::Finished)
-    {
-        clearActiveMouseTool();
-    }
+    MouseToolHandler::onGLMouseButtonRelease(ev);
 }
 
 void CamWnd::onGLMouseMove(wxMouseEvent& ev)
 {
-    handleGLMouseMove(ev.GetX(), ev.GetY(), wxutil::MouseButton::GetStateForMouseEvent(ev));
-}
-
-void CamWnd::handleGLMouseMove(int x, int y, unsigned int state)
-{
-    // Construct the mousedown event and see which tool is able to handle it
-    ui::CameraMouseToolEvent mouseEvent = createMouseEvent(Vector2(x, y));
-
-    if (_activeMouseTool)
-    {
-        // Ask the active mousetool to handle this event
-        switch (_activeMouseTool->onMouseMove(mouseEvent))
-        {
-        case ui::MouseTool::Result::Finished:
-            // Tool is done
-            clearActiveMouseTool();
-            return;
-
-        case ui::MouseTool::Result::Activated:
-        case ui::MouseTool::Result::Continued:
-            return;
-
-        case ui::MouseTool::Result::Ignored:
-            break;
-        };
-    }
-
-    // Send mouse move events to all tools that want them
-    GlobalCamera().foreachMouseTool([&](const ui::MouseToolPtr& tool)
-    {
-        // The active tool already received that event above
-        if (tool != _activeMouseTool && tool->alwaysReceivesMoveEvents())
-        {
-            tool->onMouseMove(mouseEvent);
-        }
-    });
+    MouseToolHandler::onGLMouseMove(ev);
 }
 
 void CamWnd::handleGLMouseMoveFreeMoveDelta(int x, int y, unsigned int state)
@@ -1169,26 +1151,6 @@ void CamWnd::handleGLMouseMoveFreeMoveDelta(int x, int y, unsigned int state)
     }
 }
 
-void CamWnd::clearActiveMouseTool()
-{
-    // Reset the escape listener in any case
-    _escapeListener.reset();
-
-    if (!_activeMouseTool)
-    {
-        return;
-    }
-
-    // Freezing mouse tools: release the mouse cursor again
-    if (_activeMouseTool->getPointerMode() & ui::MouseTool::PointerMode::Capture && !freeMoveEnabled())
-    {
-        _freezePointer.endCapture();
-    }
-
-    // Tool is done
-    _activeMouseTool.reset();
-}
-
 void CamWnd::drawTime()
 {
     if (GlobalRenderSystem().getTime() == 0)
