@@ -9,12 +9,12 @@
 #include "wxutil/WindowPosition.h"
 #include "wxutil/XmlResourceBasedWidget.h"
 #include "wxutil/event/KeyEventFilter.h"
+#include "wxutil/MouseToolHandler.h"
 
 #include <wx/wxprec.h>
 #include <wx/glcanvas.h>
 #include <wx/timer.h>
 #include "render/View.h"
-#include "map/DeferredDraw.h"
 
 #include "RadiantCameraView.h"
 #include "Camera.h"
@@ -39,7 +39,8 @@ class CamWnd :
 	public boost::noncopyable,
     public sigc::trackable,
 	private wxutil::XmlResourceBasedWidget,
-	public wxEvtHandler
+	public wxEvtHandler,
+    protected wxutil::MouseToolHandler
 {
 private:
 	// Overall panel including toolbar and GL widget
@@ -75,12 +76,8 @@ private:
 	wxTimer _timer;
     bool _timerLock; // to avoid double-timer-firings
 
-	DeferredDraw _deferredDraw;
-
 	sigc::connection _glExtensionsInitialisedNotifier;
 
-    ui::MouseToolPtr _activeMouseTool;
-
     wxutil::KeyEventFilterPtr _escapeListener;
 
 public:
@@ -93,11 +90,12 @@ public:
 	int getId();
 
     // ICameraView implementation
-    SelectionTestPtr createSelectionTestForPoint(const Vector2& point);
-    const VolumeTest& getVolumeTest() const;
-    int getDeviceWidth() const;
-    int getDeviceHeight() const;
-    void queueDraw();
+    SelectionTestPtr createSelectionTestForPoint(const Vector2& point) override;
+    const VolumeTest& getVolumeTest() const override;
+    int getDeviceWidth() const override;
+    int getDeviceHeight() const override;
+    void queueDraw() override;
+    void forceRedraw() override;
 
 	void draw();
 	void update();
@@ -110,8 +108,12 @@ public:
 
 	Camera& getCamera();
 
-	Vector3 getCameraOrigin() const;
-	void setCameraOrigin(const Vector3& origin);
+	Vector3 getCameraOrigin() const override;
+	void setCameraOrigin(const Vector3& origin) override;
+
+	Vector3 getRightVector() const override;
+	Vector3 getUpVector() const override;
+	Vector3 getForwardVector() const override;
 
 	Vector3 getCameraAngles() const;
 	void setCameraAngles(const Vector3& angles);
@@ -149,6 +151,15 @@ public:
 	void startRenderTime();
 	void stopRenderTime();
 
+protected:
+    // Required overrides being a MouseToolHandler
+    virtual MouseTool::Result processMouseDownEvent(const MouseToolPtr& tool, const Vector2& point) override;
+    virtual MouseTool::Result processMouseUpEvent(const MouseToolPtr& tool, const Vector2& point) override;
+    virtual MouseTool::Result processMouseMoveEvent(const MouseToolPtr& tool, int x, int y) override;
+    virtual void startCapture(const MouseToolPtr& tool) override;
+    virtual void endCapture() override;
+    virtual IInteractiveView& getInteractiveView() override;
+
 private:
     void constructGUIComponents();
     void constructToolbar();
@@ -164,9 +175,7 @@ private:
 	void onRender();
 	void drawTime();
 
-	void performDeferredDraw();
-
-    ui::CameraMouseToolEvent createMouseEvent(const Vector2& point, const Vector2& delta = Vector2(0, 0));
+    CameraMouseToolEvent createMouseEvent(const Vector2& point, const Vector2& delta = Vector2(0, 0));
 
 	void onGLResize(wxSizeEvent& ev);
 
@@ -176,20 +185,12 @@ private:
 	void onGLMouseButtonRelease(wxMouseEvent& ev);
     void onGLMouseMove(wxMouseEvent& ev);
 
-    // Regular mouse move, when no mousetool is active
-	void handleGLMouseMove(int x, int y, unsigned int state);
-    
-    // Mouse motion callback when an active tool is capturing the mouse
-    void handleGLCapturedMouseMove(int x, int y, unsigned int state);
-
     // Mouse motion callback used in freelook mode only, processes deltas
     void handleGLMouseMoveFreeMoveDelta(int x, int y, unsigned int state);
     
 	void onGLExtensionsInitialised();
 
 	void onFrame(wxTimerEvent& ev);
-
-    void clearActiveMouseTool();
 };
 
 /**
