@@ -16,6 +16,7 @@
 #include "tools/ShaderClipboardTools.h"
 #include "tools/JumpToObjectTool.h"
 #include "tools/FreeMoveTool.h"
+#include "tools/PanViewTool.h"
 
 #include "FloatingCamWnd.h"
 #include <functional>
@@ -248,6 +249,25 @@ void GlobalCameraManager::update() {
 	}
 }
 
+void GlobalCameraManager::forceDraw()
+{
+    // Issue the update call to all cameras
+    for (CamWndMap::iterator i = _cameras.begin(); i != _cameras.end(); /* in-loop */)
+    {
+        CamWndPtr cam = i->second.lock();
+
+        if (cam)
+        {
+            cam->forceRedraw();
+            ++i;
+        }
+        else
+        {
+            _cameras.erase(i++);
+        }
+    }
+}
+
 void GlobalCameraManager::changeFloorUp(const cmd::ArgumentList& args) {
 	CamWndPtr camWnd = getActiveCamWnd();
 	if (camWnd == NULL) return;
@@ -505,7 +525,7 @@ unsigned int GlobalCameraManager::getStrafeForwardModifierFlags()
 
 ui::MouseToolStack GlobalCameraManager::getMouseToolsForEvent(wxMouseEvent& ev)
 {
-    unsigned int state = wxutil::MouseButton::GetStateForMouseEvent(ev);
+    unsigned int state = wxutil::MouseButton::GetButtonStateChangeForMouseEvent(ev);
     return GlobalMouseToolManager().getMouseToolsForEvent(ui::IMouseToolGroup::Type::CameraView, state);
 }
 
@@ -581,6 +601,7 @@ void GlobalCameraManager::initialiseModule(const ApplicationContext& ctx)
     IMouseToolGroup& toolGroup = GlobalMouseToolManager().getGroup(IMouseToolGroup::Type::CameraView);
 
     toolGroup.registerMouseTool(std::make_shared<FreeMoveTool>());
+	toolGroup.registerMouseTool(std::make_shared<PanViewTool>());
     toolGroup.registerMouseTool(std::make_shared<PickShaderTool>());
     toolGroup.registerMouseTool(std::make_shared<PasteShaderProjectedTool>());
     toolGroup.registerMouseTool(std::make_shared<PasteShaderNaturalTool>());
