@@ -25,6 +25,12 @@ Console::Console(wxWindow* parent) :
 	GetSizer()->Add(_view, 1, wxEXPAND);
 	GetSizer()->Add(_commandEntry, 0, wxEXPAND);
 
+    GlobalCommandSystem().addCommand("clear",
+        std::bind(&Console::clearCmd, this, std::placeholders::_1));
+
+    // Get a lock on the logging system before doing these changes
+    std::lock_guard<std::mutex> lock(module::GlobalModuleRegistry().getApplicationContext().getStreamLock());
+
 	// We're ready to catch log output, register ourselves
 	applog::LogWriter::Instance().attach(this);
 
@@ -37,16 +43,16 @@ Console::Console(wxWindow* parent) :
 			 level < applog::SYS_NUM_LOGLEVELS;
 			 level++)
 		{
-			writeLog(logger.getString(static_cast<applog::ELogLevel>(level)) + "\n",
-				static_cast<applog::ELogLevel>(level));
+            std::string bufferedText = logger.getString(static_cast<applog::ELogLevel>(level));
+
+            if (bufferedText.empty()) continue;
+
+            writeLog(bufferedText + "\n", static_cast<applog::ELogLevel>(level));
 		}
 	}
 
 	// Destruct the temporary buffer
 	applog::StringLogDevice::destroy();
-
-	GlobalCommandSystem().addCommand("clear", 
-        std::bind(&Console::clearCmd, this, std::placeholders::_1));
 }
 
 Console::~Console()
