@@ -6,15 +6,16 @@
 #include "iscenegraph.h"
 #include "idialogmanager.h"
 #include "ieventmanager.h"
-#include "iundo.h"
 #include "ifilesystem.h"
 #include "ifiletypes.h"
+#include "iselectiongroup.h"
 #include "ifilter.h"
 #include "icounter.h"
 #include "iradiant.h"
 #include "imainframe.h"
 #include "imapresource.h"
-#include "iselectionset.h"
+#include "iaasfile.h"
+#include "igame.h"
 
 #include "registry/registry.h"
 #include "stream/textfilestream.h"
@@ -28,12 +29,10 @@
 #include "brush/BrushModule.h"
 #include "xyview/GlobalXYWnd.h"
 #include "camera/GlobalCamera.h"
-#include "map/AutoSaver.h"
-#include "map/BasicContainer.h"
+#include "scene/BasicRootNode.h"
 #include "map/MapFileManager.h"
 #include "map/MapPositionManager.h"
 #include "map/PointFile.h"
-#include "map/RegionManager.h"
 #include "map/RootNode.h"
 #include "map/MapResource.h"
 #include "map/algorithm/Clone.h"
@@ -46,137 +45,60 @@
 #include "ui/layers/LayerControlDialog.h"
 #include "ui/prefabselector/PrefabSelector.h"
 #include "selection/algorithm/Primitives.h"
+#include "selection/algorithm/Group.h"
 #include "selection/shaderclipboard/ShaderClipboard.h"
 #include "modulesystem/ModuleRegistry.h"
 #include "modulesystem/StaticModule.h"
+#include "RenderableAasFile.h"
 
 #include <boost/format.hpp>
 #include "algorithm/ChildPrimitives.h"
 
-namespace map {
+namespace map 
+{
 
-    namespace {
+    namespace 
+	{
         const char* const MAP_UNNAMED_STRING = N_("unnamed.map");
 
         const char* const GKEY_LAST_CAM_POSITION = "/mapFormat/lastCameraPositionKey";
         const char* const GKEY_LAST_CAM_ANGLE = "/mapFormat/lastCameraAngleKey";
         const char* const GKEY_PLAYER_START_ECLASS = "/mapFormat/playerStartPoint";
         const char* const GKEY_PLAYER_HEIGHT = "/defaults/playerHeight";
-
-        // Traverse all entities and store the first worldspawn into the map
-        class MapWorldspawnFinder :
-            public scene::NodeVisitor
-        {
-        public:
-            virtual bool pre(const scene::INodePtr& node) {
-                if (node_is_worldspawn(node)) {
-                    if (GlobalMap().getWorldspawn() == NULL) {
-                        GlobalMap().setWorldspawn(node);
-                    }
-                }
-                return false;
-            }
-        };
-
-        class CollectAllWalker :
-            public scene::NodeVisitor
-        {
-            scene::INodePtr _root;
-            std::vector<scene::INodePtr>& _nodes;
-        public:
-            CollectAllWalker(scene::INodePtr root, std::vector<scene::INodePtr>& nodes) :
-                _root(root),
-                _nodes(nodes)
-            {}
-
-            ~CollectAllWalker() {
-                for (std::vector<scene::INodePtr>::iterator i = _nodes.begin();
-                     i != _nodes.end(); ++i)
-                {
-                    _root->removeChildNode(*i);
-                }
-            }
-
-            virtual bool pre(const scene::INodePtr& node) {
-                // Add this to the list
-                _nodes.push_back(node);
-                // Don't traverse deeper than first level
-                return false;
-            }
-        };
-
-        void Node_insertChildFirst(scene::INodePtr parent, scene::INodePtr child) {
-            // Create a container to collect all the existing entities in the scenegraph
-            std::vector<scene::INodePtr> nodes;
-
-            // Collect all the child nodes of <parent> and move them into the container
-            {
-                CollectAllWalker visitor(parent, nodes);
-                parent->traverseChildren(visitor);
-
-                // the CollectAllWalker removes the nodes from the parent on destruction
-            }
-
-            // Now that the <parent> is empty, insert the worldspawn as first child
-            parent->addChildNode(child);
-
-            // Insert all the nodes again
-            for (std::vector<scene::INodePtr>::iterator i = nodes.begin();
-                 i != nodes.end();
-                 ++i)
-            {
-                parent->addChildNode(*i);
-            }
-        }
-
-        scene::INodePtr createWorldspawn()
-        {
-          scene::INodePtr worldspawn(GlobalEntityCreator().createEntity(GlobalEntityClassManager().findOrInsert("worldspawn", true)));
-          Node_insertChildFirst(GlobalSceneGraph().root(), worldspawn);
-          return worldspawn;
-        }
     }
 
 Map::Map() :
     _lastCopyMapName(""),
-    m_valid(false),
     _saveInProgress(false)
 {
 	_mapSaveTimer.Pause();
 }
 
-void Map::realiseResource() {
-    if (m_resource != NULL) {
-        m_resource->realise();
-    }
-}
+void Map::loadMapResourceFromPath(const std::string& path)
+{
+	// Map loading started
+	signal_mapEvent().emit(MapLoading);
 
-void Map::unrealiseResource() {
-    if (m_resource != NULL) {
-        m_resource->unrealise();
-    }
-}
+	_resource = GlobalMapResourceManager().loadFromPath(_mapName);
 
-void Map::onResourceRealise() {
-    if (m_resource == NULL) {
+    if (!_resource)
+    {
         return;
     }
 
-    if (isUnnamed() || !m_resource->load()) {
+    if (isUnnamed() || !_resource->load())
+    {
         // Map is unnamed or load failed, reset map resource node to empty
-        m_resource->setNode(NewMapRoot(""));
-        IMapFileChangeTrackerPtr map = Node_getMapFile(m_resource->getNode());
-
-        if (map != NULL) {
-            map->save();
-        }
+        _resource->setNode(std::make_shared<RootNode>(""));
 
+        _resource->getNode()->getUndoChangeTracker().save();
+        
         // Rename the map to "unnamed" in any case to avoid overwriting the failed map
         setMapName(_(MAP_UNNAMED_STRING));
     }
 
     // Take the new node and insert it as map root
-    GlobalSceneGraph().setRoot(m_resource->getNode());
+    GlobalSceneGraph().setRoot(_resource->getNode());
 
     // Associate the Scenegaph with the global RenderSystem
     // This usually takes a while since all editor textures are loaded - display a dialog to inform the user
@@ -187,38 +109,8 @@ void Map::onResourceRealise() {
             module::GlobalModuleRegistry().getModule(MODULE_RENDERSYSTEM)));
     }
 
-    AutoSaver().clearChanges();
-
-    setValid(true);
-}
-
-void Map::onResourceUnrealise() {
-    if(m_resource != 0)
-    {
-        setValid(false);
-      setWorldspawn(scene::INodePtr());
-
-      GlobalUndoSystem().clear();
-      GlobalSelectionSetManager().deleteAllSelectionSets();
-
-      GlobalSceneGraph().setRoot(scene::INodePtr());
-    }
-}
-
-sigc::signal<void> Map::signal_mapValidityChanged() const
-{
-    return _sigMapValidityChanged;
-}
-
-void Map::setValid(bool valid)
-{
-    m_valid = valid;
-    _sigMapValidityChanged();
-}
-
-bool Map::isValid() const
-{
-    return m_valid;
+    // Map loading finished, emit the signal
+    signal_mapEvent().emit(MapLoaded);
 }
 
 void Map::updateTitle()
@@ -241,8 +133,9 @@ void Map::setMapName(const std::string& newName) {
     _mapName = newName;
 
     // Update the map resource's root node, if there is one
-    if (m_resource != NULL) {
-        m_resource->rename(newName);
+    if (_resource)
+	{
+        _resource->rename(newName);
     }
 
     // Update the title of the main window
@@ -257,21 +150,30 @@ bool Map::isUnnamed() const {
     return _mapName == _(MAP_UNNAMED_STRING);
 }
 
-void Map::setWorldspawn(scene::INodePtr node) {
-    m_world_node = node;
+void Map::setWorldspawn(const scene::INodePtr& node)
+{
+    _worldSpawnNode = node;
 }
 
-scene::INodePtr Map::getWorldspawn() {
-    return m_world_node;
+Map::MapEventSignal Map::signal_mapEvent() const
+{
+	return _mapEvent;
 }
 
-IMapRootNodePtr Map::getRoot() {
-    if (m_resource != NULL) {
+const scene::INodePtr& Map::getWorldspawn()
+{
+    return _worldSpawnNode;
+}
+
+scene::IMapRootNodePtr Map::getRoot()
+{
+    if (_resource)
+	{
         // Try to cast the node onto a root node and return
-        return std::dynamic_pointer_cast<IMapRootNode>(m_resource->getNode());
+        return std::dynamic_pointer_cast<scene::IMapRootNode>(_resource->getNode());
     }
 
-    return IMapRootNodePtr();
+    return scene::IMapRootNodePtr();
 }
 
 MapFormatPtr Map::getFormatForFile(const std::string& filename)
@@ -293,21 +195,20 @@ MapFormatPtr Map::getFormat()
 }
 
 // free all map elements, reinitialize the structures that depend on them
-void Map::freeMap() {
-    map::PointFile::Instance().clear();
+void Map::freeMap() 
+{
+	// Fire the map unloading event, 
+	// This will de-select stuff, clear the pointfile, etc.
+	signal_mapEvent().emit(MapUnloading);
 
-    GlobalSelectionSystem().setSelectedAll(false);
-    GlobalSelectionSystem().setSelectedAllComponents(false);
+	setWorldspawn(scene::INodePtr());
 
-    GlobalShaderClipboard().clear();
-    GlobalRegion().clear();
+	GlobalSceneGraph().setRoot(scene::IMapRootNodePtr());
 
-    m_resource->removeObserver(*this);
+	signal_mapEvent().emit(MapUnloaded);
 
     // Reset the resource pointer
-    m_resource = IMapResourcePtr();
-
-    GlobalLayerSystem().reset();
+    _resource.reset();
 }
 
 bool Map::isModified() const {
@@ -335,9 +236,9 @@ void Map::removeCameraPosition() {
     const std::string keyLastCamPos = game::current::getValue<std::string>(GKEY_LAST_CAM_POSITION);
     const std::string keyLastCamAngle = game::current::getValue<std::string>(GKEY_LAST_CAM_ANGLE);
 
-    if (m_world_node != NULL) {
+    if (_worldSpawnNode != NULL) {
         // Retrieve the entity from the worldspawn node
-        Entity* worldspawn = Node_getEntity(m_world_node);
+        Entity* worldspawn = Node_getEntity(_worldSpawnNode);
         assert(worldspawn != NULL); // This must succeed
 
         worldspawn->setKeyValue(keyLastCamPos, "");
@@ -352,9 +253,9 @@ void Map::saveCameraPosition()
     const std::string keyLastCamPos = game::current::getValue<std::string>(GKEY_LAST_CAM_POSITION);
     const std::string keyLastCamAngle = game::current::getValue<std::string>(GKEY_LAST_CAM_ANGLE);
 
-    if (m_world_node != NULL) {
+    if (_worldSpawnNode != NULL) {
         // Retrieve the entity from the worldspawn node
-        Entity* worldspawn = Node_getEntity(m_world_node);
+        Entity* worldspawn = Node_getEntity(_worldSpawnNode);
         assert(worldspawn != NULL); // This must succeed
 
         ui::CamWndPtr camWnd = GlobalCamera().getActiveCamWnd();
@@ -378,9 +279,9 @@ void Map::gotoStartPosition()
     Vector3 angles(0,0,0);
     Vector3 origin(0,0,0);
 
-    if (m_world_node != NULL) {
+    if (_worldSpawnNode != NULL) {
         // Retrieve the entity from the worldspawn node
-        Entity* worldspawn = Node_getEntity(m_world_node);
+        Entity* worldspawn = Node_getEntity(_worldSpawnNode);
         assert(worldspawn != NULL); // This must succeed
 
         // Try to find a saved "last camera position"
@@ -391,7 +292,7 @@ void Map::gotoStartPosition()
             // Construct the vector out of the std::string
             origin = string::convert<Vector3>(savedOrigin);
 
-            Vector3 angles = string::convert<Vector3>(
+            angles = string::convert<Vector3>(
                 worldspawn->getKeyValue(keyLastCamAngle)
             );
 
@@ -433,26 +334,49 @@ void Map::gotoStartPosition()
     focusViews(origin, angles);
 }
 
-scene::INodePtr Map::findWorldspawn() {
-    // Clear the current worldspawn node
-    setWorldspawn(scene::INodePtr());
+scene::INodePtr Map::findWorldspawn()
+{
+	scene::INodePtr worldspawn;
 
     // Traverse the scenegraph and search for the worldspawn
-    MapWorldspawnFinder visitor;
-    GlobalSceneGraph().root()->traverseChildren(visitor);
+	GlobalSceneGraph().root()->foreachNode([&](const scene::INodePtr& node)
+	{
+		if (Node_isWorldspawn(node)) 
+		{
+			worldspawn = node;
+			return false; // done traversing
+		}
 
-    return getWorldspawn();
+		return true;
+	});
+
+	// Set the worldspawn, might be null if nothing was found
+	setWorldspawn(worldspawn);
+
+    return worldspawn;
 }
 
-void Map::updateWorldspawn() {
-    if (findWorldspawn() == NULL) {
-        setWorldspawn(createWorldspawn());
-    }
+scene::INodePtr Map::createWorldspawn()
+{
+	scene::INodePtr worldspawn(GlobalEntityCreator().createEntity(
+		GlobalEntityClassManager().findOrInsert("worldspawn", true)));
+
+	// We want the world spawn entity to go for the pole position
+	GlobalSceneGraph().root()->addChildNode(worldspawn);
+
+	return worldspawn;
 }
 
-scene::INodePtr Map::findOrInsertWorldspawn() {
-    updateWorldspawn();
-    return getWorldspawn();
+const scene::INodePtr& Map::findOrInsertWorldspawn()
+{
+	// If we don't know any worldspawn yet, and can't find one either,
+	// let's create one afresh
+	if (!_worldSpawnNode && findWorldspawn() == nullptr)
+	{
+		setWorldspawn(createWorldspawn());
+	}
+
+    return _worldSpawnNode;
 }
 
 void Map::load(const std::string& filename) {
@@ -460,20 +384,13 @@ void Map::load(const std::string& filename) {
 
     setMapName(filename);
 
-    // Reset all layers before loading the file
-    GlobalLayerSystem().reset();
-    GlobalSelectionSystem().setSelectedAll(false);
-
     {
         wxutil::ScopeTimer timer("map load");
 
-        m_resource = GlobalMapResourceManager().capture(_mapName);
-        // greebo: Add the observer, this usually triggers a onResourceRealise() call.
-        m_resource->addObserver(*this);
+		loadMapResourceFromPath(_mapName);
 
         // Traverse the scenegraph and find the worldspawn
-        MapWorldspawnFinder finder;
-        GlobalSceneGraph().root()->traverseChildren(finder);
+		findWorldspawn();
     }
 
     rMessage() << "--- LoadMapFile ---\n";
@@ -491,9 +408,6 @@ void Map::load(const std::string& filename) {
     // Remove them, so that the user doesn't get bothered with them
     GlobalMapPosition().removePositions();
 
-    // Disable the region to make sure
-    GlobalRegion().disable();
-
     // Clear the shaderclipboard, the references are most probably invalid now
     GlobalShaderClipboard().clear();
 
@@ -527,7 +441,7 @@ bool Map::save(const MapFormatPtr& mapFormat)
     wxutil::ScopeTimer timer("map save");
 
     // Save the actual map resource
-    bool success = m_resource->save(mapFormat);
+    bool success = _resource->save(mapFormat);
 
     // Remove the saved camera position
     removeCameraPosition();
@@ -553,8 +467,7 @@ bool Map::save(const MapFormatPtr& mapFormat)
 void Map::createNew() {
     setMapName(_(MAP_UNNAMED_STRING));
 
-    m_resource = GlobalMapResourceManager().capture(_mapName);
-    m_resource->addObserver(*this);
+	loadMapResourceFromPath(_mapName);
 
     SceneChangeNotify();
 
@@ -570,7 +483,7 @@ bool Map::import(const std::string& filename)
     bool success = false;
 
     {
-        IMapResourcePtr resource = GlobalMapResourceManager().capture(filename);
+        IMapResourcePtr resource = GlobalMapResourceManager().loadFromPath(filename);
 
         if (resource->load())
         {
@@ -794,9 +707,9 @@ void Map::loadPrefabAt(const Vector3& targetCoords)
     /*MapFileSelection fileInfo =
         MapFileManager::getMapFileSelection(true, _("Load Prefab"), "prefab");*/
 
-	std::string path = ui::PrefabSelector::ChoosePrefab();
+	ui::PrefabSelector::Result result = ui::PrefabSelector::ChoosePrefab();
 
-	if (!path.empty())
+	if (!result.prefabPath.empty())
 	{
         UndoableCommand undo("loadPrefabAt");
 
@@ -804,7 +717,7 @@ void Map::loadPrefabAt(const Vector3& targetCoords)
         GlobalSelectionSystem().setSelectedAll(false);
 
         // Now import the prefab (imported items get selected)
-		import(path);
+		import(result.prefabPath);
 
         // Switch texture lock on
         bool prevTexLockState = GlobalBrush().textureLockEnabled();
@@ -815,6 +728,20 @@ void Map::loadPrefabAt(const Vector3& targetCoords)
 
         // Revert to previous state
         GlobalBrush().setTextureLock(prevTexLockState);
+
+		// Check whether we should group the prefab parts
+		if (result.insertAsGroup && GlobalSelectionSystem().countSelected() > 1)
+		{
+			try
+			{
+				selection::algorithm::groupSelected();
+			}
+			catch (selection::algorithm::CommandNotAvailableException& ex)
+			{
+				// Ignore grouping errors on prefab insert, just log the message
+				rError() << "Error grouping the prefab: " << ex.what() << std::endl;
+			}
+		}
     }
 }
 
@@ -849,11 +776,10 @@ void Map::registerCommands()
 }
 
 // Static command targets
-void Map::newMap(const cmd::ArgumentList& args) {
-    if (GlobalMap().askForSave(_("New Map"))) {
-        // Turn regioning off when starting a new map
-        GlobalRegion().disable();
-
+void Map::newMap(const cmd::ArgumentList& args)
+{
+    if (GlobalMap().askForSave(_("New Map")))
+	{
         GlobalMap().freeMap();
         GlobalMap().createNew();
     }
@@ -937,14 +863,14 @@ void Map::rename(const std::string& filename) {
         SceneChangeNotify();
     }
     else {
-        m_resource->save();
+        _resource->save();
         setModified(false);
     }
 }
 
 void Map::importSelected(std::istream& in)
 {
-    BasicContainerPtr root(new BasicContainer);
+    scene::INodePtr root(new scene::BasicRootNode);
 
     // Instantiate the default import filter
     class MapImportFilter :
@@ -1031,11 +957,14 @@ const std::string& Map::getName() const {
     return _name;
 }
 
-const StringSet& Map::getDependencies() const {
+const StringSet& Map::getDependencies() const 
+{
     static StringSet _dependencies;
 
-    if (_dependencies.empty()) {
+    if (_dependencies.empty())
+	{
         _dependencies.insert(MODULE_RADIANT);
+		_dependencies.insert(MODULE_GAMEMANAGER);
     }
 
     return _dependencies;
@@ -1057,9 +986,6 @@ void Map::initialiseModule(const ApplicationContext& ctx)
     // Add the Map-related commands to the EventManager
     registerCommands();
 
-    // Add the region-related commands to the EventManager
-    RegionManager::initialiseCommands();
-
     // Add the map position commands to the EventManager
     GlobalMapPosition().initialise();
 
