@@ -1,8 +1,7 @@
 #include "OpenGLShader.h"
 
 #include "GLProgramFactory.h"
-#include "OpenGLShaderPassAdd.h"
-#include "render/OpenGLRenderSystem.h"
+#include "../OpenGLRenderSystem.h"
 
 #include "iuimanager.h"
 #include "ishaders.h"
@@ -41,9 +40,20 @@ struct OpenGLShader::DBSTriplet
     }
 };
 
+OpenGLShader::OpenGLShader(OpenGLRenderSystem& renderSystem) :
+    _renderSystem(renderSystem),
+    _isVisible(true),
+    _useCount(0)
+{}
+
+OpenGLRenderSystem& OpenGLShader::getRenderSystem()
+{
+    return _renderSystem;
+}
+
 void OpenGLShader::destroy()
 {
-    _material = MaterialPtr();
+    _material.reset();
     _shaderPasses.clear();
 }
 
@@ -60,10 +70,12 @@ void OpenGLShader::addRenderable(const OpenGLRenderable& renderable,
 
 		if (pass->state().testRenderFlag(RENDER_BUMP))
 		{
-			if (lights != NULL)
+			if (lights != nullptr)
 			{
-				OpenGLShaderPassAdd add(*pass, renderable, modelview);
-				lights->forEachLight(std::bind(&OpenGLShaderPassAdd::visit, &add, std::placeholders::_1));
+                lights->forEachLight([&](const RendererLight& light)
+                {
+                    pass->addRenderable(renderable, modelview, &light);
+                });
 			}
 		}
 		else
@@ -84,10 +96,12 @@ void OpenGLShader::addRenderable(const OpenGLRenderable& renderable,
     {
         if (pass->state().testRenderFlag(RENDER_BUMP))
 		{
-			if (lights != NULL)
+			if (lights != nullptr)
 			{
-				OpenGLShaderPassAdd add(*pass, renderable, modelview, &entity);
-				lights->forEachLight(std::bind(&OpenGLShaderPassAdd::visit, &add, std::placeholders::_1));
+                lights->forEachLight([&](const RendererLight& light)
+                {
+                    pass->addRenderable(renderable, modelview, entity, &light);
+                });
 			}
 		}
 		else
@@ -120,7 +134,7 @@ bool OpenGLShader::isVisible() const
 
 void OpenGLShader::incrementUsed()
 {
-    if (++m_used == 1 && _material)
+    if (++_useCount == 1 && _material)
     {
 		_material->SetInUse(true);
     }
@@ -128,12 +142,35 @@ void OpenGLShader::incrementUsed()
 
 void OpenGLShader::decrementUsed()
 {
-    if (--m_used == 0 && _material)
+    if (--_useCount == 0 && _material)
     {
 		_material->SetInUse(false);
     }
 }
 
+bool OpenGLShader::realised() const
+{
+    return _material != 0;
+}
+
+void OpenGLShader::attach(ModuleObserver& observer)
+{
+    if (realised())
+    {
+        observer.realise();
+    }
+    m_observers.attach(observer);
+}
+
+void OpenGLShader::detach(ModuleObserver& observer)
+{
+    if (realised())
+    {
+        observer.unrealise();
+    }
+    m_observers.detach(observer);
+}
+
 void OpenGLShader::realise(const std::string& name)
 {
     // Construct the shader passes based on the name
@@ -144,7 +181,7 @@ void OpenGLShader::realise(const std::string& name)
 		// greebo: Check the filtersystem whether we're filtered
 		_material->setVisible(GlobalFilterSystem().isVisible(FilterRule::TYPE_TEXTURE, name));
 
-		if (m_used != 0)
+		if (_useCount != 0)
 		{
 			_material->SetInUse(true);
 		}
@@ -162,7 +199,7 @@ void OpenGLShader::insertPasses()
          i != _shaderPasses.end();
           ++i)
     {
-    	_glStateManager.insertSortedState(
+    	_renderSystem.insertSortedState(
             OpenGLStates::value_type((*i)->statePtr(), *i)
         );
     }
@@ -175,7 +212,7 @@ void OpenGLShader::removePasses()
          i != _shaderPasses.end();
          ++i)
 	{
-    	_glStateManager.eraseSortedState((*i)->statePtr());
+        _renderSystem.eraseSortedState((*i)->statePtr());
     }
 }
 
@@ -188,6 +225,11 @@ void OpenGLShader::unrealise()
     destroy();
 }
 
+const MaterialPtr& OpenGLShader::getMaterial() const
+{
+    return _material;
+}
+
 unsigned int OpenGLShader::getFlags() const
 {
     return _material->getMaterialFlags();
@@ -196,7 +238,7 @@ unsigned int OpenGLShader::getFlags() const
 // Append a default shader pass onto the back of the state list
 OpenGLState& OpenGLShader::appendDefaultPass()
 {
-    _shaderPasses.push_back(OpenGLShaderPassPtr(new OpenGLShaderPass(*this)));
+    _shaderPasses.push_back(std::make_shared<OpenGLShaderPass>(*this));
     OpenGLState& state = _shaderPasses.back()->state();
     return state;
 }
@@ -204,11 +246,8 @@ OpenGLState& OpenGLShader::appendDefaultPass()
 // Test if we can render in bump map mode
 bool OpenGLShader::canUseLightingMode() const
 {
-    return (
-        GlobalRenderSystem().shaderProgramsAvailable()
-    	&& GlobalRenderSystem().getCurrentShaderProgram() 
-           == RenderSystem::SHADER_PROGRAM_INTERACTION
-    );
+    return GlobalOpenGL().shaderProgramsAvailable() && 
+        _renderSystem.getCurrentShaderProgram() == RenderSystem::SHADER_PROGRAM_INTERACTION;
 }
 
 void OpenGLShader::setGLTexturesFromTriplet(OpenGLState& pass,
@@ -281,7 +320,7 @@ void OpenGLShader::appendInteractionLayer(const DBSTriplet& triplet)
 
         zPass.setSortPosition(OpenGLState::SORT_ZFILL);
 
-        zPass.glProgram = GLProgramFactory::instance().getProgram("depthFill");
+        zPass.glProgram = _renderSystem.getGLProgramFactory().getBuiltInProgram("depthFill");
     }
 
     // Add the DBS pass
@@ -300,7 +339,7 @@ void OpenGLShader::appendInteractionLayer(const DBSTriplet& triplet)
     dbsPass.setRenderFlag(RENDER_BUMP);
     dbsPass.setRenderFlag(RENDER_PROGRAM);
 
-    dbsPass.glProgram = GLProgramFactory::instance().getProgram("bumpMap");
+    dbsPass.glProgram = _renderSystem.getGLProgramFactory().getBuiltInProgram("bumpMap");
 
     if (vcolMode != ShaderLayer::VERTEX_COLOUR_NONE)
     {
@@ -543,6 +582,26 @@ void OpenGLShader::appendBlendLayer(const ShaderLayerPtr& layer)
 
     // Polygon offset
     state.polygonOffset = _material->getPolygonOffset();
+
+#if 0
+    if (!layer->getVertexProgram().empty() || !layer->getFragmentProgram().empty())
+    {
+        try
+        {
+            state.glProgram = _renderSystem.getGLProgramFactory().getProgram(
+                layer->getVertexProgram(),
+                layer->getFragmentProgram()
+            );
+        }
+        catch (std::runtime_error& ex)
+        {
+            rError() << "Failed to create GL program for material " <<
+                _material->getName() << ": " << ex.what() << std::endl;
+
+            state.glProgram = nullptr;
+        }
+    }
+#endif
 }
 
 // Construct a normal shader
@@ -724,6 +783,18 @@ void OpenGLShader::construct(const std::string& name)
               state.m_linewidth = 2;
               state.m_linestipple_factor = 3;
             }
+			else if (name == "$XY_OVERLAY_GROUP")
+			{
+				Vector3 colorSelBrushes(0, 0.4, 0.8);
+				state.setColour(colorSelBrushes[0],
+					colorSelBrushes[1],
+					colorSelBrushes[2],
+					1);
+				state.setRenderFlag(RENDER_LINESTIPPLE);
+				state.setSortPosition(OpenGLState::SORT_OVERLAY_FIRST);
+				state.m_linewidth = 2;
+				state.m_linestipple_factor = 3;
+			}
             else if (name == "$DEBUG_CLIPPED")
             {
               state.setRenderFlag(RENDER_DEPTHWRITE);
@@ -789,6 +860,24 @@ void OpenGLShader::construct(const std::string& name)
                                  | RENDER_POLYGONSTIPPLE);
               state.setSortPosition(OpenGLState::SORT_OVERLAY_FIRST);
             }
+            else if (name == "$AAS_AREA")
+            {
+				state.setColour(1, 1, 1, 1);
+				state.setRenderFlags(RENDER_DEPTHWRITE
+					| RENDER_DEPTHTEST
+					| RENDER_OVERRIDE);
+				state.setSortPosition(OpenGLState::SORT_OVERLAY_LAST);
+				state.setDepthFunc(GL_LEQUAL);
+
+				OpenGLState& hiddenLine = appendDefaultPass();
+				hiddenLine.setColour(1, 1, 1, 1);
+				hiddenLine.setRenderFlags(RENDER_DEPTHWRITE
+					| RENDER_DEPTHTEST
+					| RENDER_OVERRIDE
+					| RENDER_LINESTIPPLE);
+				hiddenLine.setSortPosition(OpenGLState::SORT_OVERLAY_LAST);
+				hiddenLine.setDepthFunc(GL_GREATER);
+            }
             else
             {
                 assert(false);
