@@ -2,8 +2,10 @@
 
 #include "iundo.h"
 #include "igrid.h"
+#include "iselectiongroup.h"
 #include "iradiant.h"
 #include "ieventmanager.h"
+#include "ilightnode.h"
 #include "imousetoolmanager.h"
 #include "editable.h"
 #include "Selectors.h"
@@ -23,33 +25,9 @@
 // Initialise the shader pointer
 ShaderPtr RadiantSelectionSystem::_state;
 
-    namespace {
-        const std::string RKEY_ROTATION_PIVOT = "user/ui/rotationPivotIsOrigin";
-    }
-
-// ------------ Helper Functions --------------------------------------------
-
-inline void matrix4_assign_rotation(Matrix4& matrix, const Matrix4& other) {
-    matrix[0] = other[0];
-    matrix[1] = other[1];
-    matrix[2] = other[2];
-    matrix[4] = other[4];
-    matrix[5] = other[5];
-    matrix[6] = other[6];
-    matrix[8] = other[8];
-    matrix[9] = other[9];
-    matrix[10] = other[10];
-}
-
-void matrix4_assign_rotation_for_pivot(Matrix4& matrix, const scene::INodePtr& node) {
-    EditablePtr editable = Node_getEditable(node);
-    // If the instance is editable, take the localpivot point into account, otherwise just apply the rotation
-    if (editable != 0) {
-        matrix4_assign_rotation(matrix, node->localToWorld().getMultipliedBy(editable->getLocalPivot()));
-    }
-    else {
-        matrix4_assign_rotation(matrix, node->localToWorld());
-    }
+namespace 
+{
+    const std::string RKEY_ROTATION_PIVOT = "user/ui/rotationPivotIsOrigin";
 }
 
 // --------- RadiantSelectionSystem Implementation ------------------------------------------
@@ -215,7 +193,7 @@ void RadiantSelectionSystem::pivotChanged() const
     SceneChangeNotify();
 }
 
-void RadiantSelectionSystem::pivotChangedSelection(const Selectable& selectable) {
+void RadiantSelectionSystem::pivotChangedSelection(const ISelectable& selectable) {
     pivotChanged();
 }
 
@@ -272,8 +250,8 @@ std::size_t RadiantSelectionSystem::countSelectedComponents() const {
 }
 
 // This is called if the selection changes, so that the local list of selected nodes can be updated
-void RadiantSelectionSystem::onSelectedChanged(const scene::INodePtr& node, const Selectable& selectable) {
-
+void RadiantSelectionSystem::onSelectedChanged(const scene::INodePtr& node, const ISelectable& selectable)
+{
     // Cache the selection state
     bool isSelected = selectable.isSelected();
     int delta = isSelected ? +1 : -1;
@@ -318,8 +296,8 @@ void RadiantSelectionSystem::onSelectedChanged(const scene::INodePtr& node, cons
 
 // greebo: This should be called "onComponentSelectionChanged", as it is a similar function of the above one
 // Updates the internal list of component nodes if the component selection gets changed
-void RadiantSelectionSystem::onComponentSelection(const scene::INodePtr& node, const Selectable& selectable) {
-
+void RadiantSelectionSystem::onComponentSelection(const scene::INodePtr& node, const ISelectable& selectable)
+{
     int delta = selectable.isSelected() ? +1 : -1;
 
     _countComponent += delta;
@@ -594,59 +572,101 @@ void RadiantSelectionSystem::SelectPoint(const render::View& view,
         }
 
         // Was the selection test successful (have we found anything to select)?
-        if (candidates.size() > 0) {
-            // Yes, now determine how we should interpret the click
-            switch (modifier) {
-                // If we are in toggle mode (Shift-Left-Click by default), just toggle the
-                // selection of the "topmost" item
-                case SelectionSystem::eToggle: {
-                    Selectable* best = *candidates.begin();
-                    // toggle selection of the object with least depth (=first in the list)
-                    best->setSelected(!best->isSelected());
-                }
-                break;
-                // greebo: eReplace mode gets active as soon as the user holds the replace modifiers down
-                // and clicks (by default: Alt-Shift). eReplace is only active during the first click
-                // afterwards we are in cycle mode.
-                // if cycle mode not enabled, enable it
-                case SelectionSystem::eReplace: {
-                    // select closest (=first in the list)
-                    (*candidates.begin())->setSelected(true);
-                }
-                break;
-                // select the next object in the list from the one already selected
-                // greebo: eCycle is set if the user keeps holding the replace modifiers (Alt-Shift)
-                // and does NOT move the mouse between the clicks, otherwise we fall back into eReplace mode
-                // Note: The mode is set in SelectObserver::testSelect()
-                case SelectionSystem::eCycle: {
-                    // Cycle through the selection pool and activate the item right after the currently selected
-                    SelectablesList::iterator i = candidates.begin();
-
-                    while (i != candidates.end()) {
-                        if ((*i)->isSelected()) {
-                            // unselect the currently selected one
-                            (*i)->setSelected(false);
-                            // check if there is a "next" item in the list, if not: select the first item
-                            ++i;
-                            if (i != candidates.end()) {
-                                (*i)->setSelected(true);
-                            }
-                            else {
-                                (*candidates.begin())->setSelected(true);
-                            }
-                            break;
-                        }
-                        ++i;
-                    } // while
-                } // case
-                break;
-                default:
-                break;
-            } // switch
-        }
+		performPointSelection(candidates, modifier);
     }
 }
 
+namespace algorithm
+{
+
+// If the selectable is a GroupSelectable, the respective callback is used
+inline void setSelectionStatus(ISelectable* selectable, bool selected)
+{
+	IGroupSelectable* groupSelectable = dynamic_cast<IGroupSelectable*>(selectable);
+
+	if (groupSelectable)
+	{
+		groupSelectable->setSelected(selected, true); // propagate selection state to group peers
+	}
+	else
+	{
+		selectable->setSelected(selected);
+	}
+}
+
+}
+
+void RadiantSelectionSystem::performPointSelection(const SelectablesList& candidates, EModifier modifier)
+{
+	if (candidates.empty()) return;
+
+	// Yes, now determine how we should interpret the click
+	switch (modifier) 
+	{
+		// If we are in toggle mode (Shift-Left-Click by default), just toggle the
+		// selection of the "topmost" item
+		case eToggle: 
+		{
+			ISelectable* best = candidates.front();
+			// toggle selection of the object with least depth (=first in the list)
+			algorithm::setSelectionStatus(best, !best->isSelected());
+		}
+		break;
+
+		// greebo: eReplace mode gets active as soon as the user holds the replace modifiers down
+		// and clicks (by default: Alt-Shift). eReplace is only active during the first click
+		// afterwards we are in cycle mode.
+		// if cycle mode not enabled, enable it
+		case eReplace:
+		{
+			// select closest (=first in the list)
+			algorithm::setSelectionStatus(candidates.front(), true);
+		}
+		break;
+
+		// select the next object in the list from the one already selected
+		// greebo: eCycle is set if the user keeps holding the replace modifiers (Alt-Shift)
+		// and does NOT move the mouse between the clicks, otherwise we fall back into eReplace mode
+		// Note: The mode is set in SelectObserver::testSelect()
+		case eCycle:
+		{
+			// Cycle through the selection pool and activate the item right after the currently selected
+			SelectablesList::const_iterator i = candidates.begin();
+
+			while (i != candidates.end())
+			{
+				if ((*i)->isSelected())
+				{
+					// unselect the currently selected one
+					algorithm::setSelectionStatus(*i, false);
+					//(*i)->setSelected(false);
+
+					// check if there is a "next" item in the list, if not: select the first item
+					++i;
+
+					if (i != candidates.end()) 
+					{
+						algorithm::setSelectionStatus(*i, true);
+						//(*i)->setSelected(true);
+					}
+					else
+					{
+						algorithm::setSelectionStatus(candidates.front(), true);
+						//candidates.front()->setSelected(true);
+					}
+					break;
+				}
+
+				++i;
+			}
+		}
+		break;
+
+		default: 
+			break;
+	};
+}
+
 /* greebo: This gets called by the SelectObserver if the user drags a box and holds down
  * any of the selection modifiers. Possible selection candidates are determined and selected/deselected
  */
@@ -691,10 +711,21 @@ void RadiantSelectionSystem::SelectArea(const render::View& view,
             testSelectScene(candidates, volume, scissored, Mode(), ComponentMode());
         }
 
-        // Cycle through the selection pool and toggle the candidates, but only if we are in toggle mode
-        for (SelectablesList::iterator i = candidates.begin(); i != candidates.end(); i++) {
-            (*i)->setSelected(!(modifier == SelectionSystem::eToggle && (*i)->isSelected()));
-        }
+		// Since toggling a selectable might trigger a group-selection
+		// we need to keep track of the desired state of each selectable 
+		typedef std::map<ISelectable*, bool> SelectablesMap;
+		SelectablesMap selectableStates;
+
+		for (ISelectable* selectable : candidates)
+		{
+			bool desiredState = !(modifier == SelectionSystem::eToggle && selectable->isSelected());
+			selectableStates.insert(SelectablesMap::value_type(selectable, desiredState));
+		}
+
+		for (const SelectablesMap::value_type& state : selectableStates)
+		{
+			algorithm::setSelectionStatus(state.first, state.second);
+		}
     }
 }
 
@@ -723,27 +754,27 @@ void RadiantSelectionSystem::translate(const Vector3& translation) {
 }
 
 // Applies the rotation vector <rotation> to the current selection
-void RadiantSelectionSystem::rotate(const Quaternion& rotation) {
+void RadiantSelectionSystem::rotate(const Quaternion& rotation)
+{
     // Check if there is anything to do
-    if (!nothingSelected()) {
-        // Store the quaternion internally
-        _rotation = rotation;
+    if (nothingSelected()) return;
 
-        // Perform the rotation according to the current mode
-        if (Mode() == eComponent) {
-            Scene_Rotate_Component_Selected(GlobalSceneGraph(), _rotation, _pivot2world.t().getVector3());
+    // Store the quaternion internally
+    _rotation = rotation;
 
-            matrix4_assign_rotation_for_pivot(_pivot2world, _componentSelection.ultimate());
-        }
-        else {
-            Scene_Rotate_Selected(GlobalSceneGraph(), _rotation, _pivot2world.t().getVector3());
-
-            matrix4_assign_rotation_for_pivot(_pivot2world, _selection.ultimate());
-        }
-
-        // Update the views
-        SceneChangeNotify();
+    // Perform the rotation according to the current mode
+    if (Mode() == eComponent)
+    {
+        Scene_Rotate_Component_Selected(GlobalSceneGraph(), _rotation, _pivot2world.t().getVector3());
     }
+    else
+    {
+  	    // Cycle through the selections and rotate them
+        foreachSelected(RotateSelected(rotation, _pivot2world.t().getVector3()));
+    }
+
+    // Update the views
+    SceneChangeNotify();
 }
 
 // Applies the scaling vector <scaling> to the current selection, this is called by the according ManipulatorComponents
@@ -998,8 +1029,6 @@ void RadiantSelectionSystem::keyChanged()
 /* greebo: This calculates and constructs the pivot point of the selection.
  * It cycles through all selected objects and creates its AABB. The origin point of the AABB
  * is basically the pivot point. Pivot2World is therefore a translation from (0,0,0) to the calculated origin.
- *
- * The pivot point is also snapped to the grid.
  */
 void RadiantSelectionSystem::ConstructPivot()
 {
@@ -1012,7 +1041,13 @@ void RadiantSelectionSystem::ConstructPivot()
 
     if (!nothingSelected())
     {
-        if (_selectionInfo.entityCount == 1 && _selectionInfo.totalCount == 1 &&
+		if (_selectionInfo.entityCount == 1 && _selectionInfo.totalCount == 1 &&
+			Node_getLightNode(ultimateSelected()))
+		{
+			// When a single light is selected, use the origin for rotation
+			objectPivot = Node_getLightNode(ultimateSelected())->getSelectAABB().origin;
+		}
+        else if (_selectionInfo.entityCount == 1 && _selectionInfo.totalCount == 1 &&
             registry::getValue<bool>(RKEY_ROTATION_PIVOT))
         {
             // Test, if a single entity is selected
@@ -1049,30 +1084,6 @@ void RadiantSelectionSystem::ConstructPivot()
 
         // The pivot2world matrix is just a translation from the world origin (0,0,0) to the object pivot
         _pivot2world = Matrix4::getTranslation(objectPivot);
-
-        // Only rotation and scaling need further calculations
-        switch (_manipulatorMode) {
-            case eTranslate:
-                break;
-            case eRotate:
-                if (Mode() == eComponent) {
-                    matrix4_assign_rotation_for_pivot(_pivot2world, _componentSelection.ultimate());
-                }
-                else {
-                    matrix4_assign_rotation_for_pivot(_pivot2world, _selection.ultimate());
-                }
-                break;
-            case eScale:
-                if (Mode() == eComponent) {
-                    matrix4_assign_rotation_for_pivot(_pivot2world, _componentSelection.ultimate());
-                }
-                else {
-                    matrix4_assign_rotation_for_pivot(_pivot2world, _selection.ultimate());
-                }
-                break;
-            default:
-                break;
-        } // switch
     }
 }
 /* greebo: Renders the currently active manipulator by setting the render state and
@@ -1080,8 +1091,8 @@ void RadiantSelectionSystem::ConstructPivot()
  */
 void RadiantSelectionSystem::renderSolid(RenderableCollector& collector, const VolumeTest& volume) const {
     if (!nothingSelected()) {
-        collector.highlightFaces(false);
-        collector.highlightPrimitives(false);
+        collector.setHighlightFlag(RenderableCollector::Highlight::Faces, false);
+        collector.setHighlightFlag(RenderableCollector::Highlight::Primitives, false);
 
         collector.SetState(_state, RenderableCollector::eWireframeOnly);
         collector.SetState(_state, RenderableCollector::eFullMaterials);
@@ -1105,16 +1116,20 @@ const std::string& RadiantSelectionSystem::getName() const {
     return _name;
 }
 
-const StringSet& RadiantSelectionSystem::getDependencies() const {
+const StringSet& RadiantSelectionSystem::getDependencies() const
+{
     static StringSet _dependencies;
 
-    if (_dependencies.empty()) {
+    if (_dependencies.empty())
+	{
         _dependencies.insert(MODULE_RENDERSYSTEM);
         _dependencies.insert(MODULE_EVENTMANAGER);
         _dependencies.insert(MODULE_XMLREGISTRY);
+		_dependencies.insert(MODULE_RADIANT);
         _dependencies.insert(MODULE_GRID);
         _dependencies.insert(MODULE_SCENEGRAPH);
         _dependencies.insert(MODULE_MOUSETOOLMANAGER);
+		_dependencies.insert(MODULE_MAP);
     }
 
     return _dependencies;
@@ -1188,6 +1203,9 @@ void RadiantSelectionSystem::initialiseModule(const ApplicationContext& ctx)
     camGroup.registerMouseTool(std::make_shared<ui::DragSelectionMouseToolFaceOnly>());
     camGroup.registerMouseTool(std::make_shared<ui::CycleSelectionMouseTool>());
     camGroup.registerMouseTool(std::make_shared<ui::CycleSelectionMouseToolFaceOnly>());
+
+	GlobalMapModule().signal_mapEvent().connect(
+		sigc::mem_fun(*this, &RadiantSelectionSystem::onMapEvent));
 }
 
 void RadiantSelectionSystem::shutdownModule() 
@@ -1202,7 +1220,7 @@ void RadiantSelectionSystem::shutdownModule()
     destroyStatic();
 }
 
-void RadiantSelectionSystem::checkComponentModeSelectionMode(const Selectable& selectable)
+void RadiantSelectionSystem::checkComponentModeSelectionMode(const ISelectable& selectable)
 {
 	// This seems to be a fail-safe method, to detect situations where component mode is still
 	// active without any primitive selected - in which case the method exits component mode.
@@ -1314,6 +1332,7 @@ void RadiantSelectionSystem::toggleClipManipulatorMode(bool newState)
 		SetManipulatorMode(eClip);
 
 		onManipulatorModeChanged();
+        onComponentModeChanged();
 	}
 }
 
@@ -1396,7 +1415,7 @@ void RadiantSelectionSystem::toggleGroupPartMode(bool newState)
         std::vector<scene::INodePtr> groupEntityNodes;
         foreachSelected([&](const scene::INodePtr& node)
         {
-            if (scene::isGroupNode(node))
+            if (scene::hasChildPrimitives(node))
             {
                 groupEntityNodes.push_back(node);
             }
@@ -1472,5 +1491,14 @@ void RadiantSelectionSystem::deselectCmd(const cmd::ArgumentList& args)
 	}
 }
 
+void RadiantSelectionSystem::onMapEvent(IMap::MapEvent ev)
+{
+	if (ev == IMap::MapUnloading || ev == IMap::MapLoading)
+	{
+		setSelectedAll(false);
+		setSelectedAllComponents(false);
+	}
+}
+
 // Define the static SelectionSystem module
 module::StaticModule<RadiantSelectionSystem> radiantSelectionSystemModule;
