@@ -16,6 +16,7 @@
 #include "ui/texturebrowser/TextureBrowser.h"
 #include "string/convert.h"
 #include "selectionlib.h"
+#include "entitylib.h"
 
 #include "SelectionPolicies.h"
 #include "selection/SceneWalkers.h"
@@ -39,11 +40,6 @@ namespace selection
 namespace algorithm
 {
 
-namespace
-{
-    const char* const RKEY_FREE_MODEL_ROTATION = "user/ui/freeModelRotation";
-}
-
 EntitySelectByClassnameWalker::EntitySelectByClassnameWalker(const ClassnameList& classnames) :
 	_classnames(classnames)
 {}
@@ -318,16 +314,99 @@ void showAllHidden(const cmd::ArgumentList& args) {
 	SceneChangeNotify();
 }
 
+/**
+ * Invert Selection walker
+ * Worldspawn brushes and patches will be considered, unless 
+ * entity selection mode is active.
+ * An entity with or without any child nodes will be considered. 
+ * The worldspawn node itself will be ignored. 
+ * func_static children will be ignored.
+ */
 class InvertSelectionWalker :
 	public scene::NodeVisitor
 {
+private:
 	SelectionSystem::EMode _mode;
-	SelectablePtr _selectable;
+
 public:
 	InvertSelectionWalker(SelectionSystem::EMode mode) :
 		_mode(mode)
 	{}
 
+	bool pre(const scene::INodePtr& node)
+	{
+		// Ignore hidden nodes (including their whole subgraph)
+		if (!node->visible()) return false;
+
+		Entity* entity = Node_getEntity(node);
+
+		// Check if we have a selectable
+		ISelectablePtr selectable = Node_getSelectable(node);
+
+		if (selectable)
+		{
+			switch (_mode)
+			{
+				case SelectionSystem::eEntity:
+					// Only consider non-worldspawn entities
+					if (entity != nullptr && !entity->isWorldspawn())
+					{
+						selectable->setSelected(!selectable->isSelected());
+					}
+					break;
+				case SelectionSystem::ePrimitive:
+					// Never select the worldspawn entity
+					if (entity == nullptr || !entity->isWorldspawn())
+					{
+						selectable->setSelected(!selectable->isSelected());
+					}
+					break;
+				case SelectionSystem::eGroupPart:
+					// All child primitives can be selected (worldspawn entity is filtered out)
+					if (entity == nullptr && Node_isEntity(node->getParent()))
+					{
+						selectable->setSelected(!selectable->isSelected());
+					}
+					break;
+				case SelectionSystem::eComponent:
+					break; // not handled here
+			}
+		}
+
+		// Determine whether we want to traverse child objects of entities
+		if (entity != nullptr)
+		{
+			if (_mode == SelectionSystem::eGroupPart)
+			{
+				// In group part mode, we want to traverse all entities but worldspawn
+				return !entity->isWorldspawn();
+			}
+			else if (_mode == SelectionSystem::eEntity)
+			{
+				// In Entity selection mode, we don't need to traverse entity subgraphs
+				// as we only want the parent entity
+				return false;
+			}
+
+			// In primitive mode, we want to traverse worldspawn only
+			return entity->isWorldspawn();
+		}
+
+		// We can traverse all non-entities by default
+		return true;
+	}
+};
+
+class InvertComponentSelectionWalker :
+	public scene::NodeVisitor
+{
+	SelectionSystem::EComponentMode _mode;
+	ComponentSelectionTestablePtr _selectable;
+public:
+	InvertComponentSelectionWalker(SelectionSystem::EComponentMode mode) :
+		_mode(mode)
+	{}
+
 	bool pre(const scene::INodePtr& node)
 	{
 		// Ignore hidden nodes
@@ -336,38 +415,23 @@ public:
 		Entity* entity = Node_getEntity(node);
 
 		// Check if we have a selectable
-		SelectablePtr selectable = Node_getSelectable(node);
+		ISelectablePtr selectable = Node_getSelectable(node);
 
 		if (selectable != NULL)
 		{
-			switch (_mode)
-			{
-				case SelectionSystem::eEntity:
-					if (entity != NULL && entity->getKeyValue("classname") != "worldspawn")
-					{
-						_selectable = selectable;
-					}
-					break;
-				case SelectionSystem::ePrimitive:
-					_selectable = selectable;
-					break;
-				case SelectionSystem::eComponent:
-					// Check if we have a componentselectiontestable instance
-					ComponentSelectionTestablePtr compSelTestable =
-						Node_getComponentSelectionTestable(node);
+			// Check if we have a componentselectiontestable instance
+			ComponentSelectionTestablePtr compSelTestable =
+				Node_getComponentSelectionTestable(node);
 
-					// Only add it to the list if the instance has components and is already selected
-					if (compSelTestable != NULL && selectable->isSelected())
-					{
-						_selectable = selectable;
-					}
-					break;
+			// Only add it to the list if the instance has components and is already selected
+			if (compSelTestable && selectable->isSelected())
+			{
+				_selectable = compSelTestable;
 			}
 		}
 
 		// Do we have a groupnode? If yes, don't traverse the children
-		if (entity != NULL && scene::isGroupNode(node) &&
-			entity->getKeyValue("classname") != "worldspawn")
+		if (entity != NULL && scene::hasChildPrimitives(node) && !entity->isWorldspawn())
 		{
 			// Don't traverse the children of this groupnode
 			return false;
@@ -378,17 +442,26 @@ public:
 
 	void post(const scene::INodePtr& node)
 	{
-		if (_selectable != NULL)
+		if (_selectable)
 		{
-			_selectable->invertSelected();
-			_selectable = SelectablePtr();
+			_selectable->invertSelectedComponents(_mode);
+			_selectable.reset();
 		}
 	}
 };
 
-void invertSelection(const cmd::ArgumentList& args) {
-	InvertSelectionWalker walker(GlobalSelectionSystem().Mode());
-	GlobalSceneGraph().root()->traverse(walker);
+void invertSelection(const cmd::ArgumentList& args)
+{
+	if (GlobalSelectionSystem().Mode() == SelectionSystem::eComponent)
+	{
+		InvertComponentSelectionWalker walker(GlobalSelectionSystem().ComponentMode());
+		GlobalSceneGraph().root()->traverse(walker);
+	}
+	else
+	{
+		InvertSelectionWalker walker(GlobalSelectionSystem().Mode());
+		GlobalSceneGraph().root()->traverse(walker);
+	}
 }
 
 void deleteSelection()
@@ -459,12 +532,12 @@ public:
 			return false;
 		}
 
-		SelectablePtr selectable = Node_getSelectable(node);
+		ISelectablePtr selectable = Node_getSelectable(node);
 
 		// ignore worldspawn
 		Entity* entity = Node_getEntity(node);
 		if (entity != NULL) {
-			if (entity->getKeyValue("classname") == "worldspawn") {
+			if (entity->isWorldspawn()) {
 				return true;
 			}
 		}
@@ -580,7 +653,7 @@ AABB getCurrentSelectionBounds()
 
 	GlobalSelectionSystem().foreachSelected([&] (const scene::INodePtr& node)
 	{
-		bounds.includeAABB(Node_getPivotBounds(node));
+		bounds.includeAABB(node->worldAABB());
 	});
 
 	return bounds;
@@ -1055,7 +1128,7 @@ void registerCommands()
 
 	GlobalEventManager().addCommand("ConnectSelection", "ConnectSelection");
     GlobalEventManager().addCommand("BindSelection", "BindSelection");
-    GlobalEventManager().addRegistryToggle("ToggleFreeModelRotation", RKEY_FREE_MODEL_ROTATION);
+    GlobalEventManager().addRegistryToggle("ToggleFreeObjectRotation", RKEY_FREE_OBJECT_ROTATION);
     GlobalEventManager().addCommand("CreateCurveNURBS", "CreateCurveNURBS");
     GlobalEventManager().addCommand("CreateCurveCatmullRom", "CreateCurveCatmullRom");
 
