@@ -11,6 +11,7 @@
 #include <fstream>
 #include <stdexcept>
 #include "registry/registry.h"
+#include "modulesystem/ModuleRegistry.h"
 #include <boost/filesystem.hpp>
 #include <boost/algorithm/string/case_conv.hpp>
 
@@ -100,10 +101,10 @@ void LanguageManager::initialiseModule(const ApplicationContext& ctx)
 	GlobalRegistry().setAttribute(RKEY_LANGUAGE, "volatile", "1"); // don't save this to user.xml
 
 	// Add Preferences
-	PreferencesPagePtr page = GlobalPreferenceSystem().getPage(_("Settings/Language"));
-	page->appendCombo(_("Language"), RKEY_LANGUAGE, langs);
+	IPreferencePage& page = GlobalPreferenceSystem().getPage(_("Settings/Language"));
+	page.appendCombo(_("Language"), RKEY_LANGUAGE, langs);
 
-	page->appendLabel(_("<b>Note:</b> You'll need to restart DarkRadiant\nafter changing the language setting."));
+	page.appendLabel(_("<b>Note:</b> You'll need to restart DarkRadiant\nafter changing the language setting."));
 }
 
 void LanguageManager::shutdownModule()
@@ -156,7 +157,7 @@ void LanguageManager::init(const ApplicationContext& ctx)
 	LanguageManagerPtr instancePtr(new LanguageManager);
 
 	// Hand that over to the module registry
-	module::getRegistry().registerModule(instancePtr);
+    module::ModuleRegistry::Instance().registerModule(instancePtr);
 
 	// Initialise the module manually
 	instancePtr->initFromContext(ctx);
