@@ -11,6 +11,7 @@
 #include "igame.h"
 #include "igroupdialog.h"
 #include "imainframe.h"
+#include "itextstream.h"
 
 #include "modulesystem/StaticModule.h"
 #include "selectionlib.h"
@@ -231,6 +232,20 @@ void EntityInspector::onKeyChange(const std::string& key,
 	row[_columns.name] = wxVariant(wxDataViewIconText(key, icon));
 	row[_columns.value] = value;
 
+	if (parms.type == "bool")
+	{
+		// Render a checkbox for boolean values (store an actual bool)
+		row[_columns.booleanValue] = value == "1";
+
+		// Column is enabled by default after assignment
+	}
+	else
+	{
+		// Store false to render the checkbox as unchecked
+		row[_columns.booleanValue] = false;
+		row[_columns.booleanValue].setEnabled(false);
+	}
+
 	// Text colour
 	row[_columns.name] = black;
 	row[_columns.value] = black;
@@ -260,6 +275,12 @@ void EntityInspector::onKeyChange(const std::string& key,
 	{
 		_valEntry->SetValue(value);
 	}
+
+	// Also update the property editor if the changed key is highlighted
+	if (_currentPropertyEditor && key == selectedKey)
+	{
+		_currentPropertyEditor->updateFromEntity();
+	}
 }
 
 void EntityInspector::onKeyErase(const std::string& key,
@@ -277,7 +298,7 @@ void EntityInspector::onKeyErase(const std::string& key,
     }
     else
     {
-        std::cerr << "EntityInspector: warning: removed key '" << key
+        rConsoleError() << "EntityInspector: warning: removed key '" << key
                   << "' not found in map." << std::endl;
     }
 }
@@ -289,7 +310,8 @@ void EntityInspector::createContextMenu()
 
 	_contextMenu->addItem(
 		new wxutil::StockIconTextMenuItem(_("Add property..."), wxART_PLUS),
-		std::bind(&EntityInspector::_onAddKey, this)
+		std::bind(&EntityInspector::_onAddKey, this),
+		std::bind(&EntityInspector::_testAddKey, this)
 	);
 	_contextMenu->addItem(
 		new wxutil::StockIconTextMenuItem(_("Delete property"), wxART_MINUS),
@@ -300,22 +322,37 @@ void EntityInspector::createContextMenu()
 	_contextMenu->addSeparator();
 
 	_contextMenu->addItem(
-		new wxutil::StockIconTextMenuItem(_("Copy Spawnarg"), wxART_COPY),
+		new wxutil::StockIconTextMenuItem(_("Copy Spawnarg(s)"), wxART_COPY),
 		std::bind(&EntityInspector::_onCopyKey, this),
 		std::bind(&EntityInspector::_testCopyKey, this)
 	);
 	_contextMenu->addItem(
-		new wxutil::StockIconTextMenuItem(_("Cut Spawnarg"), wxART_CUT),
+		new wxutil::StockIconTextMenuItem(_("Cut Spawnarg(s)"), wxART_CUT),
 		std::bind(&EntityInspector::_onCutKey, this),
 		std::bind(&EntityInspector::_testCutKey, this)
 	);
 	_contextMenu->addItem(
-		new wxutil::StockIconTextMenuItem(_("Paste Spawnarg"), wxART_PASTE),
+		new wxutil::StockIconTextMenuItem(_("Paste Spawnarg(s)"), wxART_PASTE),
 		std::bind(&EntityInspector::_onPasteKey, this),
 		std::bind(&EntityInspector::_testPasteKey, this)
 	);
 }
 
+void EntityInspector::onRadiantStartup()
+{
+	// Add entity inspector to the group dialog
+	IGroupDialog::PagePtr page(new IGroupDialog::Page);
+
+	page->name = "entity";
+	page->windowLabel = _("Entity");
+	page->page = getWidget();
+	page->tabIcon = "cmenu_add_entity.png";
+	page->tabLabel = _("Entity");
+	page->position = IGroupDialog::Page::Position::EntityInspector;
+
+	GlobalGroupDialog().addPage(page);
+}
+
 void EntityInspector::onRadiantShutdown()
 {
     // Remove all previously stored pane information
@@ -354,7 +391,8 @@ const StringSet& EntityInspector::getDependencies() const
 {
 	static StringSet _dependencies;
 
-	if (_dependencies.empty()) {
+	if (_dependencies.empty())
+	{
 		_dependencies.insert(MODULE_XMLREGISTRY);
 		_dependencies.insert(MODULE_UIMANAGER);
 		_dependencies.insert(MODULE_SELECTIONSYSTEM);
@@ -372,6 +410,9 @@ void EntityInspector::initialiseModule(const ApplicationContext& ctx)
 {
 	construct();
 
+	GlobalRadiant().signal_radiantStarted().connect(
+		sigc::mem_fun(this, &EntityInspector::onRadiantStartup)
+	);
 	GlobalRadiant().signal_radiantShutdown().connect(
         sigc::mem_fun(this, &EntityInspector::onRadiantShutdown)
     );
@@ -421,12 +462,16 @@ wxWindow* EntityInspector::createTreeViewPane(wxWindow* parent)
 
 	_kvStore = new wxutil::TreeModel(_columns, true); // this is a list model
 
-	_keyValueTreeView = wxutil::TreeView::CreateWithModel(treeViewPanel, _kvStore, wxDV_SINGLE);
+	_keyValueTreeView = wxutil::TreeView::CreateWithModel(treeViewPanel, _kvStore, wxDV_MULTIPLE);
 
     // Search in both name and value columns
     _keyValueTreeView->AddSearchColumn(_columns.name);
     _keyValueTreeView->AddSearchColumn(_columns.value);
 
+	// Add the checkbox for boolean properties
+	_keyValueTreeView->AppendToggleColumn("", _columns.booleanValue.getColumnIndex(),
+		wxDATAVIEW_CELL_ACTIVATABLE, wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT);
+
 	// Create the Property column (has an icon)
 	_keyValueTreeView->AppendIconTextColumn(_("Property"), 
 		_columns.name.getColumnIndex(), wxDATAVIEW_CELL_INERT, 
@@ -439,7 +484,7 @@ wxWindow* EntityInspector::createTreeViewPane(wxWindow* parent)
 
 	// Add the help icon
 	_helpColumn = _keyValueTreeView->AppendBitmapColumn(_("?"), 
-		_columns.helpIcon.getColumnIndex(), wxDATAVIEW_CELL_INERT, wxCOL_WIDTH_AUTOSIZE);
+		_columns.helpIcon.getColumnIndex(), wxDATAVIEW_CELL_INERT, wxCOL_WIDTH_AUTOSIZE, wxALIGN_NOT);
 	_helpColumn->SetHidden(true);
 
 	// Used to update the help text
@@ -448,6 +493,11 @@ wxWindow* EntityInspector::createTreeViewPane(wxWindow* parent)
 	_keyValueTreeView->Connect(wxEVT_DATAVIEW_ITEM_CONTEXT_MENU, 
 		wxDataViewEventHandler(EntityInspector::_onContextMenu), NULL, this);
 
+	// When the toggle column is clicked to check/uncheck the box, the model's column value
+	// is directly changed by the wxWidgets event handlers. On model value change, this event is fired afterwards
+	_keyValueTreeView->Connect(wxEVT_DATAVIEW_ITEM_VALUE_CHANGED,
+		wxDataViewEventHandler(EntityInspector::_onDataViewItemChanged), NULL, this);
+
 	wxBoxSizer* buttonHbox = new wxBoxSizer(wxHORIZONTAL);
 
 	// Pack in the key and value edit boxes
@@ -475,7 +525,14 @@ wxWindow* EntityInspector::createTreeViewPane(wxWindow* parent)
 
 std::string EntityInspector::getSelectedKey()
 {
-	wxDataViewItem item = _keyValueTreeView->GetSelection();
+	wxDataViewItemArray selectedItems;
+	if (_keyValueTreeView->GetSelections(selectedItems) != 1)
+	{
+		// Multiple or nothing selected, return empty string
+		return std::string();
+	}
+
+	const wxDataViewItem& item = selectedItems.front();
 
 	if (!item.IsOk()) return "";
 
@@ -487,7 +544,14 @@ std::string EntityInspector::getSelectedKey()
 
 std::string EntityInspector::getListSelection(const wxutil::TreeModel::Column& col)
 {
-	wxDataViewItem item = _keyValueTreeView->GetSelection();
+	wxDataViewItemArray selectedItems;
+	if (_keyValueTreeView->GetSelections(selectedItems) != 1)
+	{
+		// Multiple or nothing selected, return empty string
+		return std::string();
+	}
+
+	const wxDataViewItem& item = selectedItems.front();
 
 	if (!item.IsOk()) return "";
 
@@ -498,7 +562,14 @@ std::string EntityInspector::getListSelection(const wxutil::TreeModel::Column& c
 
 bool EntityInspector::getListSelectionBool(const wxutil::TreeModel::Column& col)
 {
-	wxDataViewItem item = _keyValueTreeView->GetSelection();
+	wxDataViewItemArray selectedItems;
+	if (_keyValueTreeView->GetSelections(selectedItems) != 1)
+	{
+		// Multiple or nothing selected, return false
+		return false;
+	}
+
+	const wxDataViewItem& item = selectedItems.front();
 
 	if (!item.IsOk()) return false;
 
@@ -532,7 +603,7 @@ void EntityInspector::updateGUIElements()
 
         // Disable the dialog and clear the TreeView
 		_editorFrame->Enable(false);
-        _keyValueTreeView->Enable(false);
+        _keyValueTreeView->Enable(true); // leave the treeview enabled
         _showInheritedCheckbox->Enable(false);
         _showHelpColumnCheckbox->Enable(false);
     }
@@ -549,70 +620,13 @@ void EntityInspector::selectionChanged(const scene::INodePtr& node, bool isCompo
     requestIdleCallback();
 }
 
-namespace
-{
-
-    // SelectionSystem visitor to set a keyvalue on each entity, checking for
-    // func_static-style name=model requirements
-    class EntityKeySetter
-    : public SelectionSystem::Visitor
-    {
-        // Key and value to set on all entities
-        std::string _key;
-        std::string _value;
-
-    public:
-
-        // Construct with key and value to set
-        EntityKeySetter(const std::string& k, const std::string& v)
-        : _key(k), _value(v)
-        { }
-
-        // Required visit function
-        void visit(const scene::INodePtr& node) const
-        {
-            Entity* entity = Node_getEntity(node);
-            if (entity)
-            {
-                // Check if we have a func_static-style entity
-                std::string name = entity->getKeyValue("name");
-                std::string model = entity->getKeyValue("model");
-                bool isFuncType = (!name.empty() && name == model);
-
-                // Set the actual value
-                entity->setKeyValue(_key, _value);
-
-                // Check for name key changes of func_statics
-                if (isFuncType && _key == "name")
-                {
-                    // Adapt the model key along with the name
-                    entity->setKeyValue("model", _value);
-				}
-            }
-			else if (Node_isPrimitive(node)) {
-				// We have a primitve node selected, check its parent
-				scene::INodePtr parent(node->getParent());
-
-				if (parent == NULL) return;
-
-				Entity* parentEnt = Node_getEntity(parent);
-
-				if (parentEnt != NULL) {
-					// We have child primitive of an entity selected, the change
-					// should go right into that parent entity
-					parentEnt->setKeyValue(_key, _value);
-				}
-			}
-        }
-    };
-}
-
 std::string EntityInspector::cleanInputString(const std::string &input)
 {
     std::string ret = input;
 
-    boost::algorithm::replace_all( ret, "\n", "" );
-    boost::algorithm::replace_all( ret, "\r", "" );
+    boost::algorithm::replace_all(ret, "\n", "");
+    boost::algorithm::replace_all(ret, "\r", "");
+
     return ret;
 }
 
@@ -627,44 +641,21 @@ void EntityInspector::setPropertyFromEntries()
     _keyEntry->SetValue(key);
 	_valEntry->SetValue(val);
 
+	UndoableCommand cmd("entitySetProperty");
+
 	// Pass the call to the specialised routine
 	applyKeyValueToSelection(key, val);
 }
 
 void EntityInspector::applyKeyValueToSelection(const std::string& key, const std::string& val)
 {
-	// greebo: Instantiate a scoped object to make this operation undoable
-	UndoableCommand command("entitySetProperty");
-
-	if (key.empty()) {
-		return;
+	try
+	{
+		selection::algorithm::setEntityKeyvalue(key, val);
 	}
-
-	if (key == "name") {
-		// Check the global namespace if this change is ok
-		IMapRootNodePtr mapRoot = GlobalMapModule().getRoot();
-		if (mapRoot != NULL) {
-			INamespacePtr nspace = mapRoot->getNamespace();
-
-			if (nspace != NULL && nspace->nameExists(val))
-            {
-				// name exists, cancel the change
-				wxutil::Messagebox::ShowError(
-					(boost::format(_("The name %s already exists in this map!")) % val).str());
-				return;
-			}
-		}
-	}
-
-	// Detect classname changes
-    if (key == "classname") {
-		// Classname changes are handled in a special way
-		selection::algorithm::setEntityClassname(val);
-	}
-	else {
-		// Regular key change, use EntityKeySetter to set value on all selected entities
-		EntityKeySetter setter(key, val);
-		GlobalSelectionSystem().foreachSelected(setter);
+	catch (std::runtime_error& ex)
+	{
+		wxutil::Messagebox::ShowError(ex.what());
 	}
 }
 
@@ -715,6 +706,11 @@ void EntityInspector::_onAddKey()
     }
 }
 
+bool EntityInspector::_testAddKey()
+{
+	return !_selectedEntity.expired();
+}
+
 void EntityInspector::_onDeleteKey()
 {
     assert(!_selectedEntity.expired());
@@ -732,6 +728,10 @@ void EntityInspector::_onDeleteKey()
 
 bool EntityInspector::_testDeleteKey()
 {
+	if (_selectedEntity.expired()) return false;
+
+	if (getSelectedKey().empty()) return false;
+
 	// Make sure the Delete item is only available for explicit
 	// (non-inherited) properties
 	if (getListSelectionBool(_columns.isInherited) == false)
@@ -742,6 +742,25 @@ bool EntityInspector::_testDeleteKey()
 
 void EntityInspector::_onCopyKey()
 {
+	wxDataViewItemArray selectedItems;
+	_keyValueTreeView->GetSelections(selectedItems);
+
+	if (selectedItems.Count() == 0) return;
+
+	_clipBoard.clear();
+
+	for (const wxDataViewItem& item : selectedItems)
+	{
+		wxutil::TreeModel::Row row(item, *_kvStore);
+
+		wxDataViewIconText iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
+
+		std::string key = iconAndName.GetText().ToStdString();
+		std::string value = row[_columns.value];
+
+		_clipBoard.push_back(std::make_pair(key, value));
+	}
+#if 0
 	std::string key = getSelectedKey();
     std::string value = getListSelection(_columns.value);
 
@@ -750,15 +769,47 @@ void EntityInspector::_onCopyKey()
 		_clipBoard.key = key;
 		_clipBoard.value = value;
 	}
+#endif
 }
 
 bool EntityInspector::_testCopyKey()
 {
-	return !getSelectedKey().empty();
+	return _keyValueTreeView->HasSelection();
 }
 
 void EntityInspector::_onCutKey()
 {
+	wxDataViewItemArray selectedItems;
+	_keyValueTreeView->GetSelections(selectedItems);
+
+	if (selectedItems.Count() == 0) return;
+
+	assert(!_selectedEntity.expired());
+	Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
+
+	_clipBoard.clear();
+
+	for (const wxDataViewItem& item : selectedItems)
+	{
+		wxutil::TreeModel::Row row(item, *_kvStore);
+
+		wxDataViewIconText iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
+
+		std::string key = iconAndName.GetText().ToStdString();
+		std::string value = row[_columns.value];
+
+		_clipBoard.push_back(std::make_pair(key, value));
+
+		// We don't delete any inherited key values
+		if (row[_columns.isInherited].getBool()) continue;
+
+		UndoableCommand cmd("cutProperty");
+
+		// Clear the key after copying
+		selectedEntity->setKeyValue(key, "");
+	}
+
+#if 0
 	std::string key = getSelectedKey();
     std::string value = getListSelection(_columns.value);
 
@@ -775,10 +826,26 @@ void EntityInspector::_onCutKey()
 		// Clear the key after copying
         selectedEntity->setKeyValue(key, "");
 	}
+#endif
 }
 
 bool EntityInspector::_testCutKey()
 {
+	wxDataViewItemArray selectedItems;
+	_keyValueTreeView->GetSelections(selectedItems);
+
+	for (const wxDataViewItem& item : selectedItems)
+	{
+		wxutil::TreeModel::Row row(item, *_kvStore);
+
+		if (!row[_columns.isInherited].getBool())
+		{
+			return true; // we have at least one non-inherited value
+		}
+	}
+
+	return false;
+#if 0
 	// Make sure the Delete item is only available for explicit
 	// (non-inherited) properties
 	if (getListSelectionBool(_columns.isInherited) == false)
@@ -790,14 +857,21 @@ bool EntityInspector::_testCutKey()
 	{
 		return false;
 	}
+#endif
 }
 
 void EntityInspector::_onPasteKey()
 {
-	if (!_clipBoard.key.empty() && !_clipBoard.value.empty())
+	// greebo: Instantiate a scoped object to make this operation undoable
+	UndoableCommand command("entitySetProperties");
+
+	for (const KeyValuePair& kv : _clipBoard)
 	{
+		// skip empty entries
+		if (kv.first.empty() || kv.second.empty()) continue;
+
 		// Pass the call
-		applyKeyValueToSelection(_clipBoard.key, _clipBoard.value);
+		applyKeyValueToSelection(kv.first, kv.second);
 	}
 }
 
@@ -810,7 +884,7 @@ bool EntityInspector::_testPasteKey()
 	}
 
 	// Return true if the clipboard contains data
-	return !_clipBoard.key.empty() && !_clipBoard.value.empty();
+	return !_clipBoard.empty();
 }
 
 // wxWidget callbacks
@@ -820,6 +894,45 @@ void EntityInspector::_onContextMenu(wxDataViewEvent& ev)
 	_contextMenu->show(_keyValueTreeView);
 }
 
+void EntityInspector::_onDataViewItemChanged(wxDataViewEvent& ev)
+{
+	if (ev.GetDataViewColumn() != nullptr && 
+		static_cast<int>(ev.GetDataViewColumn()->GetModelColumn()) == _columns.booleanValue.getColumnIndex())
+	{
+		// Model value in the boolean column has changed, this means
+		// the user has clicked the checkbox, send the value to the entity/entities
+		wxutil::TreeModel::Row row(ev.GetItem(), *_kvStore);
+
+		wxDataViewIconText iconAndName = static_cast<wxDataViewIconText>(row[_columns.name]);
+
+		std::string key = iconAndName.GetText().ToStdString();
+		bool updatedValue = row[_columns.booleanValue].getBool();
+
+		UndoableCommand cmd("entitySetProperty");
+		applyKeyValueToSelection(key, updatedValue ? "1" : "0");
+		
+		// Check if the property was an inherited one.
+		// The applyKeyValue function produced a non-inherited entry
+		// which should be visible once we're done
+		// Note: selecting the non-inherited property instead of this one
+		// is not as easy as it may appear, since the user is yet to release
+		// the mouse button (we're in the middle of the click event here)
+		// and the MouseUp handler will select this row again
+		if (row[_columns.isInherited].getBool())
+		{
+			_kvStore->ForeachNode([&](wxutil::TreeModel::Row& row)
+			{
+				wxDataViewIconText nameVal = static_cast<wxDataViewIconText>(row[_columns.name]);
+
+				if (nameVal.GetText() == key && !row[_columns.isInherited].getBool())
+				{
+					_keyValueTreeView->EnsureVisible(row.getItem());
+				}
+			});
+		}
+	}
+}
+
 void EntityInspector::_onSetProperty(wxCommandEvent& ev)
 {
 	setPropertyFromEntries();
@@ -889,57 +1002,70 @@ void EntityInspector::updateHelpText(const wxutil::TreeModel::Row& row)
 // and making sure it refers to the currently-selected Entity.
 void EntityInspector::_onTreeViewSelectionChanged(wxDataViewEvent& ev)
 {
-	ev.Skip();
+    ev.Skip();
 
-	// Abort if called without a valid entity selection (may happen during
-	// various cleanup operations).
-	if (_selectedEntity.expired()) return;
+    // Abort if called without a valid entity selection (may happen during
+    // various cleanup operations).
+    if (_selectedEntity.expired()) return;
 
-	wxutil::TreeModel::Row row(ev.GetItem(), *_kvStore);
+    wxDataViewItemArray selectedItems;
+	_keyValueTreeView->GetSelections(selectedItems);
 
-	if (_showHelpColumnCheckbox->IsChecked())
-	{
-		updateHelpText(row);
-	}
+    if (selectedItems.Count() == 1)
+    {
+        wxDataViewItem selectedItem = selectedItems.front();
 
-	// Don't go further without a proper tree selection
-	if (!ev.GetItem().IsOk()) return;
+        wxutil::TreeModel::Row row(selectedItem, *_kvStore);
 
-    // Get the selected key and value in the tree view
-	std::string key = getSelectedKey();
-    std::string value = getListSelection(_columns.value);
+        if (_showHelpColumnCheckbox->IsChecked())
+        {
+            updateHelpText(row);
+        }
 
-    // Get the type for this key if it exists, and the options
-	PropertyParms parms = getPropertyParmsForKey(key);
+        // Don't go further without a proper tree selection
+        if (!selectedItem.IsOk()) return;
 
-    Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
+        // Get the selected key and value in the tree view
+        std::string key = getSelectedKey();
+        std::string value = getListSelection(_columns.value);
 
-    // If the type was not found, also try looking on the entity class
-    if (parms.type.empty())
-	{
-    	IEntityClassConstPtr eclass = selectedEntity->getEntityClass();
-		parms.type = eclass->getAttribute(key).getType();
+        // Get the type for this key if it exists, and the options
+        PropertyParms parms = getPropertyParmsForKey(key);
+
+        Entity* selectedEntity = Node_getEntity(_selectedEntity.lock());
+
+        // If the type was not found, also try looking on the entity class
+        if (parms.type.empty())
+        {
+            IEntityClassConstPtr eclass = selectedEntity->getEntityClass();
+            parms.type = eclass->getAttribute(key).getType();
+        }
+
+        // Construct and add a new PropertyEditor
+        _currentPropertyEditor = PropertyEditorFactory::create(_editorFrame,
+            parms.type, selectedEntity, key, parms.options);
+
+        if (_currentPropertyEditor)
+        {
+            // Don't use wxEXPAND to allow for horizontal centering, just add a 6 pixel border
+            // Using wxALIGN_CENTER_HORIZONTAL will position the property editor's panel in the middle
+            _editorFrame->GetSizer()->Add(_currentPropertyEditor->getWidget(), 1, wxALIGN_CENTER_HORIZONTAL | wxALL, 6);
+            _editorFrame->GetSizer()->Layout();
+        }
+
+        // Update key and value entry boxes, but only if there is a key value. If
+        // there is no selection we do not clear the boxes, to allow keyval copying
+        // between entities.
+        if (!key.empty())
+        {
+            _keyEntry->SetValue(key);
+            _valEntry->SetValue(value);
+        }
     }
-
-    // Construct and add a new PropertyEditor
-    _currentPropertyEditor = PropertyEditorFactory::create(_editorFrame,
-		parms.type, selectedEntity, key, parms.options);
-
-	if (_currentPropertyEditor)
+	else if (selectedItems.Count() > 1)
 	{
-		// Don't use wxEXPAND to allow for horizontal centering, just add a 6 pixel border
-		// Using wxALIGN_CENTER_HORIZONTAL will position the property editor's panel in the middle
-		_editorFrame->GetSizer()->Add(_currentPropertyEditor->getWidget(), 1, wxALIGN_CENTER_HORIZONTAL | wxALL, 6);
-		_editorFrame->GetSizer()->Layout();
-	}
-
-    // Update key and value entry boxes, but only if there is a key value. If
-    // there is no selection we do not clear the boxes, to allow keyval copying
-    // between entities.
-	if (!key.empty())
-	{
-		_keyEntry->SetValue(key);
-		_valEntry->SetValue(value);
+		// When multiple items are selected, clear the property editor
+		_currentPropertyEditor.reset();
 	}
 }
 
@@ -985,6 +1111,17 @@ void EntityInspector::addClassAttribute(const EntityClassAttribute& a)
         row[_columns.name] = wxVariant(wxDataViewIconText(a.getName(), _emptyIcon)); 
         row[_columns.value] = a.getValue();
 
+		// Inherited values have an inactive checkbox, so assign a false value and disable
+		if (a.getType() == "bool")
+		{
+			row[_columns.booleanValue] = a.getValue() == "1";
+		}
+		else
+		{
+			row[_columns.booleanValue] = false;
+			row[_columns.booleanValue].setEnabled(false);
+		}
+
 		row[_columns.name] = grey;
         row[_columns.value] = grey;
 
