@@ -66,7 +66,7 @@ void EmbeddedLayout::activate()
     GlobalGroupDialog().hideDialogWindow();
 
     // Add a new texture browser to the group dialog pages
-    wxWindow* textureBrowser = GlobalTextureBrowser().constructWindow(notebookPanel);
+    wxWindow* textureBrowser = new TextureBrowser(notebookPanel);
 
     // Texture Page
     {
@@ -77,6 +77,7 @@ void EmbeddedLayout::activate()
         page->page = textureBrowser;
         page->tabIcon = "icon_texture.png";
         page->tabLabel = _("Textures");
+		page->position = IGroupDialog::Page::Position::TextureBrowser;
 
         GlobalGroupDialog().addPage(page);
     }
@@ -125,7 +126,6 @@ void EmbeddedLayout::deactivate()
     // Hide the group dialog
     GlobalGroupDialog().hideDialogWindow();
 
-    GlobalTextureBrowser().destroyWindow();
     GlobalGroupDialog().removePage("textures"); // do this after destroyWindow()
 
     // Disconnect before destroying stuff
