@@ -49,7 +49,7 @@ void FloatingLayout::activate()
 	}
 
 	// Add a new texture browser to the group dialog pages
-	wxWindow* textureBrowser = GlobalTextureBrowser().constructWindow(topLevelWindow);
+	wxWindow* textureBrowser = new TextureBrowser(topLevelWindow);
 
 	// Texture Page
 	{
@@ -60,6 +60,7 @@ void FloatingLayout::activate()
 		page->page = textureBrowser;
 		page->tabIcon = "icon_texture.png";
 		page->tabLabel = _("Textures");
+		page->position = IGroupDialog::Page::Position::TextureBrowser;
 
 		GlobalGroupDialog().addPage(page);
 	}
@@ -99,7 +100,6 @@ void FloatingLayout::deactivate()
 	GlobalGroupDialog().hideDialogWindow();
 
 	// Remove the texture browser from the groupdialog
-	GlobalTextureBrowser().destroyWindow();
 	GlobalGroupDialog().removePage("textures");
 
 	// Destroy the camera window
