@@ -2,35 +2,26 @@
 
 #include "i18n.h"
 #include "RadiantModule.h"
-#include "iuimanager.h"
-#include "idialogmanager.h"
 #include "igroupdialog.h"
 #include "ieventmanager.h"
+#include "iuimanager.h"
 #include "ipreferencesystem.h"
-#include "igrid.h"
 #include "ientityinspector.h"
 #include "iorthoview.h"
+#include "iregistry.h"
 
-#include "ui/splash/Splash.h"
-#include "ui/menu/FiltersMenu.h"
 #include "log/Console.h"
 #include "xyview/GlobalXYWnd.h"
 #include "ui/mediabrowser/MediaBrowser.h"
-#include "ui/texturebrowser/TextureBrowser.h"
-#include "ui/layers/LayerControlDialog.h"
-#include "ui/overlay/Overlay.h"
 #include "camera/GlobalCamera.h"
-#include "camera/CameraSettings.h"
-#include "selection/shaderclipboard/ShaderClipboard.h"
 
 #include "registry/registry.h"
-#include "map/AutoSaver.h"
-#include "brush/BrushModule.h"
 #include "wxutil/MultiMonitor.h"
 
 #include "ui/mainframe/ScreenUpdateBlocker.h"
 #include "ui/mainframe/EmbeddedLayout.h"
 #include "ui/mainframe/TopLevelFrame.h"
+#include "map/Map.h"
 
 #include "modulesystem/StaticModule.h"
 #include <functional>
@@ -57,7 +48,7 @@ namespace ui
 
 MainFrame::MainFrame() :
 	_topLevelWindow(NULL),
-	_screenUpdatesEnabled(true)
+	_screenUpdatesEnabled(false) // not enabled until constructed
 {}
 
 // RegisterableModule implementation
@@ -78,8 +69,8 @@ const StringSet& MainFrame::getDependencies() const
 		_dependencies.insert(MODULE_PREFERENCESYSTEM);
 		_dependencies.insert(MODULE_EVENTMANAGER);
 		_dependencies.insert(MODULE_COMMANDSYSTEM);
-		_dependencies.insert(MODULE_UIMANAGER);
 		_dependencies.insert(MODULE_ORTHOVIEWMANAGER);
+		_dependencies.insert(MODULE_CAMERA);
 	}
 
 	return _dependencies;
@@ -90,7 +81,7 @@ void MainFrame::initialiseModule(const ApplicationContext& ctx)
 	rMessage() << "MainFrame::initialiseModule called." << std::endl;
 
 	// Add another page for Multi-Monitor stuff
-	PreferencesPagePtr page = GlobalPreferenceSystem().getPage(_("Settings/Multi Monitor"));
+	IPreferencePage& page = GlobalPreferenceSystem().getPage(_("Settings/Multi Monitor"));
 
 	// Initialise the registry, if no key is set
 	if (GlobalRegistry().get(RKEY_MULTIMON_START_MONITOR).empty())
@@ -109,7 +100,7 @@ void MainFrame::initialiseModule(const ApplicationContext& ctx)
 		);
 	}
 
-	page->appendCombo(_("Start DarkRadiant on monitor"), RKEY_MULTIMON_START_MONITOR, list);
+	page.appendCombo(_("Start DarkRadiant on monitor"), RKEY_MULTIMON_START_MONITOR, list);
 
 	// Add the toggle max/min command for floating windows
 	GlobalCommandSystem().addCommand("ToggleFullScreenCamera",
@@ -128,9 +119,9 @@ void MainFrame::initialiseModule(const ApplicationContext& ctx)
 		if (dwmEnableComposition)
 		{
 			// Add a page for Desktop Composition stuff
-			PreferencesPagePtr page = GlobalPreferenceSystem().getPage(_("Settings/Compatibility"));
+			IPreferencePage& compatPage = GlobalPreferenceSystem().getPage(_("Settings/Compatibility"));
 
-			page->appendCheckBox("", _("Disable Windows Desktop Composition"),
+			compatPage.appendCheckBox(_("Disable Windows Desktop Composition"),
 				RKEY_DISABLE_WIN_DESKTOP_COMP);
 
 			GlobalRegistry().signalForKey(RKEY_DISABLE_WIN_DESKTOP_COMP).connect(
@@ -149,6 +140,8 @@ void MainFrame::initialiseModule(const ApplicationContext& ctx)
 void MainFrame::shutdownModule()
 {
 	rMessage() << "MainFrame::shutdownModule called." << std::endl;
+
+	disableScreenUpdates();
 }
 
 void MainFrame::keyChanged()
@@ -226,6 +219,8 @@ void MainFrame::construct()
 	// register the commands
 	GlobalMainFrameLayoutManager().registerCommands();
 
+	enableScreenUpdates();
+
     updateAllWindows();
 }
 
@@ -251,17 +246,8 @@ void MainFrame::preDestructionCleanup()
         removeLayout();
     }
 
-	// Shutdown the texturebrowser (before the GroupDialog gets shut down).
-	GlobalTextureBrowser().destroyWindow();
-
 	// Broadcast shutdown event to RadiantListeners
 	radiant::getGlobalRadiant()->broadcastShutdownEvent();
-
-	// Destroy the Overlay instance
-	Overlay::destroyInstance();
-
-	// Stop the AutoSaver class from being called
-	map::AutoSaver().stopTimer();
 }
 
 void MainFrame::onTopLevelFrameClose(wxCloseEvent& ev)
@@ -376,17 +362,6 @@ void MainFrame::create()
 	/* Construct the Group Dialog. This is the tabbed window that contains
      * a number of pages - usually Entities, Textures and possibly Console.
      */
-    // Add entity inspector widget
-	IGroupDialog::PagePtr entityInspectorPage(new IGroupDialog::Page);
-
-	entityInspectorPage->name = "entity";
-	entityInspectorPage->windowLabel = _("Entity");
-	entityInspectorPage->page = GlobalEntityInspector().getWidget();
-	entityInspectorPage->tabIcon = "cmenu_add_entity.png";
-	entityInspectorPage->tabLabel = _("Entity");
-
-	GlobalGroupDialog().addPage(entityInspectorPage);
-
 	// Add the Media Browser page
 	IGroupDialog::PagePtr mediaBrowserPage(new IGroupDialog::Page);
 
@@ -395,6 +370,7 @@ void MainFrame::create()
 	mediaBrowserPage->page = MediaBrowser::getInstance().getWidget();
 	mediaBrowserPage->tabIcon = "folder16.png";
 	mediaBrowserPage->tabLabel = _("Media");
+	mediaBrowserPage->position = IGroupDialog::Page::Position::MediaBrowser;
 
 	GlobalGroupDialog().addPage(mediaBrowserPage);
 
@@ -406,21 +382,12 @@ void MainFrame::create()
 	consolePage->page = new Console(getWxTopLevelWindow());
 	consolePage->tabIcon = "iconConsole16.png";
 	consolePage->tabLabel = _("Console");
+	consolePage->position = IGroupDialog::Page::Position::Console;
 
 	GlobalGroupDialog().addPage(consolePage);
 
 	// Load the previous window settings from the registry
 	restoreWindowPosition();
-
-	_topLevelWindow->Show();
-
-	// Start the autosave timer so that it can periodically check the map for changes
-	map::AutoSaver().startTimer();
-
-	// Initialise the shaderclipboard
-	GlobalShaderClipboard().clear();
-
-	LayerControlDialog::init();
 }
 
 void MainFrame::saveWindowPosition()
@@ -450,10 +417,20 @@ void MainFrame::disableScreenUpdates() {
 	_screenUpdatesEnabled = false;
 }
 
-void MainFrame::updateAllWindows()
+void MainFrame::updateAllWindows(bool force)
 {
-	GlobalCamera().update();
-	GlobalXYWndManager().updateAllViews();
+	if (!_screenUpdatesEnabled) return;
+
+    if (force)
+    {
+        GlobalCamera().forceDraw();
+    }
+    else
+    {
+        GlobalCamera().update();
+    }
+
+    GlobalXYWndManager().updateAllViews(force);
 }
 
 void MainFrame::setActiveLayoutName(const std::string& name)
