@@ -1,5 +1,5 @@
 #include "ModelSelector.h"
-#include "ModelFileFunctor.h"
+#include "ModelPopulator.h"
 #include "ModelDataInserter.h"
 
 #include "math/Vector3.h"
@@ -44,15 +44,18 @@ namespace
 ModelSelector::ModelSelector() : 
 	DialogBase(_(MODELSELECTOR_TITLE)),
 	_dialogPanel(loadNamedPanel(this, "ModelSelectorPanel")),
-	_treeStore(new wxutil::TreeModel(_columns)),
-	_treeStoreWithSkins(new wxutil::TreeModel(_columns)),
-	_infoTable(NULL),
-	_materialsList(NULL),
+    _treeStore(new wxutil::TreeModel(_columns)),
+	_treeModelFilter(new wxutil::TreeModelFilter(_treeStore)),
+	_infoTable(nullptr),
+    _materialsList(nullptr),
 	_lastModel(""),
 	_lastSkin(""),
 	_populated(false),
+    _showSkins(true),
 	_showOptions(true)
 {
+    _modelIcon.CopyFromBitmap(wxArtProvider::GetBitmap(GlobalUIManager().ArtIdPrefix() + MODEL_ICON));
+
     // Set the default size of the window
     _position.connect(this);
     _position.readPosition();
@@ -67,9 +70,10 @@ ModelSelector::ModelSelector() :
     _modelPreview->setSize(static_cast<int>(_position.getSize()[0]*0.4f),
                            static_cast<int>(_position.getSize()[1]*0.2f));
 	
-	wxPanel* leftPanel = findNamedObject<wxPanel>(this, "ModelSelectorLeftPanel");
+    wxPanel* leftPanel = findNamedObject<wxPanel>(this, "ModelSelectorLeftPanel");
 
-	// Set up view widgets
+    // Set up view widgets
+    setupTreeView(leftPanel);
 	setupAdvancedPanel(leftPanel);
 
     // Connect buttons
@@ -88,6 +92,11 @@ ModelSelector::ModelSelector() :
 
 	_panedPosition.connect(splitter);
 	_panedPosition.loadFromPath(RKEY_SPLIT_POS);
+
+    Connect(wxutil::EV_TREEMODEL_POPULATION_FINISHED,
+            TreeModelPopulationFinishedHandler(ModelSelector::onTreeStorePopulationFinished), NULL, this);
+    Connect(wxutil::EV_TREEMODEL_POPULATION_PROGRESS,
+            TreeModelPopulationProgressHandler(ModelSelector::onTreeStorePopulationProgress), NULL, this);
 }
 
 void ModelSelector::setupAdvancedPanel(wxWindow* parent)
@@ -104,15 +113,12 @@ void ModelSelector::setupAdvancedPanel(wxWindow* parent)
 		sigc::mem_fun(*_modelPreview, &wxutil::ModelPreview::queueDraw)
     );
 
-	parent->GetSizer()->Prepend(_infoTable, 0, wxEXPAND | wxTOP, 6);
-	parent->GetSizer()->Prepend(_materialsList, 0, wxEXPAND | wxTOP, 6);
+    parent->GetSizer()->Add(_materialsList, 0, wxEXPAND | wxTOP, 6);
+	parent->GetSizer()->Add(_infoTable, 0, wxEXPAND | wxTOP, 6);
 }
 
 void ModelSelector::cancelDialog()
 {
-	_lastModel = "";
-    _lastSkin = "";
-
 	_panedPosition.saveToPath(RKEY_SPLIT_POS);
 
     EndModal(wxID_CANCEL);
@@ -154,50 +160,104 @@ void ModelSelector::onRadiantShutdown()
 
 	// Model references are kept by this class, release them before shutting down
 	_treeView.reset();
-	_treeStore.reset(NULL);
-	_treeStoreWithSkins.reset(NULL);
+    _treeStore.reset(nullptr);
+    _treeModelFilter.reset(nullptr);
 
     // Destroy the window
 	SendDestroyEvent();
     InstancePtr().reset();
 }
 
+void ModelSelector::onTreeStorePopulationProgress(wxutil::TreeModel::PopulationProgressEvent& ev)
+{
+    if (!_progressItem.IsOk()) return;
+
+    if (_populator && !_populator->IsAlive())
+    {
+        return; // we might be in the process of being destructed
+    }
+
+    wxutil::TreeModel::Row row(_progressItem, *_treeStore);
+    row[_columns.filename] = wxVariant(wxDataViewIconText(ev.GetMessage(), _modelIcon));
+    row.SendItemChanged();
+}
+
+void ModelSelector::onTreeStorePopulationFinished(wxutil::TreeModel::PopulationFinishedEvent& ev)
+{
+    // Store the treemodel and create a new filter
+    _treeStore = ev.GetTreeModel();
+    _treeModelFilter.reset(new wxutil::TreeModelFilter(_treeStore));
+
+    // Choose the model based on the "showSkins" setting
+    _treeModelFilter->SetVisibleFunc([this](wxutil::TreeModel::Row& row)->bool
+    {
+        return _showSkins || !row[_columns.isSkin].getBool();
+    });
+
+    _treeView->AssociateModel(_treeModelFilter.get());
+
+    // Trigger resize of the column
+    _treeView->TriggerColumnSizeEvent();
+
+    // Time to highlight the node the dialog as asked to select
+    preSelectModel();
+
+    // Set the flag, we're done
+    _populated = true;
+    _progressItem = wxDataViewItem();
+}
+
+void ModelSelector::preSelectModel()
+{
+    // If an empty string was passed for the current model, use the last selected one
+    std::string previouslySelected = (!_preselectedModel.empty()) ? _preselectedModel : _lastModel;
+
+    if (!previouslySelected.empty())
+    {
+        wxutil::TreeModel* model = static_cast<wxutil::TreeModel*>(_treeView->GetModel());
+
+        // Lookup the model path in the treemodel
+        wxDataViewItem found = model->FindString(previouslySelected, _columns.vfspath);
+
+        if (found.IsOk())
+        {
+            _treeView->Select(found);
+            _treeView->EnsureVisible(found);
+
+            showInfoForSelectedModel();
+        }
+    }
+}
+
 // Show the dialog and enter recursive main loop
 ModelSelectorResult ModelSelector::showAndBlock(const std::string& curModel,
                                                 bool showOptions,
                                                 bool showSkins)
 {
+    _showSkins = showSkins;
+    _preselectedModel = curModel;
+
     if (!_populated)
     {
-        // Attempt to construct the static instance. This could throw an
-        // exception if the population of models is aborted by the user.
-        try
-        {
-            // Populate the tree of models
-            populateModels();
-        }
-        catch (wxutil::ModalProgressDialog::OperationAbortedException&)
-        {
-            // Return a blank model and skin
-            return ModelSelectorResult("", "", false);
-        }
+        // Populate the tree of models
+        populateModels();
     }
 
     // Choose the model based on the "showSkins" setting
-	_treeView->AssociateModel(showSkins ? _treeStoreWithSkins.get() : _treeStore.get());
-
-    // If an empty string was passed for the current model, use the last selected one
-    std::string previouslySelected = (!curModel.empty()) ? curModel : _lastModel;
-
-    if (!previouslySelected.empty())
+    _treeModelFilter->SetVisibleFunc([showSkins, this](wxutil::TreeModel::Row& row)->bool
     {
-		wxutil::TreeModel* model = static_cast<wxutil::TreeModel*>(_treeView->GetModel());
+        return showSkins || !row[_columns.isSkin].getBool();
+    });
 
-		// Lookup the model path in the treemodel
-		wxDataViewItem found = model->FindString(previouslySelected, _columns.vfspath);
-		_treeView->Select(found);
-		_treeView->EnsureVisible(found);
-	}
+#if !defined(__linux__)
+    // Trigger a rebuild of the tree
+    _treeView->Rebuild();
+#endif
+
+    if (_populated)
+    {
+        preSelectModel();
+    }
 
     showInfoForSelectedModel();
 
@@ -207,17 +267,25 @@ ModelSelectorResult ModelSelector::showAndBlock(const std::string& curModel,
 	findNamedObject<wxPanel>(this, "ModelSelectorOptionsPanel")->Show(_showOptions);
 
     // show and enter recursive main loop.
-    ShowModal();
+    int returnCode = ShowModal();
 
 	// Remove the model from the preview's scenegraph before returning
 	_modelPreview->setModel("");
 
-    // Construct the model/skin combo and return it
-    return ModelSelectorResult(
-        _lastModel,
-        _lastSkin,
-        findNamedObject<wxCheckBox>(this, "ModelSelectorMonsterClipOption")->GetValue()
-    );
+    if (returnCode == wxID_OK)
+    {
+        // Construct the model/skin combo and return it
+        return ModelSelectorResult(
+            _lastModel,
+            _lastSkin,
+            findNamedObject<wxCheckBox>(this, "ModelSelectorMonsterClipOption")->GetValue()
+        );
+    }
+    else
+    {
+        // Return empty result on cancel
+        return ModelSelectorResult("", "", false);
+    }
 }
 
 // Static function to display the instance, and return the selected model to the
@@ -230,22 +298,20 @@ ModelSelectorResult ModelSelector::chooseModel(const std::string& curModel,
     return Instance().showAndBlock(curModel, showOptions, showSkins);
 }
 
-void ModelSelector::refresh()
+void ModelSelector::Refresh()
 {
     // Clear the flag, this triggers a new population next time the dialog is shown
     Instance()._populated = false;
 }
 
 // Helper function to create the TreeView
-void ModelSelector::setupTreeView()
+void ModelSelector::setupTreeView(wxWindow* parent)
 {
-    wxPanel* parent = findNamedObject<wxPanel>(this, "ModelSelectorLeftPanel");
-    wxASSERT(parent);
-
-	_treeView = wxutil::TreeView::Create(parent,
-                                         wxBORDER_STATIC | wxDV_NO_HEADER);
+	_treeView = wxutil::TreeView::Create(parent, wxBORDER_STATIC | wxDV_NO_HEADER);
     _treeView->SetMinSize(wxSize(200, 200));
 
+    _treeView->AssociateModel(_treeModelFilter.get());
+
 	// Single visible column, containing the directory/shader name and the icon
 	_treeView->AppendIconTextColumn(
         _("Model Path"), _columns.filename.getColumnIndex(), 
@@ -269,36 +335,24 @@ void ModelSelector::populateModels()
 {
     // Clear the treestore first
     _treeStore->Clear();
-    _treeStoreWithSkins->Clear();
 
-    // Create a VFSTreePopulator for the treestore
-    wxutil::VFSTreePopulator pop(_treeStore);
-    wxutil::VFSTreePopulator popSkins(_treeStoreWithSkins);
+    wxutil::TreeModel::Row row = _treeStore->AddItem();
+    row[_columns.filename] = wxVariant(wxDataViewIconText(_("Loading..."), _modelIcon));
+    row[_columns.isSkin] = false;
+    row[_columns.isFolder] = false;
+    row[_columns.isFolder] = std::string();
 
-    // Use a ModelFileFunctor to add paths to the populator
-    ModelFileFunctor functor(pop, popSkins);
-    GlobalFileSystem().forEachFile(MODELS_FOLDER,
-                                   "*",
-                                   [&](const std::string& filename) { functor(filename); },
-                                   0);
+    row.SendItemAdded();
+    _progressItem = row.getItem();
 
-    // Fill in the column data (TRUE = including skins)
-    ModelDataInserter inserterSkins(_columns, true);
-    popSkins.forEachNode(inserterSkins);
+    // Spawn the population thread
+    _populator.reset(new ModelPopulator(_columns, this));
+    _populator->Run();
+}
 
-    // Insert data into second model (FALSE = without skins)
-    ModelDataInserter inserter(_columns, false);
-    pop.forEachNode(inserter);
-
-	// Sort the models
-	_treeStore->SortModelFoldersFirst(_columns.filename, _columns.isFolder);
-	_treeStoreWithSkins->SortModelFoldersFirst(_columns.filename, _columns.isFolder);
-
-    // Setup the tree view
-    setupTreeView();
-
-    // Set the flag, we're done
-    _populated = true;
+void ModelSelector::Populate()
+{
+    Instance().populateModels();
 }
 
 // Get the value from the selected column
@@ -379,7 +433,7 @@ void ModelSelector::onOK(wxCommandEvent& ev)
 
 	_panedPosition.saveToPath(RKEY_SPLIT_POS);
 
-	EndModal(wxOK); // break main loop
+	EndModal(wxID_OK); // break main loop
 	Hide();
 }
 
