@@ -10,7 +10,7 @@
 #include "wxutil/preview/ModelPreview.h"
 #include "wxutil/WindowPosition.h"
 #include "wxutil/PanedPosition.h"
-#include "wxutil/TreeModel.h"
+#include "wxutil/TreeModelFilter.h"
 #include "wxutil/XmlResourceBasedWidget.h"
 #include "wxutil/KeyValueTable.h"
 
@@ -37,6 +37,8 @@ struct ModelSelectorResult
 	: model(m), skin(s), createClip(clip) {}
 };
 
+class ModelPopulator;
+
 class ModelSelector;
 typedef std::shared_ptr<ModelSelector> ModelSelectorPtr;
 
@@ -54,12 +56,14 @@ public:
 			filename(add(wxutil::TreeModel::Column::IconText)),
 			vfspath(add(wxutil::TreeModel::Column::String)),
 			skin(add(wxutil::TreeModel::Column::String)),
+            isSkin(add(wxutil::TreeModel::Column::Boolean)),
 			isFolder(add(wxutil::TreeModel::Column::Boolean))
 		{}
 
 		wxutil::TreeModel::Column filename;	// e.g. "chair1.lwo"
 		wxutil::TreeModel::Column vfspath;	// e.g. "models/darkmod/props/chair1.lwo"
 		wxutil::TreeModel::Column skin;		// e.g. "chair1_brown_wood", or "" for no skin
+        wxutil::TreeModel::Column isSkin;	// TRUE if this is a skin entry, FALSE if actual model or folder
 		wxutil::TreeModel::Column isFolder;	// whether this is a folder
 	};
 
@@ -71,9 +75,11 @@ private:
 	// Model preview widget
     wxutil::ModelPreviewPtr _modelPreview;
 
-	// Tree store containing model names (one with and one without skins)
+	// Tree store containing model names (including skins)
 	wxutil::TreeModel::Ptr _treeStore;
-	wxutil::TreeModel::Ptr _treeStoreWithSkins;
+
+    // Tree model filter for dynamically excluding the skins
+    wxutil::TreeModelFilter::Ptr _treeModelFilter;
 
     // Main tree view with model hierarchy
 	wxutil::TreeView::Ptr _treeView;
@@ -95,10 +101,18 @@ private:
 
 	// TRUE if the treeview has been populated
 	bool _populated;
+    std::unique_ptr<ModelPopulator> _populator;
+    bool _showSkins;
+
+    // The model to highlight on show
+    std::string _preselectedModel;
 
     // Whether to show advanced options panel
     bool _showOptions;
 
+    wxIcon _modelIcon;
+    wxDataViewItem _progressItem;
+
 private:
 	// Private constructor, creates widgets
 	ModelSelector();
@@ -116,7 +130,8 @@ private:
 
 	// Helper functions to configure GUI components
     void setupAdvancedPanel(wxWindow* parent);
-	void setupTreeView();
+    void setupTreeView(wxWindow* parent);
+    void preSelectModel();
 
 	// Populate the tree view with models
 	void populateModels();
@@ -131,6 +146,8 @@ private:
 	// wx callbacks
 	void onOK(wxCommandEvent& ev);
 	void onCancel(wxCommandEvent& ev);
+    void onTreeStorePopulationProgress(wxutil::TreeModel::PopulationProgressEvent& ev);
+    void onTreeStorePopulationFinished(wxutil::TreeModel::PopulationFinishedEvent& ev);
 
 	// Update the info table with information from the currently-selected model, and
 	// update the displayed model.
@@ -156,7 +173,10 @@ public:
 			const std::string& curModel = "", bool showOptions = true, bool showSkins = true);
 
 	// greebo: Lets the modelselector repopulate its treeview next time the dialog is shown.
-	static void refresh();
+	static void Refresh();
+
+    // Starts the background population thread
+    static void Populate();
 
 	void onRadiantShutdown();
 };
