@@ -5,47 +5,43 @@
 
 #include <string>
 #include <memory>
+#include <sigc++/trackable.h>
 
 namespace ui
 {
 
-class Splash;
-typedef Splash* SplashPtr;
-
 class wxImagePanel;
 
 class Splash :
-	public wxFrame
+	public wxFrame,
+    public sigc::trackable
 {
+#if !defined(__linux__)
 private:
 	wxGauge* _progressBar;
 	wxImagePanel* _imagePanel;
 
-public:
-	// Private constructor, creates all the widgets
-	Splash();
+    // Private constructor, creates all the widgets
+    Splash();
+#endif
 
+public:
 	/** greebo: Sets the text and/or progress of the progress bar.
 	 */
 	void setText(const std::string& text);
 	void setProgress(float fraction);
 	void setProgressAndText(const std::string& text, float fraction);
 
-	// Use this static method to avoid instantiating the class just for this check
-	static bool isVisible();
-	static void destroy();
-
 	static Splash& Instance();
 
+#if !defined(__linux__)
 private:
-	// Accessor method
-	static SplashPtr& InstancePtr();
-
 	void createProgressBar();
 
 	/** greebo: Triggers a redraw of the splash screen
 	 */
 	void queueDraw();
+#endif
 };
 
 } // namespace ui
