@@ -22,6 +22,7 @@
 #include "ui/overlay/Overlay.h"
 #include "ui/texturebrowser/TextureBrowser.h"
 #include "map/RegionManager.h"
+#include "map/Map.h"
 #include "selection/algorithm/General.h"
 #include "selection/algorithm/Primitives.h"
 #include "registry/registry.h"
@@ -66,10 +67,10 @@ namespace
 
 // Constructor
 XYWnd::XYWnd(int id, wxWindow* parent) :
+    MouseToolHandler(IMouseToolGroup::Type::OrthoView),
 	_id(id),
 	_wxGLWidget(new wxutil::GLWidget(parent, std::bind(&XYWnd::onRender, this), "XYWnd")),
     _drawing(false),
-	_deferredDraw(std::bind(&XYWnd::performDeferredDraw, this)),
 	_minWorldCoord(game::current::getValue<float>("/defaults/minWorldCoord")),
 	_maxWorldCoord(game::current::getValue<float>("/defaults/maxWorldCoord")),
 	_defaultCursor(wxCURSOR_DEFAULT),
@@ -120,15 +121,10 @@ XYWnd::XYWnd(int id, wxWindow* parent) :
 
 	_wxGLWidget->Connect(wxEVT_IDLE, wxIdleEventHandler(XYWnd::onIdle), NULL, this);
 
-	_freezePointer.setCallEndMoveOnMouseUp(true);
 	_freezePointer.connectMouseEvents(
 		wxutil::FreezePointer::MouseEventFunction(),
 		std::bind(&XYWnd::onGLMouseButtonRelease, this, std::placeholders::_1));
 
-    GlobalMap().signal_mapValidityChanged().connect(
-        sigc::mem_fun(_deferredDraw, &DeferredDraw::onMapValidChanged)
-    );
-
     updateProjection();
     updateModelview();
 
@@ -166,7 +162,7 @@ void XYWnd::destroyXYView()
     // greebo: Remove <self> as CameraObserver from the CamWindow.
     GlobalCamera().removeCameraObserver(this);
 
-    _wxGLWidget = NULL;
+    _wxGLWidget = nullptr;
 }
 
 void XYWnd::setScale(float f) {
@@ -193,7 +189,7 @@ SelectionTestPtr XYWnd::createSelectionTestForPoint(const Vector2& point)
     float selectEpsilon = registry::getValue<float>(RKEY_SELECT_EPSILON);
 
     // Generate the epsilon
-    DeviceVector deviceEpsilon(selectEpsilon / getWidth(), selectEpsilon / getHeight());
+    Vector2 deviceEpsilon(selectEpsilon / getWidth(), selectEpsilon / getHeight());
 
     // Copy the current view and constrain it to a small rectangle
     render::View scissored(_view);
@@ -217,12 +213,16 @@ int XYWnd::getDeviceHeight() const
     return getHeight();
 }
 
-void XYWnd::captureStates() {
+void XYWnd::captureStates()
+{
     _selectedShader = GlobalRenderSystem().capture("$XY_OVERLAY");
+	_selectedShaderGroup = GlobalRenderSystem().capture("$XY_OVERLAY_GROUP");
 }
 
-void XYWnd::releaseStates() {
-    _selectedShader = ShaderPtr();
+void XYWnd::releaseStates()
+{
+	_selectedShader.reset();
+	_selectedShaderGroup.reset();
 }
 
 const std::string XYWnd::getViewTypeTitle(EViewType viewtype) {
@@ -251,7 +251,7 @@ const std::string XYWnd::getViewTypeStr(EViewType viewtype) {
     return "";
 }
 
-void XYWnd::forceDraw()
+void XYWnd::forceRedraw()
 {
     if (_drawing)
     {
@@ -268,7 +268,7 @@ void XYWnd::queueDraw()
         return; // deny redraw requests if we're currently drawing
     }
 
-    _deferredDraw.draw();
+	_wxGLWidget->Refresh(false);
 }
 
 void XYWnd::onSceneGraphChange() {
@@ -316,8 +316,12 @@ void XYWnd::performChaseMouse()
 
 	handleGLMouseMotion(_chasemouseCurrentX, _chasemouseCurrentY, _eventState, false);
 
-    // greebo: Restart the timer
-    _chaseMouseTimer.Start();
+    // Check if we should still be in chase mouse mode
+    if (_chasingMouse && checkChaseMouse(_eventState))
+    {
+        // greebo: Restart the timer
+        _chaseMouseTimer.Start();
+    }
 }
 
 /* greebo: This handles the "chase mouse" behaviour, if the user drags something
@@ -326,20 +330,19 @@ void XYWnd::performChaseMouse()
  *
  * @returns: true, if the mousechase has been performed, false if no mouse chase was necessary
  */
-bool XYWnd::checkChaseMouse(int x, int y, unsigned int state)
+bool XYWnd::checkChaseMouse(unsigned int state)
 {
-    // Some mouse tools disable chase mouse behaviour
-    if (_activeMouseTool && !_activeMouseTool->allowChaseMouse())
-    {
-        return false;
-    }
+    wxPoint windowMousePos = _wxGLWidget->ScreenToClient(wxGetMousePosition());
+
+    int x = windowMousePos.x;
+    int y = windowMousePos.y;
 
 	_chasemouseDeltaX = 0;
 	_chasemouseDeltaY = 0;
     _eventState = state;
 
 	// greebo: The mouse chase is only active when the corresponding setting is active
-    if (_activeMouseTool && GlobalXYWnd().chaseMouse())
+    if (GlobalXYWnd().chaseMouse())
 	{
         // If the cursor moves close enough to the window borders, chase mouse will kick in
         // The chase mouse delta is capped between 0 and a value that depends on how much
@@ -419,7 +422,7 @@ void XYWnd::setCursorType(CursorType type)
 		_wxGLWidget->SetCursor(_crossHairCursor);
         break;
     };
-	}
+}
 
 // Callback that gets invoked on camera move
 void XYWnd::cameraMoved() {
@@ -460,132 +463,6 @@ EViewType XYWnd::getViewType() const {
     return _viewType;
 }
 
-void XYWnd::clearActiveMouseTool()
-{
-    // Reset the escape listener in any case
-    _escapeListener.reset();
-
-    if (!_activeMouseTool)
-    {
-        return;
-    }
-
-    // Freezing mouse tools: release the mouse cursor again
-    if (_activeMouseTool->getPointerMode() & MouseTool::PointerMode::Capture)
-    {
-        _freezePointer.endCapture();
-    }
-
-    // Tool is done
-    _activeMouseTool.reset();
-
-    // Clear the chase mouse flag just in case
-    _chasingMouse = false;
-
-    // Reset the pointer to default type
-    setCursorType(CursorType::Default);
-}
-
-void XYWnd::handleGLMouseDown(wxMouseEvent& ev)
-{
-    // Context menu handling
-    if (ev.RightDown() && !ev.HasAnyModifiers())
-    {
-        // Remember the RMB coordinates for use in the mouseup event
-        _contextMenu = true;
-        _contextMenu_x = ev.GetX();
-        _contextMenu_y = ev.GetY();
-    }
-
-    MouseToolStack toolStack = GlobalXYWnd().getMouseToolsForEvent(ev);
-
-    // Construct the mousedown event and see if the tool is able to handle it
-    XYMouseToolEvent mouseEvent = createMouseEvent(Vector2(ev.GetX(), ev.GetY()));
-
-    _activeMouseTool = toolStack.handleMouseDownEvent(mouseEvent);
-
-    if (!_activeMouseTool)
-    {
-        return;
-    }
-
-    unsigned int pointerMode = _activeMouseTool->getPointerMode();
-
-    // Check if the mousetool requires pointer freeze
-    if (pointerMode & MouseTool::PointerMode::Capture)
-	{
-        // Enter capture mode, translate the pointermode flags to the various arguments
-        _freezePointer.startCapture(_wxGLWidget, 
-            [&](int x, int y, unsigned int state) { handleGLCapturedMouseMotion(x, y, state); }, // Motion Functor
-            [&]() { clearActiveMouseTool(); }, // End move function, also called when the capture is lost.
-            (pointerMode & MouseTool::PointerMode::Freeze) != 0,
-            (pointerMode & MouseTool::PointerMode::Hidden) != 0,
-            (pointerMode & MouseTool::PointerMode::MotionDeltas) != 0
-        );
-	}
-
-    // Register a hook to capture the ESC key during the active phase
-    _escapeListener.reset(new wxutil::KeyEventFilter(WXK_ESCAPE, [&] ()
-	{
-        _activeMouseTool->onCancel();
-
-        // This also removes the active escape listener
-        clearActiveMouseTool();
-    }));
-}
-
-void XYWnd::handleGLMouseUp(wxMouseEvent& ev)
-	{
-    // Context menu handling
-    if (ev.RightUp() && !ev.HasAnyModifiers() && _contextMenu)
-    {
-        // The user just pressed and released the RMB in the same place
-        onContextMenu();
-	}
-
-    if (_activeMouseTool)
-	{
-        XYMouseToolEvent mouseEvent = createMouseEvent(Vector2(ev.GetX(), ev.GetY()));
-
-        // Ask the active mousetool to handle this event
-        MouseTool::Result result = _activeMouseTool->onMouseUp(mouseEvent);
-
-        if (result == MouseTool::Result::Finished)
-        {
-            clearActiveMouseTool();
-            return;
-}
-    }
-}
-
-void XYWnd::handleActiveMouseToolMotion(int x, int y, bool isDelta)
-{
-    if (!_activeMouseTool) return;
-
-    // New MouseTool event, passing the delta only
-    XYMouseToolEvent ev = isDelta ?
-        createMouseEvent(Vector2(0, 0), Vector2(x, y)) :
-        createMouseEvent(Vector2(x, y));
-
-    // Ask the active mousetool to handle this event
-    switch (_activeMouseTool->onMouseMove(ev))
-    {
-    case MouseTool::Result::Finished:
-        // Tool is done
-        clearActiveMouseTool();
-        forceDraw();
-        return;
-
-    case MouseTool::Result::Activated:
-    case MouseTool::Result::Continued:
-        forceDraw();
-        return;
-
-    case MouseTool::Result::Ignored:
-        break;
-    };
-}
-
 // This gets called by the wx mousemoved callback or the periodical mousechase event
 void XYWnd::handleGLMouseMotion(int x, int y, unsigned int state, bool isDelta)
 {
@@ -613,21 +490,6 @@ void XYWnd::handleGLMouseMotion(int x, int y, unsigned int state, bool isDelta)
         }
     }
 
-    handleActiveMouseToolMotion(x, y, isDelta);
-    
-    // Construct the mousedown event
-    XYMouseToolEvent mouseEvent = createMouseEvent(Vector2(x, y));
-
-    // Send mouse move events to all tools that want them
-    GlobalXYWnd().foreachMouseTool([&] (const MouseToolPtr& tool)
-	{
-        // The active tool already received that event above
-        if (tool != _activeMouseTool && tool->alwaysReceivesMoveEvents())
-		{
-            tool->onMouseMove(mouseEvent);
-		}
-    });
-
     _mousePosition = convertXYToWorld(x, y);
     snapToGrid(_mousePosition);
 
@@ -646,23 +508,24 @@ void XYWnd::handleGLMouseMotion(int x, int y, unsigned int state, bool isDelta)
 	}
 }
 
-void XYWnd::handleGLCapturedMouseMotion(int x, int y, unsigned int mouseState)
+void XYWnd::handleGLCapturedMouseMotion(const MouseToolPtr& tool, int x, int y, unsigned int mouseState)
 {
-    if (!_activeMouseTool) return;
+    if (!tool) return;
 
-    bool mouseToolReceivesDeltas = (_activeMouseTool->getPointerMode() & MouseTool::PointerMode::MotionDeltas) != 0;
-    bool pointerFrozen = (_activeMouseTool->getPointerMode() & MouseTool::PointerMode::Freeze) != 0;
+    bool mouseToolReceivesDeltas = (tool->getPointerMode() & MouseTool::PointerMode::MotionDeltas) != 0;
+    bool pointerFrozen = (tool->getPointerMode() & MouseTool::PointerMode::Freeze) != 0;
 
     // Check if the mouse has reached exceeded the window borders for chase mouse behaviour
-    wxPoint windowMousePos = _wxGLWidget->ScreenToClient(wxGetMousePosition());
-
     // In FreezePointer mode there's no need to check for chase since the cursor is fixed anyway
-    if (!pointerFrozen && checkChaseMouse(windowMousePos.x, windowMousePos.y, mouseState))
+    if (tool->allowChaseMouse() && !pointerFrozen && checkChaseMouse(mouseState))
     {
         // Chase mouse activated, an idle callback will kick in soon
         return;
     }
 
+    // Send mouse move events to the active tool and all inactive tools that want them
+    MouseToolHandler::onGLCapturedMouseMove(x, y, mouseState);
+
     handleGLMouseMotion(x, y, mouseState, mouseToolReceivesDeltas);
 }
 
@@ -1068,9 +931,10 @@ void XYWnd::drawGrid()
     }
 }
 
-void XYWnd::drawBlockGrid() {
-    if (GlobalMap().findWorldspawn() == NULL) {
-        return;
+void XYWnd::drawBlockGrid()
+{
+    if (GlobalMap().getWorldspawn() == NULL) {
+        return; // no worldspawn yet
     }
     // Set a default blocksize of 1024
     int blockSize = GlobalXYWnd().defaultBlockSize();
@@ -1443,7 +1307,7 @@ void XYWnd::draw()
 
     {
         // Construct the renderer and render the scene
-        XYRenderer renderer(flagsMask, _selectedShader.get());
+        XYRenderer renderer(flagsMask, _selectedShader.get(), _selectedShaderGroup.get());
 
         // First pass (scenegraph traversal)
         render::RenderableCollectionWalker::collectRenderablesInScene(renderer,
@@ -1481,8 +1345,6 @@ void XYWnd::draw()
     glDisable(GL_COLOR_MATERIAL);
     GlobalOpenGL().assertNoErrors();
 
-    GlobalOpenGL().assertNoErrors();
-
     // greebo: Check, if the size info should be displayed (if there are any items selected)
     if (xyWndManager.showSizeInfo() && GlobalSelectionSystem().countSelected() != 0)
     {
@@ -1544,7 +1406,7 @@ void XYWnd::draw()
         drawCameraIcon(cam->getCameraOrigin(), cam->getCameraAngles());
     }
 
-    if (_activeMouseTool)
+    if (!_activeMouseTools.empty())
     {
         glMatrixMode(GL_PROJECTION);
         glLoadIdentity();
@@ -1553,9 +1415,12 @@ void XYWnd::draw()
         glMatrixMode(GL_MODELVIEW);
         glLoadIdentity();
 
-        _activeMouseTool->renderOverlay();
+        for (const ActiveMouseTools::value_type& i : _activeMouseTools)
+        {
+            i.second->renderOverlay();
+        }
     }
-
+    
     if (xyWndManager.showOutline()) {
         if (isActive()) {
             glMatrixMode (GL_PROJECTION);
@@ -1588,6 +1453,10 @@ void XYWnd::draw()
 
     GlobalOpenGL().assertNoErrors();
 
+    // Reset the depth mask to its initial value (enabled)
+    glDepthMask(GL_TRUE);
+    GlobalOpenGL().assertNoErrors();
+
     glFinish();
 }
 
@@ -1646,11 +1515,6 @@ void XYWnd::onIdle(wxIdleEvent& ev)
 	}
 }
 
-void XYWnd::performDeferredDraw()
-{
-    _wxGLWidget->Refresh(false);
-}
-
 void XYWnd::onGLResize(wxSizeEvent& ev)
 {
 	const wxSize clientSize = _wxGLWidget->GetClientSize();
@@ -1665,7 +1529,7 @@ void XYWnd::onGLResize(wxSizeEvent& ev)
 
 void XYWnd::onRender()
 {
-	if (GlobalMap().isValid() && GlobalMainFrame().screenUpdatesEnabled())
+	if (GlobalMainFrame().screenUpdatesEnabled())
 	{
         util::ScopedBoolLock drawLock(_drawing);
 
@@ -1695,25 +1559,102 @@ void XYWnd::onGLMouseButtonPress(wxMouseEvent& ev)
 	// Mark this XY view as active
 	GlobalXYWnd().setActiveXY(_id);
 
-	handleGLMouseDown(ev);
+	// Context menu handling
+    if (ev.RightDown() && !ev.HasAnyModifiers())
+    {
+        // Remember the RMB coordinates for use in the mouseup event
+        _contextMenu = true;
+        _contextMenu_x = ev.GetX();
+        _contextMenu_y = ev.GetY();
+    }
+
+    // Send the event to the mouse tool handler
+    MouseToolHandler::onGLMouseButtonPress(ev);
 
 	queueDraw();
 }
 
 void XYWnd::onGLMouseButtonRelease(wxMouseEvent& ev)
 {
-	// Call the according mouseUp method
-	handleGLMouseUp(ev);
+	// Do the context menu handling first
+    if (ev.RightUp() && !ev.HasAnyModifiers() && _contextMenu)
+    {
+        // The user just pressed and released the RMB in the same place
+        onContextMenu();
+	}
+
+    // Regular mouse tool processing
+    MouseToolHandler::onGLMouseButtonRelease(ev);
 
 	queueDraw();
 }
 
 void XYWnd::onGLMouseMove(wxMouseEvent& ev)
 {
+    MouseToolHandler::onGLMouseMove(ev);
+
     handleGLMouseMotion(ev.GetX(), ev.GetY(), wxutil::MouseButton::GetStateForMouseEvent(ev), false);
 }
 
+MouseTool::Result XYWnd::processMouseDownEvent(const MouseToolPtr& tool, const Vector2& point)
+{
+    XYMouseToolEvent ev = createMouseEvent(point);
+    return tool->onMouseDown(ev);
+}
+
+MouseTool::Result XYWnd::processMouseUpEvent(const MouseToolPtr& tool, const Vector2& point)
+{
+    XYMouseToolEvent ev = createMouseEvent(point);
+    return tool->onMouseUp(ev);
+}
+
+MouseTool::Result XYWnd::processMouseMoveEvent(const MouseToolPtr& tool, int x, int y)
+{
+    bool mouseToolReceivesDeltas = (tool->getPointerMode() & MouseTool::PointerMode::MotionDeltas) != 0;
+
+    // New MouseTool event, optionally passing the delta only
+    XYMouseToolEvent ev = mouseToolReceivesDeltas ?
+        createMouseEvent(Vector2(0, 0), Vector2(x, y)) :
+        createMouseEvent(Vector2(x, y));
+
+    return tool->onMouseMove(ev);
+}
+
+void XYWnd::startCapture(const MouseToolPtr& tool)
+{
+    if (_freezePointer.isCapturing(_wxGLWidget))
+    {
+        return;
+    }
+
+    unsigned int pointerMode = tool->getPointerMode();
+
+     _freezePointer.startCapture(_wxGLWidget, 
+        [&, tool](int x, int y, unsigned int state) { handleGLCapturedMouseMotion(tool, x, y, state); }, // Motion Functor
+        [&, tool]() { MouseToolHandler::handleCaptureLost(tool); }, // called when the capture is lost.
+        (pointerMode & MouseTool::PointerMode::Freeze) != 0,
+        (pointerMode & MouseTool::PointerMode::Hidden) != 0,
+        (pointerMode & MouseTool::PointerMode::MotionDeltas) != 0
+    );
+}
+
+void XYWnd::endCapture()
+{
+    if (!_freezePointer.isCapturing(_wxGLWidget))
+    {
+        return;
+    }
+
+    _freezePointer.endCapture();
+}
+
+IInteractiveView& XYWnd::getInteractiveView()
+{
+    return *this;
+}
+
 /* STATICS */
 ShaderPtr XYWnd::_selectedShader;
+ShaderPtr XYWnd::_selectedShaderGroup;
 
 } // namespace 
